SystemOrganization addCategory: #'XML-Parser'!
SystemOrganization addCategory: #'XML-Parser-DTD'!
SystemOrganization addCategory: #'XML-Parser-Exceptions'!
SystemOrganization addCategory: #'XML-Parser-Namespaces'!
SystemOrganization addCategory: #'XML-Parser-Nodes'!

!ByteString methodsFor: '*xml-parser' stamp: 'JAA 3/22/2011 11:43'!
applyLanguageInfomation: aLanguageEnvironment! !

!String methodsFor: '*xml-parser' stamp: 'JAAyer 8/11/2010 22:36'!
applyLanguageInfomation: aLanguageEnvironment
	| leadingChar |

	leadingChar := aLanguageEnvironment leadingChar.
	self withIndexDo: [:each :i |
		each asciiValue > 255
			ifTrue: [
				self
					at: i
					put: (Character leadingChar: leadingChar code: each asUnicode)]]! !

!String methodsFor: '*xml-parser' stamp: 'JAAyer 4/14/2010 21:26'!
asXMLString
	^ XMLString string: self! !

!String methodsFor: '*xml-parser' stamp: 'JAAyer 8/9/2010 14:46'!
splitQualifiedNameInto: aTwoArgumentBlock
	"Split the name into namespace prefix and local name (the block arguments)."
	| i |

	i := self size.
	[i > 1]
		whileTrue: [
			((self at: i) == $:)
				ifTrue: [
					^ aTwoArgumentBlock
						value: (self copyFrom: 1 to: i - 1)
						value: (self copyFrom: i + 1 to: self size)].
			i := i - 1].
	^ aTwoArgumentBlock
		value: ''
		value: self! !

Error subclass: #XMLParserException
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-Exceptions'!

XMLParserException subclass: #XMLDOMException
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-Exceptions'!

XMLParserException subclass: #XMLNamespaceException
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-Exceptions'!

XMLParserException subclass: #XMLParsingException
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-Exceptions'!

XMLParserException subclass: #XMLWellFormednessException
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-Exceptions'!

Warning subclass: #XMLParserWarning
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-Exceptions'!

SequenceableCollection subclass: #XMLOrderedList
	instanceVariableNames: 'collection'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-Nodes'!

!XMLOrderedList commentStamp: '<historical>' prior: 0!
This class functions as an OrderedCollection that will not raise exceptions when attempts are made to access absent objects. This means messages like #at:, #after:, and #first will return nil when the requested object cannot be retrieved. Messages that modify the collection based on the presence or absence of other objects such as #add:after: or #at:put: will still raise exceptions as usual.

!

XMLOrderedList subclass: #XMLObservableList
	instanceVariableNames: 'notifier'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-Nodes'!

!XMLObservableList commentStamp: '<historical>' prior: 0!
This class is an is an ordered list that sends notifications to observers when objects are added to it or removed from it. Subclasses should override the #beforeAdding:, #afterAdding: and #afterRemoving: messages to send custom notifications.!

XMLObservableList subclass: #XMLAttributeList
	instanceVariableNames: 'attributesByName'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-Nodes'!

!XMLAttributeList commentStamp: '<historical>' prior: 0!
This class represents an ordered list of attribute nodes that also keeps a dictionary mapping attribute names to nodes for faster lookups. You usually shouldn't have to use this class directly.!

!XMLAttributeList methodsFor: 'adding' stamp: 'JAAyer 10/15/2010 12:41'!
addName: aName value: aValue
	^ self addLast:
		(XMLAttribute
			name: aName
			value: aValue)! !

!XMLAttributeList methodsFor: 'private' stamp: 'JAAyer 12/6/2010 21:25'!
afterAdding: anObject
	anObject addObserver: self.
	self attributesByName
		at: anObject name ifPresent: [self removeAttributeNodeAt: anObject name];
		at: anObject name put: anObject.

	self notifyOf: #addedAttribute: with: anObject.! !

!XMLAttributeList methodsFor: 'private' stamp: 'JAAyer 11/22/2010 15:42'!
afterRemoving: anAttribute
	anAttribute removeObserver: self.
	self attributesByName removeKey: anAttribute name.
	self notifyOf: #removedAttribute: with: anAttribute.! !

!XMLAttributeList methodsFor: 'converting' stamp: 'JAAyer 12/6/2010 22:55'!
asDictionary
	^ (OrderPreservingDictionary newFrom: self asNameValueAssociations) defaultValue: ''! !

!XMLAttributeList methodsFor: 'converting' stamp: 'JAAyer 6/25/2010 22:15'!
asNameValueAssociations
	^ self collect: [:each | each name -> each value]! !

!XMLAttributeList methodsFor: 'accessing' stamp: 'JAAyer 6/25/2010 22:08'!
attributeNames
	^ self collect: [:each | each name]! !

!XMLAttributeList methodsFor: 'enumerating' stamp: 'JAAyer 6/25/2010 22:14'!
attributeNamesAndValuesDo: aTwoArgumentBlock
	self do: [:each | aTwoArgumentBlock value: each name value: each value]! !

!XMLAttributeList methodsFor: 'enumerating' stamp: 'JAAyer 6/25/2010 22:09'!
attributeNamesDo: aBlock
	self do: [:each | aBlock value: each name]! !

!XMLAttributeList methodsFor: 'accessing' stamp: 'JAAyer 11/26/2010 12:27'!
attributeNodeAt: aName
	^ self attributeNodeAt: aName ifAbsent: [self defaultValue]! !

!XMLAttributeList methodsFor: 'accessing' stamp: 'JAAyer 11/26/2010 12:27'!
attributeNodeAt: aName ifAbsent: aBlock
	^ self attributesByName at: aName ifAbsent: aBlock! !

!XMLAttributeList methodsFor: 'accessing' stamp: 'JAAyer 11/26/2010 12:28'!
attributeNodeAt: aLocalName namespaceURI: aUri
	^ self attributeNodeAt: aLocalName namespaceURI: aUri ifAbsent: [self defaultValue]! !

!XMLAttributeList methodsFor: 'accessing' stamp: 'JAAyer 11/26/2010 12:28'!
attributeNodeAt: aLocalName namespaceURI: aUri ifAbsent: aBlock
	^ self
		detect: [:each | (each isNamed: aLocalName) and: [each namespaceURI = aUri]]
		ifNone: aBlock! !

!XMLAttributeList methodsFor: 'accessing' stamp: 'JAAyer 6/25/2010 22:11'!
attributeValueAt: aName
	^ self attributeValueAt: aName ifAbsent: [self defaultValue]! !

!XMLAttributeList methodsFor: 'accessing' stamp: 'JAAyer 6/25/2010 22:11'!
attributeValueAt: aName ifAbsent: aBlock
	^ (self attributesByName
		at: aName
		ifAbsent: [^ aBlock value]) value! !

!XMLAttributeList methodsFor: 'accessing' stamp: 'JAAyer 7/11/2010 03:34'!
attributeValueAt: aName ifAbsentPut: aBlock
	^ self
		attributeValueAt: aName
		ifAbsent: [self attributeValueAt: aName put: aBlock value]! !

!XMLAttributeList methodsFor: 'accessing' stamp: 'JAAyer 12/6/2010 21:23'!
attributeValueAt: aName put: aValue
	(self includesAttribute: aName)
		ifTrue: [(self attributeNodeAt: aName) value: aValue]
		ifFalse: [self addName: aName value: aValue].
	^ aValue.! !

!XMLAttributeList methodsFor: 'private' stamp: 'JAAyer 6/10/2010 05:51'!
attributesByName
	^ attributesByName ifNil: [attributesByName := Dictionary new]! !

!XMLAttributeList methodsFor: 'private' stamp: 'JAAyer 11/22/2010 15:42'!
beforeAdding: anObject
	anObject element
		ifNotNil: [XMLDOMException signal: 'Cannot add attribute node from other element']! !

!XMLAttributeList methodsFor: 'testing' stamp: 'JAAyer 12/6/2010 21:23'!
includesAttribute: aName
	^ self attributesByName includesKey: aName! !

!XMLAttributeList methodsFor: 'accessing' stamp: 'JAAyer 12/6/2010 21:24'!
removeAttributeNodeAt: aName
	^ self remove: (self attributeNodeAt: aName).! !

!XMLAttributeList methodsFor: 'notifications' stamp: 'JAAyer 6/15/2010 15:13'!
renamedAttribute: anAttribute from: anOldName to: aNewName
	self attributesByName
		removeKey: anOldName;
		at: aNewName put: anAttribute! !

XMLObservableList subclass: #XMLInternalSubsetList
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-Nodes'!

!XMLInternalSubsetList commentStamp: 'JAAyer 7/27/2010 12:45' prior: 0!
This class represents the list of nodes that comprise a document's internal DTD subset.!

!XMLInternalSubsetList methodsFor: 'private' stamp: 'JAAyer 11/22/2010 15:42'!
afterAdding: aNode
	self notifyOf: #addedNode: with: aNode.! !

!XMLInternalSubsetList methodsFor: 'private' stamp: 'JAAyer 11/22/2010 15:42'!
afterRemoving: aNode
	self notifyOf: #removedNode: with: aNode.! !

!XMLInternalSubsetList methodsFor: 'private' stamp: 'JAAyer 11/22/2010 15:42'!
beforeAdding: aNode! !

XMLObservableList subclass: #XMLNodeList
	instanceVariableNames: 'elements elementsByName'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-Nodes'!

!XMLNodeList commentStamp: '<historical>' prior: 0!
This class represents an ordered list of child nodes for some kind of node with elements, probably either an XML document or element object. It treats element nodes specially, keeping both an ordered list of child elements and a dictionary that allows for faster retrieval of element(s) by qualified name or local name only.

Uniquely named elements are stored directly in the dictionary, while multiple elements sharing a common local or qualified name are stored together as an ordered list in the dictionary.!

!XMLNodeList methodsFor: 'private' stamp: 'JAAyer 11/22/2010 18:40'!
afterAdding: aNode
	aNode addObserver: self.
	aNode isElement
		ifTrue: [
			((self last == aNode) or: [(self findElementAfter: aNode) isNil])
				ifTrue: [self elementAddedAtEnd: aNode]
				ifFalse: [self elementInsertedBeforeEnd: aNode].
			self notifyOf: #addedElement: with: aNode].
	self notifyOf: #addedNode: with: aNode.! !

!XMLNodeList methodsFor: 'private' stamp: 'JAAyer 11/22/2010 15:42'!
afterRemoving: aNode
	aNode removeObserver: self.
	aNode isElement
		ifTrue: [
			self elements remove: aNode.
			self elementsByNameAt: aNode name removeElement: aNode.
			aNode hasPrefix
				ifTrue: [self elementsByNameAt: aNode localName removeElement: aNode].
			self notifyOf: #removedElement: with: aNode].
	self notifyOf: #removedNode: with: aNode.! !

!XMLNodeList methodsFor: 'private' stamp: 'JAAyer 11/22/2010 15:42'!
beforeAdding: aNode
	aNode hasParent
		ifTrue: [XMLDOMException signal: 'Cannot add child node that already has a parent']! !

!XMLNodeList methodsFor: 'private' stamp: 'JAAyer 11/22/2010 15:46'!
elementAddedAtEnd: anElement
	self elements add: anElement.
	self
		elementsByNameAt: anElement name
		addElement: anElement.
	anElement hasPrefix
		ifTrue: [
			self
				elementsByNameAt: anElement localName
				addElement: anElement]! !

!XMLNodeList methodsFor: 'accessing' stamp: 'JAAyer 11/26/2010 12:32'!
elementAt: aName
	^ self elementAt: aName ifAbsent: [self defaultValue]! !

!XMLNodeList methodsFor: 'accessing' stamp: 'JAAyer 11/26/2010 12:32'!
elementAt: aName ifAbsent: aBlock
	| elementsWithName |

	elementsWithName := self elementsByName at: aName ifAbsent: [^ aBlock value].

	^ elementsWithName isCollection
		ifTrue: [elementsWithName first]
		ifFalse: [elementsWithName]! !

!XMLNodeList methodsFor: 'accessing' stamp: 'JAAyer 11/26/2010 12:32'!
elementAt: aName namespaceURI: aUri
	^ self elementAt: aName namespaceURI: aUri ifAbsent: [self defaultValue]! !

!XMLNodeList methodsFor: 'accessing' stamp: 'JAAyer 11/26/2010 12:32'!
elementAt: aName namespaceURI: aUri ifAbsent: aBlock
	^ (self elementsAt: aName)
		detect: [:each | each namespaceURI = aUri]
		ifNone: aBlock! !

!XMLNodeList methodsFor: 'private' stamp: 'JAAyer 11/22/2010 19:01'!
elementInsertedBeforeEnd: anElement
	| elementAfter |

	elementAfter := self findElementAfter: anElement.
	self elements add: anElement before: elementAfter.

	self elementsByNameAt: anElement name insertElement: anElement.
	anElement hasPrefix
		ifTrue: [
			self
				elementsByNameAt: anElement localName
				insertElement: anElement]! !

!XMLNodeList methodsFor: 'accessing' stamp: 'JAAyer 5/19/2010 18:25'!
elements
	^ elements ifNil: [elements := self species new]! !

!XMLNodeList methodsFor: 'accessing' stamp: 'JAAyer 11/26/2010 12:32'!
elementsAt: aName
	| elementsWithName |

	elementsWithName :=
		self elementsByName
			at: aName
			ifAbsent: [^ self species new].

	^ elementsWithName isCollection
		ifTrue: [elementsWithName]
		ifFalse: [
			self elementsByName
				at: aName
				put: (self species with: elementsWithName)]! !

!XMLNodeList methodsFor: 'accessing' stamp: 'JAAyer 11/26/2010 12:33'!
elementsAt: aName namespaceURI: aUri
	^ (self elementsAt: aName) select: [:each | each namespaceURI = aUri]! !

!XMLNodeList methodsFor: 'private' stamp: 'JAAyer 6/10/2010 11:07'!
elementsByName
	^ elementsByName ifNil: [elementsByName := Dictionary new].! !

!XMLNodeList methodsFor: 'private' stamp: 'JAAyer 7/28/2010 08:50'!
elementsByNameAt: aName addElement: anElement
	| elementsWithName |

	elementsWithName := 
		self elementsByName
			at: aName
			ifAbsent: [
				self elementsByName at: aName put: anElement.
				^ self].

	elementsWithName isCollection
		ifTrue: [elementsWithName add: anElement]
		ifFalse: [
			self elementsByName
				at: aName
				put: (self species with: elementsWithName with: anElement)]! !

!XMLNodeList methodsFor: 'private' stamp: 'JAAyer 11/22/2010 18:51'!
elementsByNameAt: aName insertElement: anElement
	| elementsWithName elementAfter |

	(elementAfter := self findElementAfter: anElement withName: aName)
		ifNil: [^ self elementsByNameAt: aName addElement: anElement].

	(elementsWithName := self elementsByName at: aName)
		isCollection
			ifTrue: [elementsWithName add: anElement before: elementAfter]
			ifFalse: [
				self elementsByName
					at: aName
					put: (self species with: anElement with: elementAfter)]! !

!XMLNodeList methodsFor: 'private' stamp: 'JAAyer 7/21/2010 12:07'!
elementsByNameAt: aName removeElement: anElement
	| elementsWithName |

	(elementsWithName := self elementsByName at: aName)
		isCollection
			ifTrue: [
				elementsWithName remove: anElement.
				elementsWithName ifEmpty: [self elementsByName removeKey: aName]]
			ifFalse: [self elementsByName removeKey: aName]! !

!XMLNodeList methodsFor: 'enumerating' stamp: 'JAAyer 11/26/2010 12:34'!
elementsCollect: aBlock
	^ self elements collect: aBlock! !

!XMLNodeList methodsFor: 'enumerating' stamp: 'JAAyer 11/26/2010 12:35'!
elementsDetect: aBlock
	^ self elementsDetect: aBlock ifNone: [self defaultValue]! !

!XMLNodeList methodsFor: 'enumerating' stamp: 'JAAyer 11/26/2010 12:35'!
elementsDetect: aBlock ifNone: anExceptionBlock
	^ self elements detect: aBlock ifNone: anExceptionBlock! !

!XMLNodeList methodsFor: 'enumerating' stamp: 'JAAyer 6/26/2010 00:55'!
elementsDo: aBlock
	self elements do: aBlock! !

!XMLNodeList methodsFor: 'enumerating' stamp: 'JAAyer 11/26/2010 12:34'!
elementsSelect: aBlock
	^ self elements select: aBlock! !

!XMLNodeList methodsFor: 'private' stamp: 'JAAyer 11/22/2010 19:00'!
findElementAfter: anElement
	| previousElement |

	previousElement := nil.
	self do: [:each |
		each isElement
			ifTrue: [
				previousElement == anElement
					ifTrue: [^ each].
				previousElement := each]].
	^ nil.! !

!XMLNodeList methodsFor: 'private' stamp: 'JAAyer 11/22/2010 18:59'!
findElementAfter: anElement withName: aName
	| previousElementWithName |

	previousElementWithName := nil.
	self do: [:each |
		(each isElement and: [each isNamed: aName])
			ifTrue: [
				(previousElementWithName == anElement)
					ifTrue: [^ each].
				previousElementWithName := each]].
	^ nil.! !

!XMLNodeList methodsFor: 'testing' stamp: 'JAAyer 7/20/2010 12:20'!
hasElements
	^ elements notNil and: [elements notEmpty]! !

!XMLNodeList methodsFor: 'testing' stamp: 'JAAyer 12/6/2010 21:21'!
includesElementAt: aName
	^ self elementsByName includesKey: aName! !

!XMLNodeList methodsFor: 'notifications' stamp: 'JAAyer 10/19/2010 10:55'!
renamedElement: anElement from: anOldName to: aNewName
	self elementsByNameAt: anOldName removeElement: anElement.
	anOldName splitQualifiedNameInto: [:oldPrefix :oldLocalName |
		oldPrefix ifNotEmpty: [
			self
				elementsByNameAt: oldLocalName
				removeElement: anElement]].

	self elementsByNameAt: anElement name insertElement: anElement.
	anElement hasPrefix
		ifTrue: [
			self
				elementsByNameAt: anElement localName
				insertElement: anElement].! !

!XMLObservableList class methodsFor: 'utility' stamp: 'OttoBehrens 2/14/2011 17:32'!
notifications
	| notifications parser parsedMethods |
	notifications := WriteStream on: (Array new: 10).
	parser := Parser new.
	parsedMethods := IdentityDictionary new.
	(self allMethodsInCategory: #notification)
		do: [ :notifierSelector | 
			| methodNode |
			(SystemNavigation default allSendersOf: notifierSelector inClass: self)
				do: [ :methodReference | 
					methodNode := parsedMethods
						at: methodReference selector
						ifAbsentPut: (parser parse: methodReference sourceCode class: self).
					methodNode
						nodesDo: [ :node | 
							| firstArgument |
							(node isMessage
								and: [ node selector key asSymbol = notifierSelector and: [ (firstArgument := node arguments first) isLiteral ] ])
								ifTrue: [ notifications nextPut: firstArgument key asSymbol ] ] ] ].
	^ notifications contents! !

!XMLObservableList methodsFor: 'adding' stamp: 'JAAyer 8/4/2010 14:46'!
add: anObject after: aPriorObject
	^ self with: [super add: anObject after: aPriorObject] add: anObject! !

!XMLObservableList methodsFor: 'adding' stamp: 'JAAyer 7/22/2010 10:56'!
add: anObject afterIndex: anIndex
	^ self with: [super add: anObject afterIndex: anIndex] add: anObject! !

!XMLObservableList methodsFor: 'adding' stamp: 'JAAyer 8/4/2010 14:46'!
add: anObject before: aSubsequentObject
	^ self with: [super add: anObject before: aSubsequentObject] add: anObject! !

!XMLObservableList methodsFor: 'adding' stamp: 'JAAyer 7/22/2010 10:57'!
add: anObject beforeIndex: anIndex
	^ self with: [super add: anObject beforeIndex: anIndex] add: anObject! !

!XMLObservableList methodsFor: 'adding' stamp: 'JAAyer 7/22/2010 10:57'!
addFirst: anObject
	^ self with: [super addFirst: anObject] add: anObject! !

!XMLObservableList methodsFor: 'adding' stamp: 'JAAyer 7/22/2010 10:57'!
addLast: anObject
	^ self with: [super addLast: anObject] add: anObject! !

!XMLObservableList methodsFor: 'observing' stamp: 'OttoBehrens 2/14/2011 17:32'!
addObserver: anObserver
	self notifier addObserver: anObserver! !

!XMLObservableList methodsFor: 'private' stamp: 'JAAyer 11/23/2010 05:41'!
afterAdding: anObject
	self notifyOf: #added:to: with: anObject with: self.! !

!XMLObservableList methodsFor: 'private' stamp: 'JAAyer 11/22/2010 15:42'!
afterRemoving: anObject
	self notifyOf: #removed:from: with: anObject with: self.! !

!XMLObservableList methodsFor: 'removing' stamp: 'JAA 3/21/2011 20:08'!
at: anIndex put: anObject
	| presentObject |

	(presentObject := self at: anIndex)
		ifNotNil: [
			self
				with: [super at: anIndex put: nil]
				remove: presentObject].

	^ self with: [super at: anIndex put: anObject] add: anObject.! !

!XMLObservableList methodsFor: 'private' stamp: 'JAAyer 11/22/2010 15:42'!
beforeAdding: anObject
	self notifyOf: #adding:to: with: anObject with: self! !

!XMLObservableList methodsFor: 'accessing' stamp: 'JAAyer 6/11/2010 10:50'!
notifier
	^ notifier ifNil: [notifier := XMLNotifier new]! !

!XMLObservableList methodsFor: 'notification' stamp: 'OttoBehrens 2/14/2011 17:32'!
notifyOf: aSelector with: anArgument
	self notifier notifyOf: aSelector with: anArgument! !

!XMLObservableList methodsFor: 'notification' stamp: 'OttoBehrens 2/14/2011 17:32'!
notifyOf: aSelector with: aFirstArgument with: aSecondArgument
	self notifier notifyOf: aSelector with: aFirstArgument with: aSecondArgument! !

!XMLObservableList methodsFor: 'notification' stamp: 'OttoBehrens 2/14/2011 17:32'!
notifyOf: aSelector with: aFirstArgument with: aSecondArgument with: aThirdArgument
	self notifier
		notifyOf: aSelector
		with: aFirstArgument
		with: aSecondArgument
		with: aThirdArgument! !

!XMLObservableList methodsFor: 'notification' stamp: 'OttoBehrens 2/14/2011 17:32'!
notifyOf: aSelector withArguments: anArray
	self notifier notifyOf: aSelector withArguments: anArray! !

!XMLObservableList methodsFor: 'removing' stamp: 'JAAyer 7/22/2010 10:58'!
remove: anObject ifAbsent: aBlock
	^ self with: [super remove: anObject ifAbsent: [^ aBlock value]] remove: anObject! !

!XMLObservableList methodsFor: 'removing' stamp: 'JAAyer 6/11/2010 09:03'!
removeAt: anIndex
	^ self with: [super removeAt: anIndex] remove: (self at: anIndex)! !

!XMLObservableList methodsFor: 'removing' stamp: 'JAAyer 6/11/2010 09:03'!
removeFirst
	^ self removeAt: 1! !

!XMLObservableList methodsFor: 'removing' stamp: 'JAAyer 6/11/2010 09:03'!
removeLast
	^ self removeAt: self size! !

!XMLObservableList methodsFor: 'observing' stamp: 'OttoBehrens 2/14/2011 17:32'!
removeObserver: anObserver
	self notifier removeObserver: anObserver! !

!XMLObservableList methodsFor: 'private' stamp: 'JAAyer 11/22/2010 15:42'!
with: aBlock add: anObject
	self
		beforeAdding: anObject;
		with: [aBlock value] onError: [^ nil];
		afterAdding: anObject.
	^ anObject.! !

!XMLObservableList methodsFor: 'private' stamp: 'JAA 3/26/2011 19:04'!
with: aBlock onError: errorBlock
	^ aBlock
		on: Error
		do: [:error |
			error signal.
			errorBlock valueWithPossibleArgument: error]! !

!XMLObservableList methodsFor: 'private' stamp: 'JAAyer 11/22/2010 15:42'!
with: aBlock remove: anObject
	self
		with: [aBlock value] onError: [^ nil];
		afterRemoving: anObject.
	^ anObject.! !

!XMLOrderedList class methodsFor: 'instance creation' stamp: 'JAAyer 8/4/2010 14:44'!
new
	^ self new: 10! !

!XMLOrderedList class methodsFor: 'instance creation' stamp: 'JAAyer 8/4/2010 16:33'!
new: aSize
	^ self basicNew setCollection: (OrderedCollection new: aSize)! !

!XMLOrderedList class methodsFor: 'instance creation' stamp: 'JAAyer 8/4/2010 14:41'!
new: aSize withAll: anObject
	^ self basicNew setCollection: (OrderedCollection new: aSize withAll: anObject)! !

!XMLOrderedList class methodsFor: 'instance creation' stamp: 'JAAyer 8/4/2010 14:42'!
newFrom: aCollection
	^ self basicNew setCollection: (OrderedCollection newFrom: aCollection)! !

!XMLOrderedList class methodsFor: 'instance creation' stamp: 'JAAyer 8/4/2010 14:43'!
ofSize: aSize
	^ self new: aSize withAll: nil! !

!XMLOrderedList methodsFor: 'adding' stamp: 'JAAyer 8/4/2010 15:01'!
add: anObject
	^ self addLast: anObject! !

!XMLOrderedList methodsFor: 'adding' stamp: 'JAAyer 8/4/2010 13:08'!
add: anObject after: aPriorObject
	^ collection add: anObject after: aPriorObject! !

!XMLOrderedList methodsFor: 'adding' stamp: 'JAAyer 8/4/2010 13:09'!
add: anObject afterIndex: anIndex
	^ collection add: anObject afterIndex: anIndex! !

!XMLOrderedList methodsFor: 'adding' stamp: 'JAAyer 8/4/2010 13:09'!
add: anObject before: aSubsequentObject
	^ collection add: anObject before: aSubsequentObject! !

!XMLOrderedList methodsFor: 'adding' stamp: 'JAAyer 8/4/2010 13:10'!
add: anObject beforeIndex: anIndex
	^ collection add: anObject beforeIndex: anIndex! !

!XMLOrderedList methodsFor: 'adding' stamp: 'JAAyer 8/4/2010 15:02'!
addAll: aCollection
	^ self addAllLast: aCollection! !

!XMLOrderedList methodsFor: 'adding' stamp: 'JAAyer 8/4/2010 15:10'!
addAllFirst: aCollection
	aCollection reverseDo: [:each | self addFirst: each].
	^ aCollection.! !

!XMLOrderedList methodsFor: 'adding' stamp: 'JAAyer 8/4/2010 15:11'!
addAllLast: aCollection
	aCollection do: [:each | self addLast: each].
	^ aCollection.! !

!XMLOrderedList methodsFor: 'adding' stamp: 'JAAyer 8/4/2010 13:11'!
addFirst: anObject
	^ collection addFirst: anObject! !

!XMLOrderedList methodsFor: 'adding' stamp: 'JAAyer 8/4/2010 13:12'!
addLast: anObject
	^ collection addLast: anObject! !

!XMLOrderedList methodsFor: 'accessing' stamp: 'JAAyer 8/4/2010 12:48'!
after: anObject
	^ collection after: anObject ifAbsent: [self defaultValue]! !

!XMLOrderedList methodsFor: 'converting' stamp: 'JAAyer 8/4/2010 12:53'!
asArray
	^ collection asArray! !

!XMLOrderedList methodsFor: 'accessing' stamp: 'JAAyer 8/4/2010 12:49'!
at: anIndex
	^ collection at: anIndex ifAbsent: [self defaultValue]! !

!XMLOrderedList methodsFor: 'adding' stamp: 'JAAyer 8/4/2010 15:12'!
at: anIndex ifAbsentPut: aValue
	^ self
		at: anIndex
		ifAbsent: [self at: anIndex put: aValue]! !

!XMLOrderedList methodsFor: 'accessing' stamp: 'JAA 3/25/2011 14:32'!
at: anIndex put: aValue
	^ collection at: anIndex put: aValue! !

!XMLOrderedList methodsFor: 'accessing' stamp: 'JAAyer 8/5/2010 00:55'!
atLast: anIndexFromEnd
	^ self atLast: anIndexFromEnd ifAbsent: [self defaultValue]! !

!XMLOrderedList methodsFor: 'accessing' stamp: 'JAAyer 8/4/2010 12:50'!
before: anObject
	^ collection before: anObject ifAbsent: [self defaultValue]! !

!XMLOrderedList methodsFor: 'accessing' stamp: 'JAAyer 8/4/2010 13:14'!
capacity
	^ collection capacity! !

!XMLOrderedList methodsFor: 'enumerating' stamp: 'JAAyer 11/28/2010 13:04'!
collect: aBlock
	^ self newWithCollection: (collection collect: aBlock)! !

!XMLOrderedList methodsFor: 'enumerating' stamp: 'JAAyer 11/28/2010 13:04'!
collect: aBlock from: aStartIndex to: anEndIndex
	^ self newWithCollection:
		(collection collect: aBlock from: aStartIndex to: anEndIndex)! !

!XMLOrderedList methodsFor: 'enumerating' stamp: 'JAAyer 11/28/2010 13:04'!
collect: aCollectBlock thenSelect: aSelectBlock
	^ self newWithCollection:
		(collection collect: aCollectBlock thenSelect: aSelectBlock)! !

!XMLOrderedList methodsFor: 'private' stamp: 'JAA 3/25/2011 15:35'!
collection
	^ collection! !

!XMLOrderedList methodsFor: 'copying' stamp: 'JAAyer 8/4/2010 14:54'!
copy
	^ self species basicNew setCollection: collection copy! !

!XMLOrderedList methodsFor: 'copying' stamp: 'JAAyer 3/12/2010 07:30'!
copyEmpty
	^ self species new: self size! !

!XMLOrderedList methodsFor: 'copying' stamp: 'JAAyer 11/28/2010 13:04'!
copyFrom: aStart to: anEnd
	^ self newWithCollection:
		(collection copyFrom: (aStart max: 1) to: (self size min: anEnd))! !

!XMLOrderedList methodsFor: 'copying' stamp: 'JAA 3/25/2011 15:18'!
copyWith: anElement 
	^ self copy
	 	add: anElement;
		yourself! !

!XMLOrderedList methodsFor: 'defaults' stamp: 'JAAyer 3/12/2010 06:48'!
defaultValue
	^ nil! !

!XMLOrderedList methodsFor: 'enumerating' stamp: 'JAAyer 8/4/2010 12:49'!
do: aBlock
	collection do: aBlock! !

!XMLOrderedList methodsFor: 'inspecting' stamp: 'JAAyer 11/23/2010 05:42'!
inspectorClass
	^ OrderedCollectionInspector! !

!XMLOrderedList methodsFor: 'private' stamp: 'JAAyer 11/28/2010 13:04'!
newWithCollection: aCollection
	^ self species basicNew setCollection: aCollection! !

!XMLOrderedList methodsFor: 'removing' stamp: 'JAAyer 8/4/2010 13:33'!
remove: anObject ifAbsent: aBlock
	^ collection remove: anObject ifAbsent: aBlock! !

!XMLOrderedList methodsFor: 'removing' stamp: 'JAAyer 8/4/2010 16:00'!
removeAll
	[self isEmpty] whileFalse: [self removeLast]! !

!XMLOrderedList methodsFor: 'removing' stamp: 'JAAyer 8/4/2010 15:46'!
removeAt: anIndex
	^ collection removeAt: anIndex! !

!XMLOrderedList methodsFor: 'removing' stamp: 'JAAyer 8/4/2010 13:34'!
removeFirst
	^ collection removeFirst! !

!XMLOrderedList methodsFor: 'removing' stamp: 'JAAyer 8/4/2010 13:34'!
removeFirst: aNumber
	^ collection removeFirst: aNumber! !

!XMLOrderedList methodsFor: 'removing' stamp: 'JAAyer 8/4/2010 13:34'!
removeLast
	^ collection removeLast! !

!XMLOrderedList methodsFor: 'removing' stamp: 'JAAyer 8/4/2010 13:34'!
removeLast: aNumber
	^ collection removeLast: aNumber! !

!XMLOrderedList methodsFor: 'accessing' stamp: 'JAAyer 8/4/2010 13:18'!
replaceFrom: aStartingIndex to: anEndingIndex with: aCollection startingAt: aReplacementStart
	collection
		replaceFrom: aStartingIndex
		to: anEndingIndex
		with: aCollection
		startingAt: aReplacementStart! !

!XMLOrderedList methodsFor: 'enumerating' stamp: 'JAAyer 8/4/2010 13:35'!
reverseDo: aBlock
	collection reverseDo: aBlock! !

!XMLOrderedList methodsFor: 'converting' stamp: 'JAA 3/26/2011 18:54'!
reversed
	| reversed |

	reversed := self species new.
	self reverseDo: [:each | reversed addLast: each].
	^ reversed.
! !

!XMLOrderedList methodsFor: 'enumerating' stamp: 'JAAyer 11/28/2010 13:04'!
select: aBlock
	^ self newWithCollection: (collection select: aBlock)! !

!XMLOrderedList methodsFor: 'enumerating' stamp: 'JAAyer 11/28/2010 13:04'!
select: aSelectBlock thenCollect: aCollectBlock
	^ self newWithCollection:
		(collection select: aSelectBlock thenCollect: aCollectBlock)! !

!XMLOrderedList methodsFor: 'private' stamp: 'JAAyer 8/4/2010 13:38'!
setCollection: aCollection
	collection := aCollection! !

!XMLOrderedList methodsFor: 'accessing' stamp: 'JAAyer 8/4/2010 12:49'!
size
	^ collection size! !

!XMLOrderedList methodsFor: 'enumerating' stamp: 'JAAyer 8/4/2010 13:43'!
sort
	collection sort! !

!XMLOrderedList methodsFor: 'enumerating' stamp: 'JAAyer 8/4/2010 13:43'!
sort: aSortBlock
	collection sort: aSortBlock! !

!XMLOrderedList methodsFor: 'private' stamp: 'JAAyer 3/12/2010 11:02'!
species
	^ XMLOrderedList! !

!XMLOrderedList methodsFor: 'private' stamp: 'OttoBehrens 2/16/2011 18:36'!
speciesForCollect
	^ self species! !

!XMLOrderedList methodsFor: 'enumerating' stamp: 'JAAyer 11/28/2010 13:04'!
with: aCollection collect: aBlock
	^ self newWithCollection: (collection with: aCollection collect: aBlock)! !

!XMLOrderedList methodsFor: 'enumerating' stamp: 'JAAyer 11/28/2010 13:04'!
withIndexCollect: aTwoArgumentBlock
	^ self newWithCollection: (collection withIndexCollect: aTwoArgumentBlock)! !

Object subclass: #DTDEntityDeclaration
	instanceVariableNames: 'name value ndata'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-DTD'!

DTDEntityDeclaration class
	instanceVariableNames: 'contextBehavior'!

DTDEntityDeclaration class
	instanceVariableNames: 'contextBehavior'!

!DTDEntityDeclaration class methodsFor: 'accessing' stamp: 'mir 11/16/2000 20:14'!
behaviorForContext: aContext
	^self contextBehavior at: aContext! !

!DTDEntityDeclaration class methodsFor: 'accessing' stamp: 'mir 11/16/2000 20:15'!
contextBehavior
	^contextBehavior! !

!DTDEntityDeclaration class methodsFor: 'class initialization' stamp: 'mir 1/15/2002 18:02'!
initialize
	"DTDEntityDeclaration initialize"

	contextBehavior := Dictionary new.
	contextBehavior
		at: #content put: #include ;
		at: #attributeValueContent put: #includedInLiteral ;
		at: #attributeValue put: #forbidden ;
		at: #entityValue put: #bypass ;
		at: #dtd put: #forbidden ! !

!DTDEntityDeclaration class methodsFor: 'accessing' stamp: 'mir 11/16/2000 20:27'!
leadIn
	^'&'! !

!DTDEntityDeclaration class methodsFor: 'instance creation' stamp: 'mir 11/16/2000 20:13'!
name: aString value: aValueString
	^self new
		name: aString;
		value: aValueString! !

!DTDEntityDeclaration methodsFor: 'behaviors' stamp: 'mir 1/15/2002 11:29'!
bypass
	"Return my reference as is."
	^self reference! !

!DTDEntityDeclaration methodsFor: 'behaviors' stamp: 'mir 1/15/2002 11:29'!
forbidden
	self error: 'Forbidden reference usage'! !

!DTDEntityDeclaration methodsFor: 'behaviors' stamp: 'JAAyer 11/20/2010 14:24'!
include
	"Return my expanded value."
	^ value ifNil: [XMLParserWarning signal: 'XML undefined entity ' , name printString]! !

!DTDEntityDeclaration methodsFor: 'behaviors' stamp: 'mir 1/15/2002 18:06'!
includedInLiteral
	"Return my expanded value."
	^self include! !

!DTDEntityDeclaration methodsFor: 'accessing' stamp: 'mir 1/4/2002 19:40'!
name
	^name! !

!DTDEntityDeclaration methodsFor: 'accessing' stamp: 'JAAyer 3/8/2010 04:52'!
name: aString
	name := aString! !

!DTDEntityDeclaration methodsFor: 'accessing' stamp: 'mir 12/8/2000 17:22'!
ndata
	^ndata! !

!DTDEntityDeclaration methodsFor: 'accessing' stamp: 'mir 12/8/2000 17:22'!
ndata: aString
	ndata := aString! !

!DTDEntityDeclaration methodsFor: 'behaviors' stamp: 'JAAyer 3/11/2010 09:50'!
reference
	"Return my reference as is."
	^self class leadIn, self name, ';'! !

!DTDEntityDeclaration methodsFor: 'invocation' stamp: 'mir 11/16/2000 21:23'!
registerIn: aParser
	aParser entity: self name put: self! !

!DTDEntityDeclaration methodsFor: 'accessing' stamp: 'mir 11/16/2000 10:54'!
value
	^value! !

!DTDEntityDeclaration methodsFor: 'accessing' stamp: 'mir 11/16/2000 10:55'!
value: aString
	value := aString! !

!DTDEntityDeclaration methodsFor: 'invocation' stamp: 'mir 1/15/2002 15:08'!
valueForContext: aContext
	^self perform: (self class behaviorForContext: aContext)! !

DTDEntityDeclaration subclass: #DTDParameterEntityDeclaration
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-DTD'!

!DTDParameterEntityDeclaration class methodsFor: 'class initialization' stamp: 'mir 1/14/2002 18:15'!
initialize
	"DTDParameterEntityDeclaration initialize"

	contextBehavior := Dictionary new.
	contextBehavior
		at: #content put: #notRecognized: ;
		at: #attributeValueContent put: #notRecognized: ;
		at: #attributeValue put: #notRecognized: ;
		at: #entityValue put: #include: ;
		at: #dtd put: #includePE:! !

!DTDParameterEntityDeclaration class methodsFor: 'accessing' stamp: 'mir 11/16/2000 20:27'!
leadIn
	^'%'! !

!DTDParameterEntityDeclaration methodsFor: 'invocation' stamp: 'mir 11/28/2000 17:26'!
registerIn: aParser
	aParser parameterEntity: self name put: self! !

Object subclass: #SAXHandler
	instanceVariableNames: 'input configuration elementNester scopeAllocator'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser'!

!SAXHandler commentStamp: '<historical>' prior: 0!
This class is an XML parser that uses event handling. The acronym "SAX" refers to Java's "Simple API for XML," on which SAXHandler's API is based. To use this class, create a subclass and override handlers under the "content" and "lexical" categories as needed. To instantiate it, send #on: or any of the #parse- messages to the class. The #on: message creates a new parser on the given input string or stream and returns it; the instance can then be further configured and ultimately sent #parseDocument to parse the given input. The class-side #parse- messages take some input source and parse it immediately, returning the result of #parseDocument.

By default, namespace support is enabled.!

!SAXHandler class methodsFor: 'instance creation' stamp: 'JAAyer 11/27/2010 14:06'!
on: aStringOrStream
	^ self new input: aStringOrStream! !

!SAXHandler class methodsFor: 'instance creation' stamp: 'JAAyer 11/13/2010 20:40'!
onFileNamed: aFileName
	^ self on: (FileStream readOnlyFileNamed: aFileName)! !

!SAXHandler class methodsFor: 'instance creation' stamp: 'JAAyer 11/10/2010 05:21'!
parse: aStringOrStream
	^ (self on: aStringOrStream) parseDocument! !

!SAXHandler class methodsFor: 'instance creation' stamp: 'JAAyer 11/10/2010 05:30'!
parse: aStringOrStream usingNamespaces: aBoolean
	^ (self on: aStringOrStream)
		usesNamespaces: aBoolean;
		parseDocument! !

!SAXHandler class methodsFor: 'instance creation' stamp: 'JAAyer 11/10/2010 05:32'!
parseDocumentFrom: aStringOrStream
	^ self parse: aStringOrStream.! !

!SAXHandler class methodsFor: 'instance creation' stamp: 'JAAyer 11/10/2010 05:32'!
parseDocumentFrom: aStringOrStream useNamespaces: aBoolean
	^ self parse: aStringOrStream usingNamespaces: aBoolean.! !

!SAXHandler class methodsFor: 'instance creation' stamp: 'JAAyer 11/13/2010 20:42'!
parseDocumentFromFileNamed: aFileName
	self deprecated: 'use #parseFileNamed: instead'.

	^ self parseFileNamed: aFileName.! !

!SAXHandler class methodsFor: 'instance creation' stamp: 'JAAyer 11/13/2010 20:43'!
parseFileNamed: aFileName
	^ (self onFileNamed: aFileName) parseDocument! !

!SAXHandler class methodsFor: 'instance creation' stamp: 'JAAyer 11/13/2010 20:41'!
parserOnFileNamed: aFileName
	self deprecated: 'use #onFileNamed: instead'.

	^ self onFileNamed: aFileName.! !

!SAXHandler class methodsFor: 'subclass creation' stamp: 'JAA 3/21/2011 18:08'!
parserSubclassNamed: aName category: aCategory
	^ self
		parserSubclassNamed: aName
		implementing: (self methodsInCategory: #content), (self methodsInCategory: #lexical)
		category: aCategory! !

!SAXHandler class methodsFor: 'subclass creation' stamp: 'JAA 3/21/2011 18:29'!
parserSubclassNamed: aName implementing: aSelectorArray category: aCategory
	| newSubclass |

	aSelectorArray do: [:each |
		(self canUnderstand: each asSymbol)
			ifFalse: [self error: 'Cannot create parser subclass implementing ', each asString, '; unrecognized handler']].

	newSubclass :=
		self
			subclass: aName asSymbol
			instanceVariableNames: ''
			classVariableNames: ''
			poolDictionaries: ''
			category: aCategory asString.

	aSelectorArray do: [:each |
		newSubclass
			compile:
				(((self ultimateSourceCodeAt: each asSymbol ifAbsent: []) asString
					copyUpTo: Character cr) copyUpTo: Character lf)
			classified: (self whichCategoryIncludesSelector: each asSymbol)].
	
	^ newSubclass.! !

!SAXHandler methodsFor: 'content' stamp: 'mir 1/8/2002 18:27'!
characters: aString
	"This call corresponds to the Java SAX call
	characters(char[] ch, int start, int length)."! !

!SAXHandler methodsFor: 'lexical' stamp: 'JAAyer 11/10/2010 19:23'!
comment: aCommentString
	"This call corresponds to the Java SAX ext call
	comment(char[] ch, int start, int length)."! !

!SAXHandler methodsFor: 'accessing' stamp: 'JAAyer 11/23/2010 14:37'!
configuration
	^ configuration ifNil: [configuration := self configurationClass new]! !

!SAXHandler methodsFor: 'accessing' stamp: 'JAAyer 11/23/2010 14:13'!
configuration: aConfiguration
	configuration := aConfiguration! !

!SAXHandler methodsFor: 'private' stamp: 'JAAyer 11/23/2010 15:35'!
configurationClass
	^ XMLConfiguration! !

!SAXHandler methodsFor: 'accessing' stamp: 'JAAyer 11/28/2010 11:40'!
currentElement
	^ self elementNester currentElement! !

!SAXHandler methodsFor: 'accessing' stamp: 'JAAyer 11/27/2010 20:36'!
currentScope
	^ self scopeAllocator currentScope! !

!SAXHandler methodsFor: 'accessing' stamp: 'JAAyer 11/27/2010 20:36'!
currentScope: aScope
	self scopeAllocator currentScope: aScope! !

!SAXHandler methodsFor: 'accessing' stamp: 'JAAyer 11/28/2010 11:40'!
elementNester
	^ elementNester ifNil: [elementNester := XMLElementNester new]! !

!SAXHandler methodsFor: 'lexical' stamp: 'JAAyer 2/21/2010 15:13'!
endCData
	"This call corresponds to the Java SAX ext call
	endCData()."! !

!SAXHandler methodsFor: 'content' stamp: 'JAAyer 11/27/2010 06:50'!
endDocument
	"This call corresponds to the Java SAX call
	endDocument()."! !

!SAXHandler methodsFor: 'content' stamp: 'JAAyer 11/10/2010 19:23'!
endElement: aQualifiedName
! !

!SAXHandler methodsFor: 'content' stamp: 'JAAyer 3/24/2010 16:50'!
endElement: aQualifiedName prefix: aPrefix uri: aUri
	"This call corresponds to the Java SAX call
	endElement(java.lang.String namespaceURI, java.lang.String localName, java.lang.String qName).
	By default this call is mapped to the following more convenient call:"

	self endElement: aQualifiedName! !

!SAXHandler methodsFor: 'content' stamp: 'JAAyer 3/24/2010 17:06'!
endElement: aQualifiedName prefix: aPrefix uri: aUri localName: aLocalName
	"This call corresponds to the Java SAX call
	endElement(java.lang.String namespaceURI, java.lang.String localName, java.lang.String qName).
	By default this call is mapped to the following more convenient call:"

	self
		invokeDeprecated: #endElement:namespace:namespaceURI:qualifiedName:
		withArguments: (Array with: aLocalName with: aPrefix with: aUri with: aQualifiedName)
		orForwardTo: #endElement:prefix:uri:
		withArguments: (Array with: aQualifiedName with: aPrefix with: aUri)! !

!SAXHandler methodsFor: 'lexical' stamp: 'JAAyer 11/10/2010 19:23'!
endEntity: anEntityName
	"This call corresponds to the Java SAX ext call
	endEntity(java.lang.String name)."! !

!SAXHandler methodsFor: 'content' stamp: 'JAA 3/22/2011 11:32'!
endPrefixMapping: aPrefix
	"This call corresonds to the Java SAX call
	endPrefixMapping(java.lang.String prefix)."! !

!SAXHandler methodsFor: 'content' stamp: 'mir 8/11/2000 16:25'!
ignorableWhitespace: aString
	"This call corresonds to the Java SAX call
	ignorableWhitespace(char[] ch, int start, int length)."! !

!SAXHandler methodsFor: 'private' stamp: 'JAAyer 11/27/2010 14:05'!
input
	^ input! !

!SAXHandler methodsFor: 'private' stamp: 'JAAyer 11/27/2010 14:05'!
input: aStringOrStream
	input := aStringOrStream.! !

!SAXHandler methodsFor: 'error handling' stamp: 'JAAyer 3/24/2010 17:06'!
invokeDeprecated: aDeprecatedSelector withArguments: anOldArgumentArray orForwardTo: aNewSelector withArguments: aNewArgumentArray
	| parentContext |

	parentContext := thisContext sender.
	(self class includesSelector: aDeprecatedSelector)
		ifFalse: [
			aNewSelector ifNil: [^ self].
			^ self
				perform: aNewSelector
				withArguments: aNewArgumentArray].

	(Deprecation
		method: (self class lookupSelector: aDeprecatedSelector)
		explanation: 'use ', parentContext selector, ' instead'
		on: nil
		in: nil) signal.
	self
		perform: aDeprecatedSelector
		withArguments: anOldArgumentArray.! !

!SAXHandler methodsFor: 'testing' stamp: 'JAAyer 11/23/2010 14:20'!
isValidating
	^ self configuration isValidating! !

!SAXHandler methodsFor: 'accessing' stamp: 'JAAyer 11/23/2010 14:14'!
isValidating: aBoolean
	self configuration isValidating: aBoolean! !

!SAXHandler methodsFor: 'parsing' stamp: 'JAAyer 11/27/2010 13:45'!
parseDocument
	self parser parseDocument.

	"Result of parsing"
	^ self.! !

!SAXHandler methodsFor: 'parsing' stamp: 'JAAyer 11/28/2010 08:51'!
parser
	self configuration parserClass: self class.

	^ XMLParser
		driver: (SAXDriver saxHandler: self)
		on: input.! !

!SAXHandler methodsFor: 'content' stamp: 'JAA 3/22/2011 11:31'!
processingInstruction: aTarget data: aDataString
	"This call corresonds to the Java SAX call
	processingInstruction(java.lang.String target, java.lang.String data)."! !

!SAXHandler methodsFor: 'accessing' stamp: 'JAAyer 11/28/2010 11:40'!
scopeAllocator
	^ scopeAllocator ifNil: [scopeAllocator := XMLNamespaceScopeAllocator new]! !

!SAXHandler methodsFor: 'content' stamp: 'JAAyer 11/10/2010 19:22'!
skippedEntity: anEntityName
	"This call corresonds to the Java SAX call
	skippedEntity(java.lang.String name)."! !

!SAXHandler methodsFor: 'lexical' stamp: 'mir 8/11/2000 18:53'!
startCData
	"This call corresponds to the Java SAX ext call
	startCData()."! !

!SAXHandler methodsFor: 'lexical' stamp: 'JAAyer 11/10/2010 19:21'!
startDTD: aRootElementName publicID: aPublicId systemID: aSystemId
	"This call corresponds to the Java SAX ext call
	startDTD(java.lang.String name, java.lang.String publicId, java.lang.String systemId)."! !

!SAXHandler methodsFor: 'content' stamp: 'mir 8/11/2000 16:45'!
startDocument
	"This call corresonds to the Java SAX call
	startDocument()."! !

!SAXHandler methodsFor: 'content' stamp: 'JAAyer 3/24/2010 17:06'!
startElement: aQualifiedName attributes: aDictionary
	self
		invokeDeprecated: #startElement:attributeList:
		withArguments: (Array with: aQualifiedName with: aDictionary)
		orForwardTo: nil
		withArguments: nil.! !

!SAXHandler methodsFor: 'content' stamp: 'JAAyer 3/24/2010 16:47'!
startElement: aQualifiedName prefix: aPrefix uri: aUri attributes: aDictionary
	self startElement: aQualifiedName attributes: aDictionary! !

!SAXHandler methodsFor: 'content' stamp: 'JAAyer 8/9/2010 17:20'!
startElement: aQualifiedName prefix: aPrefix uri: aUri localName: aLocalName attributes: aDictionary
	"This call corresonds to the Java SAX call
	startElement(java.lang.String namespaceURI, java.lang.String localName,
		java.lang.String qName, Attributes atts)."

	self
		invokeDeprecated: #startElement:namespaceURI:namespace:attributeList:
		withArguments: (Array with: aLocalName with: aUri with: aPrefix with: aDictionary)
		orForwardTo: #startElement:prefix:uri:attributes:
		withArguments: (Array with: aQualifiedName with: aPrefix with: aUri with: aDictionary).! !

!SAXHandler methodsFor: 'lexical' stamp: 'JAAyer 11/10/2010 19:22'!
startEntity: anEntityName
	"This call corresponds to the Java SAX ext call
	startEntity(java.lang.String name)."! !

!SAXHandler methodsFor: 'content' stamp: 'JAAyer 11/10/2010 19:22'!
startPrefixMapping: aPrefix uri: aUri
	"This call corresonds to the Java SAX call
	startPrefixMapping(java.lang.String prefix, java.lang.String uri)."! !

!SAXHandler methodsFor: 'accessing' stamp: 'JAAyer 11/23/2010 14:20'!
useNamespaces: aBoolean
	self deprecated: 'use #usesNamespaces: instead'.

	self usesNamespaces: aBoolean.! !

!SAXHandler methodsFor: 'testing' stamp: 'JAAyer 11/23/2010 14:20'!
usesNamespaces
	^ self configuration usesNamespaces! !

!SAXHandler methodsFor: 'accessing' stamp: 'JAAyer 11/23/2010 14:14'!
usesNamespaces: aBoolean
	self configuration usesNamespaces: aBoolean! !

!SAXHandler methodsFor: 'content' stamp: 'JAAyer 11/13/2010 21:09'!
xmlVersion: aVersionFloat encoding: anEncoding standalone: aBoolean
	self
		invokeDeprecated: #documentAttributes:
		withArguments:
			(Array with: (Dictionary
				with: 'version' -> aVersionFloat asString
				with: 'encoding' -> anEncoding))
		orForwardTo: nil
		withArguments: nil! !

SAXHandler subclass: #XMLDOMParser
	instanceVariableNames: 'document nodeStack isInCDataSection'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser'!

!XMLDOMParser commentStamp: '<historical>' prior: 0!
This class is an XML parser that produces a tree of nodes representing the parsed document's structure. The root of this tree will typically be a document node, and it will have a single child element node as its root. The root element can have any number of additional elements as its children, as well as non-element child nodes. Those elements can themselves have an arbitrary number of children, and so on. To see what type of node objects a node tree created by this parser will contain and for how to manipulate them browse the XML-Parser-Nodes category.

By default, CDATA sections are treated as regular string nodes and comments are ignored entirely. To suppress this behavior, enable #preservesCDataNodes: or #preservesCommentNodes: before parsing. 

Lastly. you can control what node classes the DOM parser uses to create node objects by injecting a custom node factory with #nodeFactory: prior to parsing. See XMLNodeFactory for more information.!

!XMLDOMParser class methodsFor: 'instance creation' stamp: 'JAAyer 11/28/2010 13:25'!
on: aStringOrStream withRootNode: aRootNode
	^ (self on: aStringOrStream)
		pushNode: aRootNode;
		yourself! !

!XMLDOMParser methodsFor: 'content' stamp: 'JAAyer 11/23/2010 14:19'!
characters: aString
	(self preservesCDataNodes and: [self isInCDataSection])
		ifTrue: [self topNode addNode: (self nodeFactory cdataClass string: aString)]
		ifFalse: [self topNode addContent: (self nodeFactory stringClass string: aString)]
! !

!XMLDOMParser methodsFor: 'lexical' stamp: 'JAAyer 11/23/2010 14:43'!
comment: aComment
	self preservesCommentNodes
		ifTrue: [self topNode addNode: (self nodeFactory commentClass string: aComment)]! !

!XMLDOMParser methodsFor: 'private' stamp: 'JAAyer 11/23/2010 14:37'!
configurationClass
	^ XMLDOMConfiguration! !

!XMLDOMParser methodsFor: 'accessing' stamp: 'JAAyer 3/9/2010 01:38'!
document
	^ document! !

!XMLDOMParser methodsFor: 'accessing' stamp: 'JAAyer 11/24/2010 12:31'!
document: aDocument
	(document := aDocument) configuration: self configuration! !

!XMLDOMParser methodsFor: 'lexical' stamp: 'JAAyer 8/9/2010 17:27'!
endCData
	isInCDataSection := false! !

!XMLDOMParser methodsFor: 'content' stamp: 'JAAyer 11/27/2010 06:53'!
endDocument
	self popNode! !

!XMLDOMParser methodsFor: 'content' stamp: 'JAA 3/21/2011 19:45'!
endElement: aQualifiedName prefix: aPrefix uri: aUri localName: aLocalName
	self popNode! !

!XMLDOMParser methodsFor: 'testing' stamp: 'JAAyer 8/9/2010 17:27'!
isInCDataSection
	^ isInCDataSection ifNil: [isInCDataSection := false]! !

!XMLDOMParser methodsFor: 'accessing' stamp: 'JAAyer 11/23/2010 14:18'!
nodeFactory
	^ self configuration nodeFactory! !

!XMLDOMParser methodsFor: 'accessing' stamp: 'JAAyer 11/23/2010 14:18'!
nodeFactory: aNodeFactory
	self configuration nodeFactory: aNodeFactory! !

!XMLDOMParser methodsFor: 'private' stamp: 'JAAyer 3/22/2010 15:28'!
nodeStack
	^ nodeStack ifNil: [nodeStack := OrderedCollection new: 30]! !

!XMLDOMParser methodsFor: 'parsing' stamp: 'JAAyer 3/9/2010 01:30'!
parseDocument
	super parseDocument.
	^ self document.! !

!XMLDOMParser methodsFor: 'private' stamp: 'JAAyer 3/16/2010 09:31'!
popNode
	^ self nodeStack removeLast! !

!XMLDOMParser methodsFor: 'testing' stamp: 'JAAyer 11/23/2010 14:18'!
preservesCDataNodes
	^ self configuration preservesCDataNodes! !

!XMLDOMParser methodsFor: 'accessing' stamp: 'JAAyer 11/23/2010 14:17'!
preservesCDataNodes: aBoolean
	self configuration preservesCDataNodes: aBoolean! !

!XMLDOMParser methodsFor: 'testing' stamp: 'JAAyer 11/23/2010 14:17'!
preservesCDataSections
	self deprecated: 'use #preservesCDataNodes instead'.

	^ self preservesCDataNodes.! !

!XMLDOMParser methodsFor: 'accessing' stamp: 'JAAyer 11/23/2010 14:17'!
preservesCDataSections: aBoolean
	self deprecated: 'use #preservesCDataNodes: instead'.

	self preservesCDataNodes: aBoolean.! !

!XMLDOMParser methodsFor: 'testing' stamp: 'JAAyer 11/23/2010 14:18'!
preservesCommentNodes
	^ self configuration preservesCommentNodes! !

!XMLDOMParser methodsFor: 'accessing' stamp: 'JAAyer 11/23/2010 14:18'!
preservesCommentNodes: aBoolean
	self configuration preservesCommentNodes: aBoolean! !

!XMLDOMParser methodsFor: 'content' stamp: 'JAA 3/22/2011 11:31'!
processingInstruction: aTarget data: aDataString
	self topNode addNode: (self nodeFactory piClass target: aTarget data: aDataString)! !

!XMLDOMParser methodsFor: 'private' stamp: 'JAAyer 11/28/2010 13:25'!
pushElement: anElement
	self nodeStack ifNotEmpty: [self topNode addElement: anElement].
	self pushNode: anElement.! !

!XMLDOMParser methodsFor: 'private' stamp: 'JAAyer 11/28/2010 13:25'!
pushNode: aNode
	^ self nodeStack addLast: aNode
! !

!XMLDOMParser methodsFor: 'lexical' stamp: 'JAAyer 8/9/2010 17:27'!
startCData
	isInCDataSection := true! !

!XMLDOMParser methodsFor: 'content' stamp: 'JAAyer 11/28/2010 13:25'!
startDocument
	self topNode ifNotNil: [^ self].

	self document: self nodeFactory documentClass new.
	self pushNode: self document ! !

!XMLDOMParser methodsFor: 'content' stamp: 'JAAyer 11/10/2010 05:43'!
startElement: aQualifiedName prefix: aPrefix uri: aUri localName: aLocalName attributes: aDictionary
	self pushElement:
		((self nodeFactory
			classForElement: aQualifiedName
			prefix: aPrefix
			uri: aUri
			localName: aLocalName)
				qualifiedName: aQualifiedName
				prefix: aPrefix
				localName: aLocalName
				attributes: aDictionary
				namespaceScope: self currentScope)! !

!XMLDOMParser methodsFor: 'private' stamp: 'JAAyer 3/16/2010 09:29'!
topNode
	^ self nodeStack isEmpty
		ifTrue: [nil]
		ifFalse: [self nodeStack last]! !

!XMLDOMParser methodsFor: 'content' stamp: 'JAAyer 11/27/2010 06:45'!
xmlVersion: aVersionFloat encoding: anEncoding standalone: aBoolean
	self document ifNotNil: [
		self document
			version: aVersionFloat;
			encoding: anEncoding]! !

Object subclass: #XMLConfiguration
	instanceVariableNames: 'isValidating usesNamespaces parserClass'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser'!

!XMLConfiguration commentStamp: '<historical>' prior: 0!
This class stores the shared settings and components for a parser.!

!XMLConfiguration methodsFor: 'accessing' stamp: 'JAAyer 11/23/2010 14:09'!
isValidating
	^ isValidating ifNil: [isValidating := true]! !

!XMLConfiguration methodsFor: 'accessing' stamp: 'JAAyer 11/23/2010 14:05'!
isValidating: aBoolean
	isValidating := aBoolean! !

!XMLConfiguration methodsFor: 'accessing' stamp: 'JAAyer 11/27/2010 06:33'!
parserClass
	^ parserClass ifNil: [parserClass := SAXHandler]! !

!XMLConfiguration methodsFor: 'accessing' stamp: 'JAAyer 11/27/2010 06:37'!
parserClass: aClass
	parserClass := aClass! !

!XMLConfiguration methodsFor: 'accessing' stamp: 'JAAyer 11/23/2010 14:07'!
usesNamespaces
	^ usesNamespaces ifNil: [usesNamespaces := true]! !

!XMLConfiguration methodsFor: 'accessing' stamp: 'JAAyer 11/23/2010 14:06'!
usesNamespaces: aBoolean
	usesNamespaces := aBoolean! !

XMLConfiguration subclass: #XMLDOMConfiguration
	instanceVariableNames: 'preservesCDataNodes preservesCommentNodes nodeFactory'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser'!

!XMLDOMConfiguration commentStamp: '<historical>' prior: 0!
This class stores additional settings and components relevant to the DOM parser and DOM nodes.!

!XMLDOMConfiguration methodsFor: 'accessing' stamp: 'JAAyer 11/23/2010 14:10'!
nodeFactory
	^ nodeFactory ifNil: [nodeFactory := XMLNodeFactory new]! !

!XMLDOMConfiguration methodsFor: 'accessing' stamp: 'JAAyer 11/23/2010 14:10'!
nodeFactory: aFactory
	nodeFactory := aFactory! !

!XMLDOMConfiguration methodsFor: 'accessing' stamp: 'JAAyer 11/27/2010 06:34'!
parserClass
	^ parserClass ifNil: [parserClass := XMLDOMParser]! !

!XMLDOMConfiguration methodsFor: 'accessing' stamp: 'JAAyer 11/23/2010 14:08'!
preservesCDataNodes
	^ preservesCDataNodes ifNil: [preservesCDataNodes := false]! !

!XMLDOMConfiguration methodsFor: 'accessing' stamp: 'JAAyer 11/23/2010 14:08'!
preservesCDataNodes: aBoolean
	preservesCDataNodes := aBoolean! !

!XMLDOMConfiguration methodsFor: 'accessing' stamp: 'JAAyer 11/23/2010 14:08'!
preservesCommentNodes
	^ preservesCommentNodes ifNil: [preservesCommentNodes := false]! !

!XMLDOMConfiguration methodsFor: 'accessing' stamp: 'JAAyer 11/23/2010 14:08'!
preservesCommentNodes: aBoolean
	preservesCommentNodes := aBoolean! !

Object subclass: #XMLElementNester
	instanceVariableNames: 'openElements'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser'!

!XMLElementNester commentStamp: '<historical>' prior: 0!
The class keeps track of elements as they start and end and ensures that they nest properly.!

!XMLElementNester methodsFor: 'accessing' stamp: 'JAAyer 11/27/2010 15:29'!
currentElement
	^ self openElements
		at: self totalOpen
		ifAbsent: ['']! !

!XMLElementNester methodsFor: 'printing' stamp: 'JAAyer 10/15/2010 15:13'!
enclosedElementsAsString: aTagName
	| enclosedTags |

	enclosedTags := String new writeStream.
	self
		printOn: enclosedTags
		startingAt: (self openElements lastIndexOf: aTagName) + 1.
	^ enclosedTags contents.! !

!XMLElementNester methodsFor: 'checking' stamp: 'JAAyer 11/27/2010 15:29'!
endedElement: anElementName
	(self currentElement = anElementName)
		ifFalse: [
			^ (self hasOpenElement: anElementName)
				ifTrue: [self errorImproperlyNestedElements: anElementName]
				ifFalse: [self errorUnmatchedEndTag: anElementName]].

	self openElements removeLast.! !

!XMLElementNester methodsFor: 'checking' stamp: 'JAAyer 11/28/2010 07:03'!
ensureNoElementsRemainOpen
	self hasOpenElements
		ifTrue: [XMLWellFormednessException signal: 'Unterminated elements: ', self printString]! !

!XMLElementNester methodsFor: 'errors' stamp: 'JAAyer 11/28/2010 07:03'!
errorImproperlyNestedElements: anElementName
	XMLWellFormednessException signal:
		('<{1}> element ended before its child {2} element(s)'
			format: (Array with: anElementName with: (self enclosedElementsAsString: anElementName)))! !

!XMLElementNester methodsFor: 'errors' stamp: 'JAAyer 11/28/2010 07:03'!
errorUnmatchedEndTag: anElementName
	XMLWellFormednessException signal:
		('End </{1}> tag with no corresponding start <{1}> tag'
			format: (Array with: anElementName))! !

!XMLElementNester methodsFor: 'testing' stamp: 'JAAyer 10/15/2010 15:16'!
hasOpenElement: anElementName
	^ self openElements includes: anElementName! !

!XMLElementNester methodsFor: 'testing' stamp: 'JAAyer 10/15/2010 15:15'!
hasOpenElements
	^ self openElements notEmpty! !

!XMLElementNester methodsFor: 'accessing' stamp: 'JAAyer 10/15/2010 15:06'!
openElements
	^ openElements ifNil: [openElements := OrderedCollection new: 50]! !

!XMLElementNester methodsFor: 'private' stamp: 'JAAyer 10/16/2010 06:50'!
printLimit
	^ 5! !

!XMLElementNester methodsFor: 'printing' stamp: 'JAAyer 3/16/2010 16:07'!
printOn: aStream
	self printOn: aStream startingAt: 1! !

!XMLElementNester methodsFor: 'printing' stamp: 'JAAyer 10/16/2010 06:50'!
printOn: aStream startingAt: aPosition
	| endPosition |

	endPosition :=  self totalOpen min: (aPosition + self printLimit - 1).
	(self openElements copyFrom: aPosition to: endPosition)
		do: [:each |
			aStream
				nextPut: $<;
				nextPutAll: each;
				nextPut: $>]
		separatedBy: [aStream nextPutAll: ', '].
	(endPosition < self totalOpen)
		ifTrue: [aStream nextPutAll: '...'].! !

!XMLElementNester methodsFor: 'checking' stamp: 'JAAyer 10/15/2010 15:10'!
startedElement: aTagName
	^ self openElements addLast: aTagName! !

!XMLElementNester methodsFor: 'accessing' stamp: 'JAAyer 10/15/2010 15:06'!
totalOpen
	^ self openElements size
	! !

Object subclass: #XMLNamespaceScope
	instanceVariableNames: 'defaultNamespace prefixMappings'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-Namespaces'!

!XMLNamespaceScope commentStamp: '<historical>' prior: 0!
A namespace scope is a possibly empty set of mappings from namespace prefixes to URIs and an optional default namespace URI to be used for unprefixed elements. Scopes can be created from other scopes by sending #enclosingScope: to the class with the other scope as the argument; the new scope will inherit the mappings from the parent.!

!XMLNamespaceScope class methodsFor: 'instance creation' stamp: 'JAAyer 3/5/2010 09:50'!
enclosingScope: aScope
	^ self new inheritMappingsFrom: aScope! !

!XMLNamespaceScope methodsFor: 'accessing' stamp: 'JAAyer 11/24/2010 12:19'!
defaultNamespace
	"Rather than lazily initializing this to an empty string, we leave it as nil, so we can
	distinguish between no default namespace being set and an empty default namespace
	intended to override a non-empty declaration in a parent."

	^ defaultNamespace ifNil: ['']! !

!XMLNamespaceScope methodsFor: 'accessing' stamp: 'JAAyer 11/23/2010 07:41'!
defaultNamespace: aUri
	aUri = self xmlNamespaceURI
		ifTrue: [XMLNamespaceException signal: 'Cannot use "xml" namespace URI as default namespace'].
	aUri = self xmlnsNamespaceURI
		ifTrue: [XMLNamespaceException signal: 'Cannot use "xmlns" namespace URI as default namespace'].

	defaultNamespace := aUri.! !

!XMLNamespaceScope methodsFor: 'private' stamp: 'JAAyer 11/5/2010 22:30'!
ensureAttributePrefix: aPrefix localName: aLocalName hasNoAliasIn: aDictionary
	| namespaceUri aliasingAttributeName |

	namespaceUri := self resolvePrefix: aPrefix.

	(self prefixesAliasing: aPrefix) do: [:aliasingPrefix |
		aliasingAttributeName := aliasingPrefix, ':', aLocalName.
		(aDictionary includesKey: aliasingAttributeName)
			ifTrue: [
				self
					errorAttribute: aliasingAttributeName
					aliases: aPrefix, ':', aLocalName
					with: namespaceUri]].! !

!XMLNamespaceScope methodsFor: 'private' stamp: 'JAAyer 6/20/2010 23:17'!
errorAttribute: anAttribute aliases: anAliasedAttribute with: aUri
	XMLNamespaceException signal:
		('Attribute "{1}" aliases attribute "{2}"; both prefixes map to {3}'
			format: (Array with: anAttribute with: anAliasedAttribute with: aUri))! !

!XMLNamespaceScope methodsFor: 'testing' stamp: 'JAAyer 11/23/2010 07:48'!
hasDefaultNamespace
	^ defaultNamespace notNil! !

!XMLNamespaceScope methodsFor: 'testing' stamp: 'JAAyer 8/5/2010 01:24'!
hasNamespaces
	^ self hasDefaultNamespace or: [self hasPrefixMappings]! !

!XMLNamespaceScope methodsFor: 'testing' stamp: 'JAAyer 7/26/2010 18:17'!
hasPrefixMappings
	^ prefixMappings notNil and: [prefixMappings notEmpty]! !

!XMLNamespaceScope methodsFor: 'testing' stamp: 'JAAyer 11/6/2010 01:43'!
includesPrefix: aPrefix
	^ self prefixMappings includesKey: aPrefix! !

!XMLNamespaceScope methodsFor: 'testing' stamp: 'JAAyer 8/8/2010 14:43'!
includesPrefix: aPrefix mappedTo: aUri
	^ (self includesPrefix: aPrefix)
		and: [(self resolvePrefix: aPrefix) = aUri]! !

!XMLNamespaceScope methodsFor: 'initializing' stamp: 'JAAyer 11/24/2010 12:18'!
inheritMappingsFrom: anEnclosingScope
	anEnclosingScope hasNamespaces
		ifFalse: [^ self].

	(anEnclosingScope defaultNamespace notEmpty and: [self hasDefaultNamespace not])
		ifTrue: [self defaultNamespace: anEnclosingScope defaultNamespace].

	anEnclosingScope hasPrefixMappings
		ifTrue: [
			anEnclosingScope prefixesAndURIsDo: [:prefix :uri |
				(self includesPrefix: prefix)
					ifFalse: [self mapPrefix: prefix to: uri]]].! !

!XMLNamespaceScope methodsFor: 'accessing' stamp: 'JAAyer 11/23/2010 06:59'!
mapPrefix: aPrefix to: aUri
	aPrefix isEmptyOrNil
		ifTrue: [^ self defaultNamespace: aUri].
	aUri isEmptyOrNil
		ifTrue: [XMLNamespaceException signal: 'Cannot clear prefix by mapping it to nil or an empty string.'].

	aPrefix = 'xml'
		ifTrue: [
			aUri = self xmlNamespaceURI
				ifFalse: [
					XMLNamespaceException signal:
						'Cannot rebind the "xml" prefix to a different namespace URI'].
			^ self].

	aPrefix = 'xmlns'
		ifTrue: [XMLNamespaceException signal: 'Cannot bind "xmlns" prefix at all'].

	(aUri = self xmlNamespaceURI or: [aUri = self xmlnsNamespaceURI])
		ifTrue: [XMLNamespaceException signal: 'Cannot alias the "xml" or "xmlns" prefixes'].

	self prefixMappings at: aPrefix put: aUri.! !

!XMLNamespaceScope methodsFor: 'accessing' stamp: 'JAAyer 12/6/2010 22:55'!
prefixMappings
	^ prefixMappings ifNil: [prefixMappings := OrderPreservingDictionary defaultValue: '']! !

!XMLNamespaceScope methodsFor: 'accessing' stamp: 'JAAyer 8/5/2010 01:18'!
prefixes
	^ self prefixMappings keys! !

!XMLNamespaceScope methodsFor: 'accessing' stamp: 'JAAyer 8/9/2010 20:11'!
prefixesAliasing: aPrefix
	"Locate all prefixes that map to the same URI the given prefix does."
	| uri |
	
	uri := self resolvePrefix: aPrefix ifUnmapped: [^ #()].

	^ (self prefixesMappedTo: uri) copyWithout: aPrefix! !

!XMLNamespaceScope methodsFor: 'enumerating' stamp: 'JAAyer 8/5/2010 01:00'!
prefixesAndURIsDo: aTwoArgumentBlock
	self prefixMappings keysAndValuesDo: aTwoArgumentBlock! !

!XMLNamespaceScope methodsFor: 'accessing' stamp: 'JAAyer 8/8/2010 14:43'!
prefixesMappedTo: aUri
	^ self prefixes select: [:each | self includesPrefix: each mappedTo: aUri]! !

!XMLNamespaceScope methodsFor: 'accessing' stamp: 'JAAyer 11/23/2010 07:04'!
resolvePrefix: aPrefix
	^ self resolvePrefix: aPrefix ifUnmapped: ['']! !

!XMLNamespaceScope methodsFor: 'accessing' stamp: 'JAAyer 11/5/2010 22:27'!
resolvePrefix: aPrefix ifUnmapped: aBlock
	aPrefix isEmptyOrNil
		ifTrue: [^ self defaultNamespace].
	aPrefix = 'xml'
		ifTrue: [^ self xmlNamespaceURI].
	aPrefix = 'xmlns'
		ifTrue: [^ self xmlnsNamespaceURI].

	^ self prefixMappings at: aPrefix ifAbsent: aBlock! !

!XMLNamespaceScope methodsFor: 'accessing' stamp: 'JAAyer 11/23/2010 13:36'!
unmapPrefix: aPrefix
	(self includesPrefix: aPrefix)
		ifFalse: [XMLNamespaceException signal: 'Prefix ', aPrefix, ' cannot be unmapped'].

	self prefixMappings removeKey: aPrefix.! !

!XMLNamespaceScope methodsFor: 'private' stamp: 'JAAyer 11/6/2010 02:22'!
validatePrefix: aPrefix of: aMarkupEntityType named: aName
	(aPrefix isEmptyOrNil or: [aPrefix = 'xml'])
		ifTrue: [^ self].

	aPrefix = 'xmlns'
		ifTrue: [
			XMLNamespaceException signal:
				('Cannot use reserved "xmlns" prefix for {1} {2}' format:
					(Array with: aMarkupEntityType with: aName))].

	(self includesPrefix: aPrefix)
		ifFalse: [
			XMLNamespaceException signal:
				('Unmapped prefix "{1}" in {2} {3}' format:
					(Array with: aPrefix with: aMarkupEntityType with: aName))].! !

!XMLNamespaceScope methodsFor: 'validation' stamp: 'JAAyer 11/6/2010 02:00'!
validatePrefix: aPrefix ofAttributeName: anAttributeName
	self validatePrefix: aPrefix of: 'attribute' named: anAttributeName! !

!XMLNamespaceScope methodsFor: 'validation' stamp: 'JAAyer 11/6/2010 02:00'!
validatePrefix: aPrefix ofElementName: anElementName
	self validatePrefix: aPrefix of: 'element' named: anElementName! !

!XMLNamespaceScope methodsFor: 'validation' stamp: 'JAAyer 11/5/2010 22:30'!
validatePrefixedAttributeName: anAttributeName in: aDictionary
	anAttributeName splitQualifiedNameInto: [:prefix :localName |
		prefix ifEmpty: [^ self].

		self
			validatePrefix: prefix
			ofAttributeName: anAttributeName.
		self
			ensureAttributePrefix: prefix
			localName: localName
			hasNoAliasIn: aDictionary]! !

!XMLNamespaceScope methodsFor: 'validation' stamp: 'JAAyer 11/23/2010 07:05'!
validatePrefixedAttributeNames: aDictionary
	aDictionary keysDo: [:each | self validatePrefixedAttributeName: each in: aDictionary]! !

!XMLNamespaceScope methodsFor: 'accessing' stamp: 'JAAyer 11/5/2010 20:25'!
xmlNamespaceURI
	^ 'http://www.w3.org/XML/1998/namespace'! !

!XMLNamespaceScope methodsFor: 'accessing' stamp: 'JAAyer 11/5/2010 20:25'!
xmlnsNamespaceURI
	^ 'http://www.w3.org/2000/xmlns/'! !

Object subclass: #XMLNamespaceScopeAllocator
	instanceVariableNames: 'scopes'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-Namespaces'!

!XMLNamespaceScopeAllocator commentStamp: '<historical>' prior: 0!
This class handles the allocation of namespace scopes using a stack. Sending #enterScope to an instance will cause it to push a new scope onto the stack that inherits all mappings from the previous, enclosing scope and makes it the current scope. Sending #exitScop causes that scope to be removed and the parent scope to again be the current scope.!

!XMLNamespaceScopeAllocator methodsFor: 'accessing' stamp: 'JAAyer 7/27/2010 14:27'!
currentScope
	^ self scopes
		at: self scopes size
		ifAbsent: [self emptyScope]! !

!XMLNamespaceScopeAllocator methodsFor: 'accessing' stamp: 'JAAyer 11/28/2010 11:57'!
currentScope: aScope
	^ self pushScope: aScope! !

!XMLNamespaceScopeAllocator methodsFor: 'scope' stamp: 'JAAyer 8/8/2010 14:45'!
declarePrefix: aPrefix uri: aUri
	"Declare the given name space prefix with the given URL"

	self currentScope mapPrefix: aPrefix to: aUri! !

!XMLNamespaceScopeAllocator methodsFor: 'accessing' stamp: 'JAAyer 3/3/2010 22:37'!
defaultNamespace
	^ self currentScope defaultNamespace! !

!XMLNamespaceScopeAllocator methodsFor: 'private' stamp: 'JAAyer 3/13/2010 13:44'!
emptyScope
	^ XMLNamespaceScope new! !

!XMLNamespaceScopeAllocator methodsFor: 'accessing' stamp: 'JAAyer 3/2/2010 22:14'!
enclosingScope
	^ self scopes
		at: self scopes size - 1
		ifAbsent: [self emptyScope]! !

!XMLNamespaceScopeAllocator methodsFor: 'scope' stamp: 'JAAyer 3/13/2010 13:42'!
enterScope
	self pushScope: (XMLNamespaceScope enclosingScope: self currentScope)! !

!XMLNamespaceScopeAllocator methodsFor: 'scope' stamp: 'JAAyer 7/27/2010 14:31'!
exitScope
	self popScope! !

!XMLNamespaceScopeAllocator methodsFor: 'testing' stamp: 'JAAyer 8/6/2010 12:07'!
includesPrefix: aPrefix
	^ self currentScope includesPrefix: aPrefix! !

!XMLNamespaceScopeAllocator methodsFor: 'testing' stamp: 'JAAyer 8/8/2010 14:43'!
includesPrefix: aPrefix mappedTo: aUri
	^ self currentScope includesPrefix: aPrefix mappedTo: aUri! !

!XMLNamespaceScopeAllocator methodsFor: 'accessing' stamp: 'JAAyer 3/4/2010 00:12'!
popScope
	self scopes removeLast! !

!XMLNamespaceScopeAllocator methodsFor: 'accessing' stamp: 'JAAyer 11/24/2010 08:39'!
pushScope: aScope
	self scopes addLast: aScope! !

!XMLNamespaceScopeAllocator methodsFor: 'accessing' stamp: 'JAAyer 3/5/2010 06:00'!
resolvePrefix: aPrefix
	^ self currentScope resolvePrefix: aPrefix! !

!XMLNamespaceScopeAllocator methodsFor: 'accessing' stamp: 'JAAyer 3/5/2010 09:50'!
resolvePrefix: aPrefix ifUnmapped: aBlock
	^ self currentScope resolvePrefix: aPrefix ifUnmapped: aBlock! !

!XMLNamespaceScopeAllocator methodsFor: 'accessing' stamp: 'JAAyer 3/2/2010 21:35'!
scopes
	^ scopes ifNil: [scopes := OrderedCollection new: 20]! !

!XMLNamespaceScopeAllocator methodsFor: 'validation' stamp: 'JAAyer 8/7/2010 13:42'!
validatePrefix: aPrefix ofElementName: anElementName
	self currentScope
		validatePrefix: aPrefix
		ofElementName: anElementName! !

!XMLNamespaceScopeAllocator methodsFor: 'validation' stamp: 'JAAyer 8/6/2010 23:14'!
validatePrefixedAttributeNames: aDictionary
	self currentScope validatePrefixedAttributeNames: aDictionary.! !

Object subclass: #XMLNestedStreamReader
	instanceVariableNames: 'stream nestedStreams hasNestedStreams peekChar writeBuffer normalizedLineEndingChar currentLineNumber'
	classVariableNames: 'CarriageReturn HexDigitCharacterMap LineFeed SeparatorCharacters'
	poolDictionaries: ''
	category: 'XML-Parser'!

!XMLNestedStreamReader commentStamp: '<historical>' prior: 0!
This class represents a stream reader that implements pushBack: using nested streams. This enables subsitution/replacement to be performed without modifying the underlying collections streamed-over or having to copy them. It also performs line-ending normalization, transforming CR and CRLF sequences into a single LF character.

(The code in this class is optimized; refactor with care.)!

!XMLNestedStreamReader class methodsFor: 'initialization' stamp: 'JAA 3/23/2011 10:40'!
initialize
	"self initialize"

	SeparatorCharacters := CharacterSet newFrom: Character separators.

	CarriageReturn := Character cr.
	LineFeed := Character lf.

	HexDigitCharacterMap := Dictionary new.
	($0 to: $9) do: [:each | HexDigitCharacterMap at: each put: each asciiValue - $0 asciiValue].
	($a to: $f) do: [:each | | digitValue |
		digitValue := each asciiValue - $a asciiValue + 10.
		HexDigitCharacterMap
			at: each put: digitValue;
			at: each asUppercase put: digitValue].! !

!XMLNestedStreamReader class methodsFor: 'instance creation' stamp: 'JAAyer 3/11/2010 09:34'!
on: anInitialStream
	^ self new stream: anInitialStream! !

!XMLNestedStreamReader methodsFor: 'testing' stamp: 'JAAyer 12/14/2010 02:23'!
atEnd
	^ peekChar isNil and: [hasNestedStreams not and: [stream atEnd]]! !

!XMLNestedStreamReader methodsFor: 'accessing' stamp: 'JAAyer 12/14/2010 02:25'!
basicNext
	"Returns next character in the stream after performing line-ending normalization.
	Normalization does not occur across nested streams."

	| nextChar |

	(nextChar := stream next)
		ifNil: [
			hasNestedStreams ifFalse: [^nil].
			self popCurrentStream.
			[hasNestedStreams and: [stream atEnd]]
				whileTrue: [self popCurrentStream].
			nextChar := stream next].

	nextChar == CarriageReturn
		ifTrue: [
			nextChar := normalizedLineEndingChar.
			(stream peek == LineFeed)
				ifTrue: [stream next]]
		ifFalse: [
			(nextChar == LineFeed)
				ifTrue: [nextChar := normalizedLineEndingChar]].

	^ nextChar.! !

!XMLNestedStreamReader methodsFor: 'accessing' stamp: 'JAAyer 7/14/2010 00:39'!
currentLineNumber
	^ currentLineNumber! !

!XMLNestedStreamReader methodsFor: 'initialization' stamp: 'JAAyer 12/14/2010 02:25'!
initialize
	stream := nil.
	nestedStreams := nil.
	hasNestedStreams := false.
	peekChar := nil.
	writeBuffer := WriteStream on: (String new: 128).
	currentLineNumber := 1.
	normalizedLineEndingChar := LineFeed.! !

!XMLNestedStreamReader methodsFor: 'private' stamp: 'JAA 3/23/2011 10:54'!
maxIntegerValue
	"SmallInteger maxValue"
	^ 1073741823! !

!XMLNestedStreamReader methodsFor: 'accessing' stamp: 'JAAyer 12/14/2010 02:44'!
next
	"Return the next character from the current input stream. If the current
	stream is at end, pop to the next nesting level if there is one."

	| nextChar |

	peekChar
		ifNil: [nextChar := self basicNext]
		ifNotNil: [
			nextChar := peekChar.
			peekChar := nil].

	nextChar == normalizedLineEndingChar
		ifTrue: [currentLineNumber := currentLineNumber + 1].

	^ nextChar.! !

!XMLNestedStreamReader methodsFor: 'accessing' stamp: 'JAA 3/23/2011 10:48'!
nextDigitWithBase: aBase
	| digit |

	aBase = 16
		ifTrue: [digit := HexDigitCharacterMap at: self peek ifAbsent: [nil]]
		ifFalse: [
			digit := self peek asciiValue - $0 asciiValue.
			(digit >= 0 and: [digit <= 10])
				ifFalse: [digit := nil]].
	digit ifNotNil: [self next].

	^ digit.! !

!XMLNestedStreamReader methodsFor: 'accessing' stamp: 'JAA 3/23/2011 11:07'!
nextInteger
	"We re-implement integer reading here because the impelmentation of Integer>>readFrom: expects
	the underlying stream to be skip:able."
	| base |

	self peek == $x
		ifTrue: [
			self next.
			base := 16]
		ifFalse: [base := 10].

	^ self nextIntegerWithBase: base.! !

!XMLNestedStreamReader methodsFor: 'accessing' stamp: 'JAA 3/23/2011 11:14'!
nextIntegerWithBase: aBase
	| integer maxValue maxBaseValue digitValue |

	integer := 0.
	maxValue := self maxIntegerValue.
	maxBaseValue := (maxValue / aBase) asInteger.
	[self atEnd not
		and: [(digitValue := self nextDigitWithBase: aBase) notNil]]
		whileTrue: [
			integer > maxBaseValue
				ifTrue: [^ integer].
			integer := integer * aBase.

			integer > (maxValue - digitValue)
				ifTrue: [^ integer].
			integer := integer + digitValue].
	^ integer.! !

!XMLNestedStreamReader methodsFor: 'testing' stamp: 'JAAyer 12/6/2010 21:40'!
nextMatchAll: aString
	| i nextChar pushBackString |

	self peek == (aString at: 1)
		ifFalse: [^ false].

	"we unrolled the first iteration with the above #peek"
	i := 2.
	self next.
	[(i <= aString size)
		and: [(aString at: i) == (nextChar := self next)]]
		whileTrue: [i := i + 1].
	(i > aString size)
		ifTrue: [^ true].

	pushBackString := aString copyFrom: 1 to: i - 1.
	self pushBack:
		(nextChar
			ifNotNil: [pushBackString copyWith: nextChar]
			ifNil: [pushBackString]).
	^ false.! !

!XMLNestedStreamReader methodsFor: 'accessing' stamp: 'JAA 3/22/2011 11:49'!
nextWhitespace
	| nextChar |

	writeBuffer reset.
	[(nextChar := self peek) notNil
		and: [SeparatorCharacters includes: nextChar]]
		whileTrue: [writeBuffer nextPut: self next].
	^ writeBuffer contents.! !

!XMLNestedStreamReader methodsFor: 'accessing' stamp: 'JAAyer 10/18/2010 16:17'!
peek
	"Return the next character from the current input stream."

	^ peekChar ifNil: [peekChar := self basicNext]! !

!XMLNestedStreamReader methodsFor: 'private' stamp: 'JAAyer 10/18/2010 16:08'!
popCurrentStream
	stream close.
	stream := nestedStreams removeLast.
	nestedStreams ifEmpty: [hasNestedStreams := false].! !

!XMLNestedStreamReader methodsFor: 'streaming' stamp: 'JAAyer 12/14/2010 02:25'!
pushBack: aString
	"Fixed to push the string before the peek char (if any)."
	| pushBackString |

	pushBackString := aString.
	peekChar
		ifNotNil: [
			pushBackString := pushBackString copyWith: peekChar.
			peekChar := nil].

	self pushStream: (ReadStream on: pushBackString).! !

!XMLNestedStreamReader methodsFor: 'streaming' stamp: 'JAAyer 12/14/2010 02:26'!
pushStream: aStream
	"Continue parsing from the new nested stream."
	peekChar ifNotNil: [self pushBack: ''].

	nestedStreams
		ifNil: [nestedStreams := OrderedCollection with: stream]
		ifNotNil: [nestedStreams addLast: stream].
	hasNestedStreams := true.

	self stream: aStream.! !

!XMLNestedStreamReader methodsFor: 'streaming' stamp: 'JAAyer 7/21/2010 11:39'!
skipSeparators
	| nextChar |

	[(nextChar := self peek) notNil
		and: [SeparatorCharacters includes: nextChar]]
		whileTrue: [self next].! !

!XMLNestedStreamReader methodsFor: 'streaming' stamp: 'JAAyer 7/12/2010 01:12'!
skipUpTo: aDelimiter
	| nextChar |

	[self atEnd or: [(nextChar := self next) == aDelimiter]] whileFalse.
	^ nextChar == aDelimiter.! !

!XMLNestedStreamReader methodsFor: 'accessing' stamp: 'JAAyer 3/11/2010 08:01'!
stream
	^ stream! !

!XMLNestedStreamReader methodsFor: 'accessing' stamp: 'JAAyer 12/14/2010 02:23'!
stream: aNewStream
	stream := XMLPeekableStreamAdapter on: aNewStream! !

!XMLNestedStreamReader methodsFor: 'streaming' stamp: 'JAA 3/22/2011 11:49'!
upTo: aDelimiter
	| nextChar |

	writeBuffer reset.
	[self atEnd or: [(nextChar := self next) == aDelimiter]]
		whileFalse: [writeBuffer nextPut: nextChar].

	^ writeBuffer contents.! !

!XMLNestedStreamReader methodsFor: 'streaming' stamp: 'JAA 3/22/2011 11:49'!
upToAll: aDelimitingString
	"Answer a subcollection from the current access position to the occurrence
	(if any, but not inclusive) of delimitingString. If delimitingString is not
	in the stream, answer the entire rest of the stream."

	writeBuffer reset.
	[self atEnd or: [self nextMatchAll: aDelimitingString]]
		whileFalse: [writeBuffer nextPut: self next].
	^ writeBuffer contents.! !

!XMLNestedStreamReader methodsFor: 'streaming' stamp: 'JAAyer 12/14/2010 02:39'!
useConverterForEncoding: anEncodingName
	| converterClass |

	converterClass :=
		(Smalltalk
			at: #TextConverter
			ifAbsent: [^ self])
				defaultConverterClassForEncoding: anEncodingName asLowercase.
		
	converterClass
		ifNotNil: [stream converter: converterClass new].! !

Object subclass: #XMLNestedStreamWriter
	instanceVariableNames: 'streams nextStreamIndex'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser'!

!XMLNestedStreamWriter commentStamp: 'JAAyer 7/12/2010 01:22' prior: 0!
This class handles the allocation and reuse of multiple WriteStreams. It uses a single execute-around method, #writeWith:, which takes a block that it evaluates with a ready-to-use WriteStream.
!

!XMLNestedStreamWriter class methodsFor: 'instance creation' stamp: 'JAAyer 9/19/2010 23:48'!
new
	^ self new: 5! !

!XMLNestedStreamWriter class methodsFor: 'instance creation' stamp: 'JAAyer 9/19/2010 23:51'!
new: aNumberOfStreams
	^ self basicNew initialize: aNumberOfStreams! !

!XMLNestedStreamWriter methodsFor: 'private' stamp: 'JAAyer 9/19/2010 23:54'!
acquireStream
	| stream |

	(nextStreamIndex <= streams size)
		 ifFalse: [self grow].

	(stream := streams at: nextStreamIndex) reset.
	nextStreamIndex := nextStreamIndex + 1.
	^ stream.! !

!XMLNestedStreamWriter methodsFor: 'private' stamp: 'JAAyer 10/16/2010 04:41'!
grow
	streams :=
		(Array new: ((streams size * 2) max: 10))
			replaceFrom: 1
			to: streams size
			with: streams
			startingAt: 1.
	nextStreamIndex to: streams size do: [:i | streams at: i put: self newStream].! !

!XMLNestedStreamWriter methodsFor: 'initialization' stamp: 'JAAyer 9/19/2010 23:50'!
initialize: aNumberOfStreams
	streams := Array new: aNumberOfStreams.
	1 to: aNumberOfStreams do: [:i | streams at: i put: self newStream].
	nextStreamIndex := 1.! !

!XMLNestedStreamWriter methodsFor: 'private' stamp: 'JAAyer 7/12/2010 01:19'!
newStream
	^ WriteStream on: (String new: 128)! !

!XMLNestedStreamWriter methodsFor: 'private' stamp: 'JAAyer 9/19/2010 23:54'!
releaseStream
	nextStreamIndex := nextStreamIndex - 1! !

!XMLNestedStreamWriter methodsFor: 'writing' stamp: 'JAAyer 9/19/2010 23:42'!
writeWith: aBlock
	| result |

	"Avoid ensure: for performance"
	result := aBlock value: self acquireStream.
	self releaseStream.
	^ result.! !

Object subclass: #XMLNode
	instanceVariableNames: 'parent notifier'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-Nodes'!

!XMLNode commentStamp: '<historical>' prior: 0!
This is a base class for XML nodes. It contains testing messages and methods to set and access the parent node and sibling nodes. It also provides a number messages to control printing.!

XMLNode subclass: #HTMLNode
	instanceVariableNames: 'title lang translate dir dataset itemScope itemType itemId itemRef itemProp properties itemValue ownerDocument attributesList'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-Nodes'!

!HTMLNode methodsFor: 'accessing' stamp: 'MohammadHoussami 7/13/2013 17:45'!
attributesList
	^ attributesList! !

!HTMLNode methodsFor: 'accessing' stamp: 'MohammadHoussami 7/13/2013 17:45'!
attributesList: anObject
	attributesList := anObject! !

!HTMLNode methodsFor: 'accessing' stamp: 'MohammadHoussami 7/11/2013 23:14'!
dataset
	^ dataset! !

!HTMLNode methodsFor: 'accessing' stamp: 'MohammadHoussami 7/11/2013 23:14'!
dataset: anObject
	dataset := anObject! !

!HTMLNode methodsFor: 'accessing' stamp: 'MohammadHoussami 7/11/2013 23:14'!
dir
	^ dir! !

!HTMLNode methodsFor: 'accessing' stamp: 'MohammadHoussami 7/11/2013 23:14'!
dir: anObject
	dir := anObject! !

!HTMLNode methodsFor: 'FromSpec' stamp: 'MohammadHoussami 7/17/2013 12:16'!
initialize 
attributesList := OrderedCollection new.! !

!HTMLNode methodsFor: 'accessing' stamp: 'MohammadHoussami 7/11/2013 23:14'!
itemId
	^ itemId! !

!HTMLNode methodsFor: 'accessing' stamp: 'MohammadHoussami 7/11/2013 23:14'!
itemId: anObject
	itemId := anObject! !

!HTMLNode methodsFor: 'accessing' stamp: 'MohammadHoussami 7/11/2013 23:14'!
itemProp
	^ itemProp! !

!HTMLNode methodsFor: 'accessing' stamp: 'MohammadHoussami 7/11/2013 23:14'!
itemProp: anObject
	itemProp := anObject! !

!HTMLNode methodsFor: 'accessing' stamp: 'MohammadHoussami 7/11/2013 23:14'!
itemRef
	^ itemRef! !

!HTMLNode methodsFor: 'accessing' stamp: 'MohammadHoussami 7/11/2013 23:14'!
itemRef: anObject
	itemRef := anObject! !

!HTMLNode methodsFor: 'accessing' stamp: 'MohammadHoussami 7/11/2013 23:14'!
itemScope
	^ itemScope! !

!HTMLNode methodsFor: 'accessing' stamp: 'MohammadHoussami 7/11/2013 23:14'!
itemScope: anObject
	itemScope := anObject! !

!HTMLNode methodsFor: 'accessing' stamp: 'MohammadHoussami 7/11/2013 23:14'!
itemType
	^ itemType! !

!HTMLNode methodsFor: 'accessing' stamp: 'MohammadHoussami 7/11/2013 23:14'!
itemType: anObject
	itemType := anObject! !

!HTMLNode methodsFor: 'accessing' stamp: 'MohammadHoussami 7/11/2013 23:14'!
itemValue
	^ itemValue! !

!HTMLNode methodsFor: 'accessing' stamp: 'MohammadHoussami 7/11/2013 23:14'!
itemValue: anObject
	itemValue := anObject! !

!HTMLNode methodsFor: 'accessing' stamp: 'MohammadHoussami 7/11/2013 23:14'!
lang
	^ lang! !

!HTMLNode methodsFor: 'accessing' stamp: 'MohammadHoussami 7/11/2013 23:14'!
lang: anObject
	lang := anObject! !

!HTMLNode methodsFor: 'FromSpec' stamp: 'MohammadHoussami 7/12/2013 18:37'!
ownerDocument
	^ ownerDocument! !

!HTMLNode methodsFor: 'FromSpec' stamp: 'MohammadHoussami 7/12/2013 18:37'!
ownerDocument: anObject
	ownerDocument := anObject! !

!HTMLNode methodsFor: 'accessing' stamp: 'MohammadHoussami 7/11/2013 23:14'!
properties
	^ properties! !

!HTMLNode methodsFor: 'accessing' stamp: 'MohammadHoussami 7/11/2013 23:14'!
properties: anObject
	properties := anObject! !

!HTMLNode methodsFor: 'accessing' stamp: 'MohammadHoussami 7/11/2013 23:14'!
title
	^ title! !

!HTMLNode methodsFor: 'accessing' stamp: 'MohammadHoussami 7/11/2013 23:14'!
title: anObject
	title := anObject! !

!HTMLNode methodsFor: 'accessing' stamp: 'MohammadHoussami 7/11/2013 23:14'!
translate
	^ translate! !

!HTMLNode methodsFor: 'accessing' stamp: 'MohammadHoussami 7/11/2013 23:14'!
translate: anObject
	translate := anObject! !

XMLNode subclass: #XMLAttribute
	instanceVariableNames: 'element name value'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-Nodes'!

!XMLAttribute commentStamp: '<historical>' prior: 0!
The class represents an attribute node, to be stored in an attribute node list. You generally won't need to interact with these objects directly; the various #attribute- messages element nodes respond to will normally suffice. The name of an attribute node can be namespaced using prefixes; however, the prefix must be mapped to a namespace URI in the containing element node. Unprefixed attributes have no namespace, not even a default namespace URI if one is in scope, per the XML namespace standard.

Note that attribute nodes are not children of the elements that contain them; rather, they are considered to be "part" of the element, and this relationship is modeled using a separate instance variable ("element") with different accessor messages.!

!XMLAttribute class methodsFor: 'instance creation' stamp: 'JAAyer 10/24/2010 18:05'!
name: aName value: aValue
	^ self new
		name: aName;
		value: aValue! !

!XMLAttribute methodsFor: 'accessing' stamp: 'JAAyer 10/24/2010 18:04'!
element
	^ element! !

!XMLAttribute methodsFor: 'accessing' stamp: 'JAAyer 10/24/2010 18:04'!
element: anElement
	element := anElement! !

!XMLAttribute methodsFor: 'accessing' stamp: 'JAAyer 4/12/2010 00:37'!
expandedName
	^ self nodeName expandWith: self namespaceURI! !

!XMLAttribute methodsFor: 'testing' stamp: 'JAAyer 6/26/2010 07:31'!
hasPrefix
	^ self nodeName isPrefixed! !

!XMLAttribute methodsFor: 'testing' stamp: 'JAAyer 7/22/2010 08:29'!
isNamed: aName
	^ self nodeName isSameAs: aName! !

!XMLAttribute methodsFor: 'accessing' stamp: 'JAAyer 6/27/2010 15:16'!
localName
	^ self nodeName localPart! !

!XMLAttribute methodsFor: 'accessing' stamp: 'JAAyer 7/12/2010 00:14'!
name
	^ self nodeName qualified! !

!XMLAttribute methodsFor: 'accessing' stamp: 'JAAyer 6/27/2010 15:19'!
name: aName
	self nodeName qualified: aName! !

!XMLAttribute methodsFor: 'accessing' stamp: 'JAAyer 10/24/2010 22:02'!
namespaceURI
	(self element notNil and: [self hasPrefix])
		ifFalse: [^ ''].
	
	^ self element resolvePrefix: self prefix.! !

!XMLAttribute methodsFor: 'private' stamp: 'OttoBehrens 2/22/2011 23:42'!
nodeName
	^ name
		ifNil: [ 
			(name := XMLNodeName new)
				addObserver: self;
				yourself ]! !

!XMLAttribute methodsFor: 'notifications' stamp: 'JAAyer 7/11/2010 02:25'!
nodeNameChangedFrom: anOldName to: aNewName
	self
		notifyOf: #renamedAttribute:from:to:
		with: self
		with: anOldName
		with: aNewName! !

!XMLAttribute methodsFor: 'accessing' stamp: 'JAAyer 3/17/2010 03:48'!
prefix
	^ self nodeName prefix! !

!XMLAttribute methodsFor: 'accessing' stamp: 'JAAyer 6/15/2010 15:15'!
prefix: aPrefix
	self nodeName prefix: aPrefix! !

!XMLAttribute methodsFor: 'accessing' stamp: 'JAAyer 7/22/2010 03:02'!
value
	^ value ifNil: [value := '']! !

!XMLAttribute methodsFor: 'accessing' stamp: 'JAAyer 3/5/2010 08:18'!
value: aString
	value := aString! !

!XMLAttribute methodsFor: 'printing' stamp: 'JAA 3/22/2011 15:15'!
writeXMLOn: aWriter
	aWriter writeName: self name value: self value! !

XMLNode subclass: #XMLComment
	instanceVariableNames: 'string'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-Nodes'!

!XMLComment commentStamp: '<historical>' prior: 0!
This class represents comments preserved as nodes. Comments are ignored during parsing by default. If you want them preserved as comment nodes, use #preservesCommentNodes: with the DOM parser before parsing.!

!XMLComment class methodsFor: 'instance creation' stamp: 'JAAyer 8/11/2010 22:14'!
string: aString
	^ self new string: aString! !

!XMLComment methodsFor: 'testing' stamp: 'JAAyer 8/11/2010 22:15'!
isComment
	^ true! !

!XMLComment methodsFor: 'accessing' stamp: 'JAAyer 8/11/2010 22:14'!
string
	^ string ifNil: [string := '']! !

!XMLComment methodsFor: 'accessing' stamp: 'JAAyer 8/11/2010 22:14'!
string: aString
	string := aString! !

!XMLComment methodsFor: 'printing' stamp: 'JAA 3/22/2011 15:15'!
writeXMLOn: aWriter
	aWriter comment: self string! !

XMLNode subclass: #XMLDeclarationNode
	instanceVariableNames: 'name'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-Nodes'!

!XMLDeclarationNode commentStamp: 'JAAyer 8/11/2010 21:03' prior: 0!
This is a generic class to represent markup declarations in the internal subset. Each subclass represents a specific kind of declaration, and each is immutable.!

XMLDeclarationNode subclass: #XMLAttributeDeclaration
	instanceVariableNames: 'attributeName type defaultDeclaration'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-Nodes'!

!XMLAttributeDeclaration class methodsFor: 'instance creation' stamp: 'JAAyer 8/9/2010 19:57'!
elementName: anElementName attributeName: anAttributeName type: aType defaultDeclaration: aDefaultDeclaration
	^ self new
		setElementName: anElementName
		attributeName: anAttributeName
		type: aType
		defaultDeclaration: aDefaultDeclaration! !

!XMLAttributeDeclaration methodsFor: 'accessing' stamp: 'JAAyer 10/30/2010 19:42'!
attributeName
	^ attributeName ifNil: [attributeName := '']! !

!XMLAttributeDeclaration methodsFor: 'accessing' stamp: 'JAAyer 10/30/2010 19:42'!
defaultDeclaration
	^ defaultDeclaration ifNil: [defaultDeclaration := '']! !

!XMLAttributeDeclaration methodsFor: 'accessing' stamp: 'JAAyer 7/27/2010 08:48'!
elementName
	^ self name! !

!XMLAttributeDeclaration methodsFor: 'initialization' stamp: 'JAAyer 8/9/2010 19:59'!
setElementName: anElementName attributeName: anAttributeName type: aType defaultDeclaration: aDefaultDeclaration
	name := anElementName.
	attributeName := anAttributeName.
	type := aType.
	defaultDeclaration := aDefaultDeclaration.! !

!XMLAttributeDeclaration methodsFor: 'accessing' stamp: 'JAAyer 10/30/2010 19:42'!
type
	^ type ifNil: [type := '']! !

!XMLAttributeDeclaration methodsFor: 'printing' stamp: 'JAA 3/22/2011 15:15'!
writeXMLOn: aWriter
	aWriter attributeDeclaration
		elementName: self elementName;
		attributeName: self attributeName;
		type: self type;
		defaultDeclaration: self defaultDeclaration! !

!XMLDeclarationNode methodsFor: 'accessing' stamp: 'JAAyer 10/30/2010 19:44'!
name
	^ name ifNil: [name := '']! !

XMLDeclarationNode subclass: #XMLElementDeclaration
	instanceVariableNames: 'contentModel'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-Nodes'!

!XMLElementDeclaration class methodsFor: 'instance creation' stamp: 'JAAyer 7/20/2010 10:40'!
name: aName contentModel: aContentModelString
	^ self new setName: aName contentModel: aContentModelString! !

!XMLElementDeclaration methodsFor: 'accessing' stamp: 'JAAyer 10/30/2010 19:43'!
contentModel
	^ contentModel ifNil: [contentModel := '']! !

!XMLElementDeclaration methodsFor: 'initialization' stamp: 'JAAyer 7/20/2010 10:41'!
setName: aName contentModel: aContentModelString
	name := aName.
	contentModel := aContentModelString.! !

!XMLElementDeclaration methodsFor: 'printing' stamp: 'JAA 3/22/2011 15:15'!
writeXMLOn: aWriter
	aWriter elementDeclaration
		name: self name;
		contentModel: self contentModel! !

XMLDeclarationNode subclass: #XMLEntityDeclaration
	instanceVariableNames: 'replacement publicId systemId ndata'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-Nodes'!

!XMLEntityDeclaration class methodsFor: 'instance creation' stamp: 'JAAyer 7/20/2010 10:53'!
name: aName publicID: aPublicId systemID: aSystemId
	^ self new
		setName: aName
		replacement: nil
		publicID: aPublicId
		systemID: aSystemId
		ndata: nil! !

!XMLEntityDeclaration class methodsFor: 'instance creation' stamp: 'JAAyer 7/20/2010 10:53'!
name: aName publicID: aPublicId systemID: aSystemId ndata: aNotationName
	^ self new
		setName: aName
		replacement: nil
		publicID: aPublicId
		systemID: aSystemId
		ndata: aNotationName! !

!XMLEntityDeclaration class methodsFor: 'instance creation' stamp: 'JAAyer 7/20/2010 10:52'!
name: aName replacement: aReplacement
	^ self new
		setName: aName
		replacement: aReplacement
		publicID: nil
		systemID: nil
		ndata: nil! !

!XMLEntityDeclaration methodsFor: 'accessing' stamp: 'JAAyer 10/30/2010 19:43'!
ndata
	^ ndata ifNil: [ndata := '']! !

!XMLEntityDeclaration methodsFor: 'accessing' stamp: 'JAAyer 10/30/2010 19:43'!
publicID
	^ publicId ifNil: [publicId := '']! !

!XMLEntityDeclaration methodsFor: 'accessing' stamp: 'JAAyer 10/30/2010 19:43'!
replacement
	^ replacement ifNil: [replacement := '']! !

!XMLEntityDeclaration methodsFor: 'initialization' stamp: 'JAAyer 7/20/2010 10:51'!
setName: aName replacement: aReplacement publicID: aPublicId systemID: aSystemId ndata: aNotationName
	name := aName.
	replacement := aReplacement.
	publicId := aPublicId.
	systemId := aSystemId.
	ndata := aNotationName.! !

!XMLEntityDeclaration methodsFor: 'accessing' stamp: 'JAAyer 10/30/2010 19:43'!
systemID
	^ systemId ifNil: [systemId := '']! !

!XMLEntityDeclaration methodsFor: 'printing' stamp: 'JAA 3/22/2011 15:15'!
writeXMLOn: aWriter
	aWriter entityDeclaration
		name: self name;
		replacement: self replacement;
		publicID: self publicID;
		systemID: self systemID;
		ndata: self ndata! !

XMLDeclarationNode subclass: #XMLNotationDeclaration
	instanceVariableNames: 'publicId systemId'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-Nodes'!

!XMLNotationDeclaration class methodsFor: 'instance creation' stamp: 'JAAyer 7/21/2010 00:55'!
name: aName publicID: aPublicId systemID: aSystemId
	^ self new setName: aName publicID: aPublicId systemID: aSystemId! !

!XMLNotationDeclaration methodsFor: 'accessing' stamp: 'JAAyer 10/30/2010 19:43'!
publicID
	^ publicId ifNil: [publicId := '']! !

!XMLNotationDeclaration methodsFor: 'initialization' stamp: 'JAAyer 7/21/2010 00:54'!
setName: aName publicID: aPublicId systemID: aSystemId
	name := aName.
	publicId := aPublicId.
	systemId := aSystemId.! !

!XMLNotationDeclaration methodsFor: 'accessing' stamp: 'JAAyer 10/30/2010 19:43'!
systemID
	^ systemId ifNil: [systemId := '']! !

!XMLNotationDeclaration methodsFor: 'printing' stamp: 'JAA 3/22/2011 15:15'!
writeXMLOn: aWriter
	aWriter notationDeclaration
		name: self name;
		publicID: self publicID;
		systemID: self systemID! !

!XMLNode class methodsFor: 'utility' stamp: 'OttoBehrens 2/15/2011 08:33'!
notifications
	| notifications parser parsedMethods |
	notifications := WriteStream on: (Array new: 10).
	parser := Parser new.
	parsedMethods := IdentityDictionary new.
	(self allMethodsInCategory: #notification)
		do: [ :notifierSelector | 
			| methodNode |
			(SystemNavigation default allSendersOf: notifierSelector inClass: self)
				do: [ :methodReference | 
					methodNode := parsedMethods
						at: methodReference selector
						ifAbsentPut: (parser parse: methodReference sourceCode class: self).
					methodNode
						nodesDo: [ :node | 
							| firstArgument |
							(node isMessage
								and: [ node selector key asSymbol = notifierSelector and: [ (firstArgument := node arguments first) isLiteral ] ])
								ifTrue: [ notifications nextPut: firstArgument key asSymbol ] ] ] ].
	^ notifications contents! !

!XMLNode methodsFor: 'accessing' stamp: 'JAAyer 7/20/2010 10:15'!
addContent: aStringOrStringNode
	XMLDOMException signal: 'Node cannot contain content'! !

!XMLNode methodsFor: 'observing' stamp: 'OttoBehrens 2/15/2011 08:33'!
addObserver: anObserver
	self notifier addObserver: anObserver! !

!XMLNode methodsFor: 'enumerating' stamp: 'JAAyer 11/25/2010 10:59'!
allNodesDo: aBlock
	aBlock value: self! !

!XMLNode methodsFor: 'printing' stamp: 'JAA 3/22/2011 15:15'!
canonicallyPrintOn: aStream
	(XMLWriter on: aStream)
		isCanonical: true;
		writeWith: [:writer | self writeXMLOn: writer]! !

!XMLNode methodsFor: 'printing' stamp: 'JAAyer 11/6/2010 01:19'!
canonicallyPrinted
	^ String streamContents: [:stream | self printCanonicalOn: stream]! !

!XMLNode methodsFor: 'accessing' stamp: 'JAAyer 7/20/2010 10:15'!
characterData
	self deprecated: 'use #contentString instead'.

	^ self contentString.! !

!XMLNode methodsFor: 'accessing' stamp: 'JAAyer 11/26/2010 12:16'!
contentString
 	"This returns the combined content string of all descendants with spaces inserted wher necessary."
	^ String streamContents: [:stream | self printContentOn: stream]! !

!XMLNode methodsFor: 'testing' stamp: 'JAAyer 10/24/2010 18:49'!
hasChildren
	^ false! !

!XMLNode methodsFor: 'testing' stamp: 'JAAyer 2/25/2010 14:37'!
hasParent
	^ self parent notNil! !

!XMLNode methodsFor: 'testing' stamp: 'JAAyer 8/11/2010 22:16'!
isCData
	^ false! !

!XMLNode methodsFor: 'testing' stamp: 'JAAyer 8/11/2010 22:15'!
isComment
	^ false! !

!XMLNode methodsFor: 'testing' stamp: 'JAAyer 4/30/2010 19:19'!
isContentNode
	^ false! !

!XMLNode methodsFor: 'testing' stamp: 'JAAyer 2/19/2010 23:50'!
isDocument
	^ false! !

!XMLNode methodsFor: 'testing' stamp: 'JAAyer 4/11/2010 22:59'!
isElement
	^ false! !

!XMLNode methodsFor: 'testing' stamp: 'JAAyer 10/24/2010 18:49'!
isEmpty
	self deprecated: 'use "hasChildren not" instead'.

	^ self hasChildren not! !

!XMLNode methodsFor: 'testing' stamp: 'JAAyer 4/13/2010 21:43'!
isPI
	^ false! !

!XMLNode methodsFor: 'testing' stamp: 'JAAyer 6/27/2010 19:31'!
isProcessingInstruction
	self deprecated: 'use #isPI instead'.

	^ self isPI.! !

!XMLNode methodsFor: 'testing' stamp: 'JAAyer 4/12/2010 22:33'!
isStringNode
	^ false! !

!XMLNode methodsFor: 'testing' stamp: 'JAAyer 6/27/2010 19:31'!
isTag
	self deprecated: 'use #isElement instead'.

	^ self isElement.! !

!XMLNode methodsFor: 'testing' stamp: 'JAAyer 10/15/2010 19:01'!
isTagOrText
	self deprecated: 'use #isContentNode instead'.

	^ self isContentNode.! !

!XMLNode methodsFor: 'testing' stamp: 'JAAyer 6/27/2010 19:31'!
isText
	self deprecated: 'use #isStringNode instead'.

	^ self isStringNode.! !

!XMLNode methodsFor: 'accessing' stamp: 'JAAyer 10/30/2010 19:56'!
nextNode
	^ self parent ifNotNil: [self parent nodeAfter: self]! !

!XMLNode methodsFor: 'testing' stamp: 'JAAyer 10/24/2010 18:49'!
notEmpty
	self deprecated: 'use #hasChildren instead'.

	^ self hasChildren.! !

!XMLNode methodsFor: 'accessing' stamp: 'JAAyer 6/11/2010 10:17'!
notifier
	^ notifier ifNil: [notifier := XMLNotifier new]! !

!XMLNode methodsFor: 'notification' stamp: 'OttoBehrens 2/15/2011 08:33'!
notifyOf: aSelector with: anArgument
	self notifier notifyOf: aSelector with: anArgument! !

!XMLNode methodsFor: 'notification' stamp: 'OttoBehrens 2/15/2011 08:34'!
notifyOf: aSelector with: aFirstArgument with: aSecondArgument
	self notifier notifyOf: aSelector with: aFirstArgument with: aSecondArgument! !

!XMLNode methodsFor: 'notification' stamp: 'OttoBehrens 2/15/2011 08:34'!
notifyOf: aSelector with: aFirstArgument with: aSecondArgument with: aThirdArgument
	self notifier
		notifyOf: aSelector
		with: aFirstArgument
		with: aSecondArgument
		with: aThirdArgument! !

!XMLNode methodsFor: 'notification' stamp: 'OttoBehrens 2/15/2011 08:34'!
notifyOf: aSelector withArguments: anArray
	self notifier notifyOf: aSelector withArguments: anArray! !

!XMLNode methodsFor: 'accessing' stamp: 'JAAyer 2/25/2010 14:37'!
parent
	^ parent! !

!XMLNode methodsFor: 'accessing' stamp: 'JAAyer 2/25/2010 14:37'!
parent: aParentNode
	parent := aParentNode! !

!XMLNode methodsFor: 'printing' stamp: 'JAA 3/22/2011 15:15'!
prettyPrintOn: aStream
	(XMLWriter on: aStream)
		enablePrettyPrinting;
		writeWith: [:writer | self writeXMLOn: writer]! !

!XMLNode methodsFor: 'printing' stamp: 'JAAyer 11/6/2010 01:18'!
prettyPrinted
	^ String streamContents: [:stream | self prettyPrintOn: stream]! !

!XMLNode methodsFor: 'accessing' stamp: 'JAAyer 10/30/2010 19:56'!
previousNode
	^ self parent ifNotNil: [self parent nodeBefore: self]! !

!XMLNode methodsFor: 'printing' stamp: 'JAAyer 11/6/2010 01:25'!
printCanonicalOn: aStream
	self deprecated: 'use #canonicallyPrintOn: instead'.

	self canonicallyPrintOn: aStream.! !

!XMLNode methodsFor: 'printing' stamp: 'JAAyer 10/20/2010 19:11'!
printContentOn: aStream! !

!XMLNode methodsFor: 'printing' stamp: 'JAA 3/22/2011 15:15'!
printOn: aStream
	(XMLWriter on: aStream) writeWith: [:writer | self writeXMLOn: writer].! !

!XMLNode methodsFor: 'printing' stamp: 'NorbertHartl 7/7/2011 13:48'!
printXMLOn: aWriter
	self deprecated: 'use #writeXMLOn: instead'.

	self writeXMLOn: aWriter.! !

!XMLNode methodsFor: 'observing' stamp: 'OttoBehrens 2/15/2011 08:34'!
removeObserver: anObserver
	self notifier removeObserver: anObserver! !

!XMLNode methodsFor: 'printing' stamp: 'JAA 3/22/2011 15:15'!
writeXMLOn: aWriter
	self subclassResponsibility! !

XMLNode subclass: #XMLNodeWithChildren
	instanceVariableNames: 'nodes'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-Nodes'!

!XMLNodeWithChildren commentStamp: '<historical>' prior: 0!
This class represents a node that can contain child nodes. You probably will only use its API through one its subclasses, specifically the document, element and doctype declaration node classes.

This class provides message to access, add and remove child nodes under the "accessing" category. The nodes themselves are stored in some type of XMLObservableList, the exact type of which varies depending on the subclass and can be accessed directly using #nodes. You can modify the #nodes list directly if you like, but adding nodes to it or removing nodes from it will have the effect of adding them to removing them from the parent node. If this is not what you want, then make sure you copy the return value of #nodes before manipulating it.

The "enumerating" category provides two different modes of enumeration: the #nodes* enumerating messages enumerate child nodes of the receiver only, while the #allNode* forms recursively enumerate all descendent nodes, including the receiver itself, using depth-first traversal.

The inner XML of a node with children can be accessed as a string using #innerXML and set using #innerXML:.!

XMLNodeWithChildren subclass: #XMLDoctypeDeclaration
	instanceVariableNames: 'root publicId systemId'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-Nodes'!

!XMLDoctypeDeclaration commentStamp: '<historical>' prior: 0!
This class represents the <!!DOCTYPE ...> declaration one often finds at the top of an XML document. It stores the root element name, formal public identifier (FPI) if any, system identifier (a URI) and then any nodes comprising the internal subset.!

!XMLDoctypeDeclaration class methodsFor: 'instance creation' stamp: 'JAAyer 7/21/2010 07:47'!
root: aRootElementName publicID: aPublicId systemID: aSystemId
	^ self new
		setRoot: aRootElementName
		publicID: aPublicId
		systemID: aSystemId! !

!XMLDoctypeDeclaration methodsFor: 'private' stamp: 'JAAyer 7/20/2010 10:26'!
nodesClass
	^ XMLInternalSubsetList! !

!XMLDoctypeDeclaration methodsFor: 'accessing' stamp: 'JAAyer 6/17/2010 21:35'!
publicID
	^ publicId! !

!XMLDoctypeDeclaration methodsFor: 'accessing' stamp: 'JAAyer 7/21/2010 07:46'!
root
	^ root! !

!XMLDoctypeDeclaration methodsFor: 'initialization' stamp: 'JAAyer 7/21/2010 07:45'!
setRoot: aRootElementName publicID: aPublicId systemID: aSystemId
	root := aRootElementName.
	publicId := aPublicId.
	systemId := aSystemId.! !

!XMLDoctypeDeclaration methodsFor: 'accessing' stamp: 'JAAyer 6/17/2010 21:35'!
systemID
	^ systemId! !

!XMLDoctypeDeclaration methodsFor: 'printing' stamp: 'JAA 3/23/2011 09:42'!
writeXMLOn: aWriter
	aWriter doctype
		root: self root;
		publicID: self publicID;
		systemID: self systemID;
		withInternalSubset: [self writeInnerXMLOn: aWriter]! !

!XMLNodeWithChildren methodsFor: 'accessing' stamp: 'JAAyer 7/20/2010 10:22'!
addNode: aNode
	^ self nodes addLast: aNode! !

!XMLNodeWithChildren methodsFor: 'accessing' stamp: 'JAAyer 8/14/2010 21:05'!
addNode: aNode after: aPriorNode
	^ self nodes add: aNode after: aPriorNode! !

!XMLNodeWithChildren methodsFor: 'accessing' stamp: 'JAAyer 8/14/2010 21:05'!
addNode: aNode before: aSubsequentNode
	^ self nodes add: aNode before: aSubsequentNode! !

!XMLNodeWithChildren methodsFor: 'accessing' stamp: 'JAAyer 7/20/2010 10:22'!
addNodeFirst: aNode
	^ self nodes addFirst: aNode! !

!XMLNodeWithChildren methodsFor: 'accessing' stamp: 'JAAyer 7/22/2010 05:12'!
addNodes: aNodeCollection
	^ self nodes addAll: aNodeCollection! !

!XMLNodeWithChildren methodsFor: 'notifications' stamp: 'JAAyer 10/24/2010 18:09'!
addedNode: aNode
	aNode
		addObserver: self;
		parent: self! !

!XMLNodeWithChildren methodsFor: 'accessing' stamp: 'JAAyer 12/4/2010 12:32'!
allNodes
	^ self allNodesCollect: [:each | each]! !

!XMLNodeWithChildren methodsFor: 'enumerating' stamp: 'JAAyer 11/28/2010 13:11'!
allNodesCollect: aBlock
	| collectedNodes |

	collectedNodes := self nodes copyEmpty.
	self allNodesDo: [:each | collectedNodes add: (aBlock value: each)].
	^ collectedNodes.! !

!XMLNodeWithChildren methodsFor: 'enumerating' stamp: 'JAAyer 11/28/2010 12:17'!
allNodesDetect: aBlock
	^ self allNodesDetect: aBlock ifNone: [nil]! !

!XMLNodeWithChildren methodsFor: 'enumerating' stamp: 'JAAyer 11/28/2010 12:21'!
allNodesDetect: aBlock ifNone: anExceptionBlock
	self allNodesDo: [:each | (aBlock value: each) ifTrue: [^ each]].
	^ anExceptionBlock value.! !

!XMLNodeWithChildren methodsFor: 'enumerating' stamp: 'JAAyer 11/25/2010 10:59'!
allNodesDo: aBlock
	super allNodesDo: aBlock.

	self hasChildren
		ifTrue: [self nodesDo: [:each | each allNodesDo: aBlock]]! !

!XMLNodeWithChildren methodsFor: 'enumerating' stamp: 'JAAyer 11/28/2010 13:11'!
allNodesSelect: aBlock
	| selectedNodes |

	selectedNodes := self nodes copyEmpty.
	self allNodesDo: [:each |
		(aBlock value: each)
			ifTrue: [selectedNodes add: each]].
	^ selectedNodes.! !

!XMLNodeWithChildren methodsFor: 'accessing' stamp: 'OttoBehrens 2/16/2011 17:37'!
firstNode
	^ self nodes ifNotEmpty: [ self nodes first ] ifEmpty: [ nil ]! !

!XMLNodeWithChildren methodsFor: 'testing' stamp: 'JAAyer 10/24/2010 18:48'!
hasChildren
	"Direct variable access avoids possibly unnecessary lazy initialization of the node list"

	^ nodes notNil and: [nodes notEmpty]! !

!XMLNodeWithChildren methodsFor: 'testing' stamp: 'JAAyer 7/20/2010 10:23'!
includesNode: aNode
	^ self nodes includes: aNode! !

!XMLNodeWithChildren methodsFor: 'accessing' stamp: 'JAA 3/23/2011 09:42'!
innerXML
	| writer |

	writer := XMLWriter new.
	self writeInnerXMLOn: writer.
	^ writer contents.! !

!XMLNodeWithChildren methodsFor: 'parsing' stamp: 'JAAyer 11/26/2010 19:23'!
innerXML: aStringOrStream
	self removeNodes.

	(self innerXMLParserOn: aStringOrStream) parseDocument.! !

!XMLNodeWithChildren methodsFor: 'private' stamp: 'JAAyer 11/26/2010 19:23'!
innerXMLParserOn: aStringOrStream
	^ XMLDOMParser on: aStringOrStream withRootNode: self! !

!XMLNodeWithChildren methodsFor: 'accessing' stamp: 'OttoBehrens 2/16/2011 17:38'!
lastNode
	^ self nodes ifNotEmpty: [ self nodes last ] ifEmpty: [ nil ]! !

!XMLNodeWithChildren methodsFor: 'accessing' stamp: 'JAAyer 7/20/2010 10:26'!
nodeAfter: aNode
	^ self nodes after: aNode! !

!XMLNodeWithChildren methodsFor: 'accessing' stamp: 'JAAyer 7/20/2010 10:27'!
nodeBefore: aNode
	^ self nodes before: aNode! !

!XMLNodeWithChildren methodsFor: 'accessing' stamp: 'JAAyer 7/20/2010 12:31'!
nodes
	^ nodes ifNil: [
		(nodes := self nodesClass new)
			addObserver: self;
			yourself]! !

!XMLNodeWithChildren methodsFor: 'private' stamp: 'JAAyer 8/6/2010 23:53'!
nodesClass
	self subclassResponsibility! !

!XMLNodeWithChildren methodsFor: 'enumerating' stamp: 'JAAyer 11/28/2010 12:13'!
nodesCollect: aBlock
	^ self nodes collect: aBlock! !

!XMLNodeWithChildren methodsFor: 'enumerating' stamp: 'JAAyer 11/28/2010 13:45'!
nodesDetect: aBlock
	^ self nodesDetect: aBlock ifNone: [nil]! !

!XMLNodeWithChildren methodsFor: 'enumerating' stamp: 'JAAyer 11/28/2010 12:14'!
nodesDetect: aBlock ifNone: anExceptionBlock
	^ self nodes detect: aBlock ifNone: anExceptionBlock! !

!XMLNodeWithChildren methodsFor: 'enumerating' stamp: 'JAAyer 7/20/2010 10:27'!
nodesDo: aBlock
	self nodes do: aBlock! !

!XMLNodeWithChildren methodsFor: 'enumerating' stamp: 'JAAyer 11/28/2010 12:13'!
nodesSelect: aBlock
	^ self nodes select: aBlock! !

!XMLNodeWithChildren methodsFor: 'accessing' stamp: 'JAAyer 7/20/2010 10:27'!
removeNode: aNode
	^ self nodes remove: aNode! !

!XMLNodeWithChildren methodsFor: 'accessing' stamp: 'JAAyer 7/20/2010 10:27'!
removeNodes
	self nodes removeAll! !

!XMLNodeWithChildren methodsFor: 'accessing' stamp: 'JAAyer 7/20/2010 10:27'!
removeNodes: aNodeCollection
	^ self nodes removeAll: aNodeCollection! !

!XMLNodeWithChildren methodsFor: 'notifications' stamp: 'JAAyer 10/24/2010 18:08'!
removedNode: aNode
	aNode
		removeObserver: self;
		parent: nil.! !

!XMLNodeWithChildren methodsFor: 'accessing' stamp: 'JAAyer 11/5/2010 22:39'!
replaceNode: aNode with: aReplacementNode
	^ self nodes
		at: (self nodes indexOf: aNode)
		put: aReplacementNode! !

!XMLNodeWithChildren methodsFor: 'accessing' stamp: 'JAAyer 7/20/2010 10:25'!
topNode
	self deprecated: 'use #firstNode instead'.

	^ self firstNode.! !

!XMLNodeWithChildren methodsFor: 'printing' stamp: 'JAA 3/23/2011 09:42'!
writeInnerXMLOn: aWriter
	self nodesDo: [:each | each writeXMLOn: aWriter]! !

!XMLNodeWithChildren methodsFor: 'printing' stamp: 'JAA 3/23/2011 09:42'!
writeXMLOn: aWriter
	self writeInnerXMLOn: aWriter! !

XMLNodeWithChildren subclass: #XMLNodeWithElements
	instanceVariableNames: 'configuration'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-Nodes'!

!XMLNodeWithElements commentStamp: '<historical>' prior: 0!
This class represents a node with elements. You will likely use its API through its subclasses instead.

Instances provide "accessing" messages to retrieve child elements by their name and namespace information. The #elementAt: forms return the first matching element, while the #elementsAt: forms return all matching child elements.

As with node enumeration in the superclass, there are two different modes of enumeration: the #elements* enumerating messages that enumerate child elements only, and the #allElements* forms that enumerate all descendent elements, including the receiver if it is an element (but not if it is a document), using depth-first traversal.

The #findElementNamed:* forms can be used to search using depth-first traversal for a specific element. Searching will start with the receiver if it is an element.

All element name matching is done using both the qualified and local name of elements, so 'prefix:element-name' will only match 'prefix:element-name' while 'element-name' will match 'element-name', 'prefix:element-name' or 'different-prefix:element-name' and so on.!

XMLNodeWithElements subclass: #XMLDocument
	instanceVariableNames: 'version encoding doctypeDeclaration'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-Nodes'!

!XMLDocument commentStamp: '<historical>' prior: 0!
This class represents a document node, which is often the root of a node tree. It can contain XML version and encoding information, a DOCTYPE declaration with a possible internal subset and a root element child node.!

XMLDocument subclass: #HTMLDocument
	instanceVariableNames: 'scriptFlag'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-Nodes'!

!HTMLDocument methodsFor: 'accessing' stamp: 'MohammadHoussami 7/12/2013 11:56'!
scriptFlag
	^ scriptFlag! !

!HTMLDocument methodsFor: 'accessing' stamp: 'MohammadHoussami 7/12/2013 11:56'!
scriptFlag: anObject
	scriptFlag := anObject! !

!HTMLDocument methodsFor: 'as yet unclassified' stamp: 'MohammadHoussami 7/12/2013 11:57'!
scriptFlagEnabled
scriptFlag = 'enabled' ifTrue: [^ true.].

^false.
! !

!XMLDocument methodsFor: 'accessing' stamp: 'JAAyer 7/10/2010 05:16'!
doctypeDeclaration
	^ doctypeDeclaration! !

!XMLDocument methodsFor: 'accessing' stamp: 'JAAyer 7/10/2010 05:16'!
doctypeDeclaration: aDoctypeDeclaration
	doctypeDeclaration := aDoctypeDeclaration! !

!XMLDocument methodsFor: 'accessing' stamp: 'JAAyer 11/24/2010 13:34'!
document
	^ self! !

!XMLDocument methodsFor: 'accessing' stamp: 'JAAyer 11/24/2010 09:26'!
document: aDocument
	self shouldNotImplement! !

!XMLDocument methodsFor: 'accessing' stamp: 'JAAyer 7/22/2010 03:03'!
encoding	
	^ encoding ifNil: [encoding := 'UTF-8']! !

!XMLDocument methodsFor: 'accessing' stamp: 'mir 1/17/2002 12:57'!
encoding: aString	
	encoding := aString! !

!XMLDocument methodsFor: 'testing' stamp: 'JAAyer 2/19/2010 23:51'!
isDocument
	^ true! !

!XMLDocument methodsFor: 'accessing' stamp: 'JAAyer 11/25/2010 15:28'!
root
	^ self firstElement! !

!XMLDocument methodsFor: 'accessing' stamp: 'JAAyer 11/26/2010 18:24'!
root: anElement
	self
		removeNodes;
		addNode: anElement! !

!XMLDocument methodsFor: 'accessing' stamp: 'JAAyer 2/25/2010 01:48'!
version	
	^ version! !

!XMLDocument methodsFor: 'accessing' stamp: 'JAAyer 8/9/2010 19:39'!
version: aVersionNumber
	version := aVersionNumber! !

!XMLDocument methodsFor: 'printing' stamp: 'JAA 3/23/2011 09:42'!
writeXMLOn: aWriter
	self version ifNotNil: [
		aWriter xml
			version: self version;
			encoding: self encoding].
	self doctypeDeclaration
		ifNotNil: [self doctypeDeclaration writeXMLOn: aWriter].
	
	self writeInnerXMLOn: aWriter.! !

XMLNodeWithElements subclass: #XMLElement
	instanceVariableNames: 'name attributes namespaceScope document'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-Nodes'!

!XMLElement commentStamp: '<historical>' prior: 0!
The class represents an element node. An element consists of a name and optionally attributes, namespace declarations and any number child nodes and elements.

The name can consist of a local name only or a qualified name, which is a namespace prefix and local name separated by a colon. If the name is qualfied and namespace support is enabled (the default), then the prefix must be mapped to a namespace URI in the element or its parent at the time the name is set. The class-side instance creation #name:namespaceURI:* messages and #name:namespaces:* messages allow you set both simulaniously, as does the instance-side #name:namespaceURI: message. If namespace support is disabled, prefixes are ignored (not validated). You can test the name of an element using #isNamed: and #isNamedAny:, which test the qualified and local names of an element for a match.

Namespaces can be declared using #declareNamespace:uri:, using #declareNamespaces: with a collection of associations (where an empty prefix represents the default namespace) or simultaniously while changing the element's name using #name:namespaceURI:. The default namespace can be accessed through #defaultNamespace and #defaultNamespace:. When you print an element, the minimum number of xmlns declarations necessary for it to be valid will be printed. You can query an element's namespace information using #namespaceURI, #prefix, #expandedName and other messages under "namespaces."" For an element to be namespaced, it must have a qualified name with a valid prefix or have a default namespace declaration in scope for it.

The #attribute- messages provide a Dictionary-like prototcol for manipulating attribute nodes. Attribute value accessors return empty strings when the specified value is not present. The underlying attribute node list can be accessed using #attributeNodes, and an order-preserving dictionary of attribute names/values can be obtained using #attributes.

See the comment for the superclass for how to access child nodes and elements.!

!XMLElement class methodsFor: 'instance creation' stamp: 'JAAyer 10/30/2010 22:09'!
name: aLocalName
	^ self new name: aLocalName! !

!XMLElement class methodsFor: 'instance creation' stamp: 'JAAyer 7/22/2010 03:05'!
name: aLocalName attributes: aDictionaryOrArray
	^ (self named: aLocalName)
		setAttributes: aDictionaryOrArray! !

!XMLElement class methodsFor: 'instance creation' stamp: 'JAAyer 7/22/2010 03:06'!
name: aQualifiedName namespaceURI: aUri
	^ self new name: aQualifiedName namespaceURI: aUri! !

!XMLElement class methodsFor: 'instance creation' stamp: 'JAAyer 7/22/2010 03:06'!
name: aQualifiedName namespaceURI: aUri attributes: aDictionaryOrArray
	^ (self name: aQualifiedName namespaceURI: aUri)
		setAttributes: aDictionaryOrArray! !

!XMLElement class methodsFor: 'instance creation' stamp: 'JAAyer 7/22/2010 03:06'!
name: aQualifiedName namespaces: aNamespaceDictionaryOrArray
	^ self new
		declareNamespaces: aNamespaceDictionaryOrArray;
		name: aQualifiedName! !

!XMLElement class methodsFor: 'instance creation' stamp: 'JAAyer 7/22/2010 03:06'!
name: aQualifiedName namespaces: aNamespaceDictionaryOrArray attributes: anAttributeDictionaryOrArray
	^ (self
		name: aQualifiedName
		namespaces: aNamespaceDictionaryOrArray)
			setAttributes: anAttributeDictionaryOrArray! !

!XMLElement class methodsFor: 'instance creation' stamp: 'JAAyer 3/11/2010 23:10'!
named: aLocalName
	^ self new name: aLocalName! !

!XMLElement class methodsFor: 'instance creation' stamp: 'JAAyer 10/24/2010 16:15'!
named: aName attributes: aDictionary
	self deprecated: 'use #name:attributes: instead'.

	^ self name: aName attributes: aDictionary.! !

!XMLElement class methodsFor: 'instance creation' stamp: 'JAAyer 9/2/2010 04:54'!
named: aLocalName namespace: aPrefix uri: aUri attributes: aDictionary
	self deprecated: 'use #name:namespaceURI:attributes: with a qualified name instead'.

	^ self
		name: aPrefix, ':', aLocalName
		namespaceURI: aUri
		attributes: aDictionary.! !

!XMLElement class methodsFor: 'private' stamp: 'JAAyer 6/25/2010 22:47'!
qualifiedName: aQualifiedName prefix: aPrefix localName: aLocalName attributes: anAttributeDictionary namespaceScope: aNamespaceScope
	^ self new
		setQualifiedName: aQualifiedName
		prefix: aPrefix
		localName: aLocalName
		attributes: anAttributeDictionary
		namespaceScope: aNamespaceScope! !

!XMLElement methodsFor: 'accessing' stamp: 'JAAyer 11/26/2010 18:15'!
@ aName
	self deprecated: 'use #attributeAt: instead'.

	^ self attributeAt: aName.! !

!XMLElement methodsFor: 'accessing' stamp: 'JAAyer 10/24/2010 19:07'!
addContent: aStringOrStringNode
	"Adds a string or string node. If the last child node is already a string node,
	the characters being added are concatenated to it."

	(self hasChildren and: [self lastNode isStringNode])
		ifTrue: [self lastNode addContent: aStringOrStringNode]
		ifFalse: [self addNode: aStringOrStringNode asXMLString]! !

!XMLElement methodsFor: 'notifications' stamp: 'JAAyer 10/24/2010 18:07'!
addedAttribute: anAttribute
	anAttribute
		addObserver: self;
		element: self.
	self validateAttributeName: anAttribute name.! !

!XMLElement methodsFor: 'notifications' stamp: 'JAAyer 11/24/2010 09:47'!
addedElement: anElement
	super addedElement: anElement.
	self hasNamespaces
		ifTrue: [anElement inheritNamespacesFrom: self]! !

!XMLElement methodsFor: 'searching' stamp: 'JAAyer 11/25/2010 10:37'!
allElementsDo: aBlock
	"See superclass version."

	aBlock value: self.

	super allElementsDo: aBlock.! !

!XMLElement methodsFor: 'accessing' stamp: 'JAAyer 7/26/2010 13:52'!
at: aName
	self deprecated: 'use #attributeAt: or @ instead'.

	^ self attributeAt: aName! !

!XMLElement methodsFor: 'accessing' stamp: 'JAAyer 7/22/2010 08:40'!
at: aName ifAbsent: aBlock	
	self deprecated: 'use #attributeAt:ifAbsent: instead'.

	^ self attributeAt: aName ifAbsent: aBlock! !

!XMLElement methodsFor: 'accessing' stamp: 'JAAyer 7/22/2010 08:40'!
at: aName ifAbsentPut: aBlock
	self deprecated: 'use #attributeAt:ifAbsentPut: instead'.

	^ self attributeAt: aName ifAbsentPut: aBlock! !

!XMLElement methodsFor: 'accessing' stamp: 'JAAyer 7/22/2010 08:41'!
at: aName put: aValue
	self deprecated: 'use #attributeAt:put: instead'.

	^ self attributeAt: aName put: aValue! !

!XMLElement methodsFor: 'accessing' stamp: 'JAAyer 6/25/2010 22:15'!
attributeAssociations
	^ self attributeNodes asNameValueAssociations! !

!XMLElement methodsFor: 'accessing' stamp: 'JAAyer 7/22/2010 08:33'!
attributeAt: aName
	^ self attributeAt: aName ifAbsent: ['']! !

!XMLElement methodsFor: 'accessing' stamp: 'JAAyer 7/22/2010 08:33'!
attributeAt: aName ifAbsent: aBlock
	^ self attributeNodes attributeValueAt: aName ifAbsent: aBlock! !

!XMLElement methodsFor: 'accessing' stamp: 'JAAyer 7/22/2010 08:33'!
attributeAt: aName ifAbsentPut: aBlock
	^ self attributeNodes attributeValueAt: aName ifAbsentPut: aBlock! !

!XMLElement methodsFor: 'accessing' stamp: 'JAAyer 7/22/2010 08:34'!
attributeAt: aName put: aValue
	^ self attributeNodes attributeValueAt: aName put: aValue! !

!XMLElement methodsFor: 'accessing' stamp: 'JAAyer 6/25/2010 22:08'!
attributeNames
	^ self attributeNodes attributeNames! !

!XMLElement methodsFor: 'enumerating' stamp: 'JAAyer 6/25/2010 22:14'!
attributeNamesAndValuesDo: aTwoArgumentBlock
	self attributeNodes attributeNamesAndValuesDo: aTwoArgumentBlock! !

!XMLElement methodsFor: 'enumerating' stamp: 'JAAyer 6/25/2010 22:09'!
attributeNamesDo: aBlock
	self attributeNodes attributeNamesDo: aBlock! !

!XMLElement methodsFor: 'accessing' stamp: 'JAAyer 11/26/2010 12:27'!
attributeNodeAt: aName
	^ self attributeNodes attributeNodeAt: aName! !

!XMLElement methodsFor: 'accessing' stamp: 'JAAyer 11/26/2010 12:27'!
attributeNodeAt: aName ifAbsent: aBlock
	^ self attributeNodes attributeNodeAt: aName ifAbsent: aBlock! !

!XMLElement methodsFor: 'accessing' stamp: 'JAAyer 11/26/2010 12:28'!
attributeNodeAt: aLocalName namespaceURI: aUri
	^ self attributeNodes attributeNodeAt: aLocalName namespaceURI: aUri! !

!XMLElement methodsFor: 'accessing' stamp: 'JAAyer 11/26/2010 12:28'!
attributeNodeAt: aLocalName namespaceURI: aUri ifAbsent: aBlock
	^ self attributeNodes attributeNodeAt: aLocalName namespaceURI: aUri ifAbsent: aBlock! !

!XMLElement methodsFor: 'accessing' stamp: 'JAAyer 10/15/2010 12:41'!
attributeNodes
	^ attributes ifNil: [
		(attributes := XMLAttributeList new)
			addObserver: self;
			yourself]! !

!XMLElement methodsFor: 'accessing' stamp: 'JAAyer 6/25/2010 22:16'!
attributes
	^ self attributeNodes asDictionary! !

!XMLElement methodsFor: 'accessing' stamp: 'JAAyer 2/25/2010 14:37'!
contentString: aString
	self
		removeNodes;
		addContent: aString! !

!XMLElement methodsFor: 'accessing' stamp: 'JAAyer 6/27/2010 19:31'!
contents
	self deprecated: 'use #stringNodes instead'.

	^ self stringNodes.! !

!XMLElement methodsFor: 'enumerating' stamp: 'JAAyer 6/27/2010 19:31'!
contentsDo: aBlock
	self deprecated: 'use #stringNodesDo: instead'.

	self stringNodesDo: aBlock.! !

!XMLElement methodsFor: 'namespaces' stamp: 'JAAyer 11/5/2010 20:44'!
declareNamespaces: aDictionaryOrArray
	aDictionaryOrArray associationsDo: [:each |
		each key isEmptyOrNil
			ifTrue: [self defaultNamespace: each value]
			ifFalse: [self declarePrefix: each key uri: each value]]! !

!XMLElement methodsFor: 'namespaces' stamp: 'JAAyer 11/24/2010 12:23'!
declarePrefix: aPrefix uri: aUri
	| oldMapping |

	oldMapping := self namespaceScope resolvePrefix: aPrefix.
	oldMapping = aUri
		ifTrue: [^ self].

	self
		updateEnclosedScopesWith: [:scope | scope mapPrefix: aPrefix to: aUri]
		whileTrue: [:scope |
			(scope includesPrefix: aPrefix) not
				or: [scope includesPrefix: aPrefix mappedTo: oldMapping]].! !

!XMLElement methodsFor: 'namespaces' stamp: 'JAAyer 11/24/2010 12:55'!
declaresDefaultNamespace
	"Did the receiver declare a new default namespace rather than inheriting one?"

	^ self namespaceScope hasDefaultNamespace
		and: [self defaultNamespace isEmpty
			or: [self isRoot
				or: [self defaultNamespace ~= self parent defaultNamespace]]]! !

!XMLElement methodsFor: 'namespaces' stamp: 'JAAyer 11/24/2010 12:54'!
declaresPrefix: aPrefix uri: aUri
	"Did the receiver declare a new prefix/uri pair rather than inheriting one?"

	^ (self isDeclaredPrefix: aPrefix uri: aUri)
		and: [self isRoot
			or: [(self parent resolvePrefix: aPrefix) ~= aUri]]! !

!XMLElement methodsFor: 'namespaces' stamp: 'JAAyer 6/20/2010 23:08'!
defaultNamespace
	^ self namespaceScope defaultNamespace! !

!XMLElement methodsFor: 'namespaces' stamp: 'JAAyer 11/24/2010 12:23'!
defaultNamespace: aUri
	| oldNamespace |

	oldNamespace := self defaultNamespace.
	self
		updateEnclosedScopesWith: [:scope |
			scope defaultNamespace:
				(aUri
					ifEmpty: [scope == self namespaceScope ifTrue: [''] ifFalse: [nil]]
					ifNotEmpty: [aUri])]
		whileTrue: [:scope |
			scope hasDefaultNamespace not
				or: [scope defaultNamespace = oldNamespace]]! !

!XMLElement methodsFor: 'accessing' stamp: 'JAAyer 6/12/2010 10:46'!
document
	^ document! !

!XMLElement methodsFor: 'accessing' stamp: 'JAAyer 11/26/2010 18:19'!
document: aDocument
	document := aDocument.

	self hasElements
		ifTrue: [self elementsDo: [:each | each document: aDocument]]! !

!XMLElement methodsFor: 'namespaces' stamp: 'JAAyer 4/12/2010 00:37'!
expandedName
	^ self nodeName expandWith: self namespaceURI! !

!XMLElement methodsFor: 'testing' stamp: 'JAAyer 3/11/2010 23:19'!
hasAttributes
	^ self attributeNodes notEmpty! !

!XMLElement methodsFor: 'testing' stamp: 'JAAyer 8/9/2010 20:32'!
hasContents
	self deprecated: 'use #hasStringNodes instead'.

	^ self hasStringNodes.! !

!XMLElement methodsFor: 'private' stamp: 'JAAyer 10/16/2010 06:07'!
hasNamespaces
	^ namespaceScope notNil and: [namespaceScope hasNamespaces]! !

!XMLElement methodsFor: 'namespaces' stamp: 'JAAyer 6/26/2010 07:31'!
hasPrefix
	^ self nodeName isPrefixed! !

!XMLElement methodsFor: 'testing' stamp: 'JAAyer 8/10/2010 04:11'!
hasStringNodes
	^ self nodes anySatisfy: [:each | each isStringNode]! !

!XMLElement methodsFor: 'testing' stamp: 'JAAyer 12/6/2010 21:23'!
includesAttribute: aName
	^ self attributeNodes includesAttribute: aName.! !

!XMLElement methodsFor: 'private' stamp: 'JAAyer 11/25/2010 10:39'!
inheritNamespacesFrom: anElement
	self allElementsDo: [:each |
		each namespaceScope inheritMappingsFrom: anElement namespaceScope]! !

!XMLElement methodsFor: 'private' stamp: 'JAAyer 11/27/2010 06:38'!
innerXMLParserOn: aStringOrStream
	^ (super innerXMLParserOn: aStringOrStream)
		currentScope: self namespaceScope! !

!XMLElement methodsFor: 'namespaces' stamp: 'JAAyer 11/24/2010 12:54'!
isDeclaredPrefix: aPrefix
	^ self namespaceScope includesPrefix: aPrefix! !

!XMLElement methodsFor: 'namespaces' stamp: 'JAAyer 11/24/2010 12:54'!
isDeclaredPrefix: aPrefix uri: aUri
	^ self namespaceScope includesPrefix: aPrefix mappedTo: aUri! !

!XMLElement methodsFor: 'testing' stamp: 'JAAyer 4/11/2010 22:59'!
isElement
	^ true! !

!XMLElement methodsFor: 'testing' stamp: 'JAAyer 11/26/2010 12:04'!
isNamed: aName
	"Does the receiver's qualified or local name match the argument?'"
	^ self nodeName isSameAs: aName! !

!XMLElement methodsFor: 'testing' stamp: 'JAAyer 11/26/2010 12:04'!
isNamedAny: aNameCollection
	"Does the receiver's qualified or local name match any of the names supplied as the argument?'"
	^ aNameCollection anySatisfy: [:each | self isNamed: each]! !

!XMLElement methodsFor: 'testing' stamp: 'JAAyer 11/26/2010 12:03'!
isRoot
	"Is the receiver the root element?"
	^ (self hasParent and: [self parent isElement]) not! !

!XMLElement methodsFor: 'accessing' stamp: 'JAAyer 6/27/2010 15:16'!
localName
	^ self nodeName localPart! !

!XMLElement methodsFor: 'accessing' stamp: 'JAAyer 7/12/2010 00:14'!
name
	^ self nodeName qualified! !

!XMLElement methodsFor: 'accessing' stamp: 'JAAyer 6/27/2010 15:19'!
name: aName
	self nodeName qualified: aName! !

!XMLElement methodsFor: 'namespaces' stamp: 'JAAyer 11/23/2010 13:40'!
name: aQualifiedName namespaceURI: aUri
	aQualifiedName splitQualifiedNameInto: [:prefix :localPart | self declarePrefix: prefix uri: aUri].
	self name: aQualifiedName.! !

!XMLElement methodsFor: 'private' stamp: 'JAAyer 6/20/2010 23:10'!
namespaceScope
	^ namespaceScope ifNil: [namespaceScope := XMLNamespaceScope new]! !

!XMLElement methodsFor: 'namespaces' stamp: 'JAAyer 3/14/2010 12:52'!
namespaceURI
	^ self resolvePrefix: self prefix! !

!XMLElement methodsFor: 'accessing' stamp: 'JAAyer 10/30/2010 19:56'!
nextElement
	^ self parent ifNotNil: [self parent elementAfter: self]! !

!XMLElement methodsFor: 'private' stamp: 'OttoBehrens 2/22/2011 23:42'!
nodeName
	^ name
		ifNil: [ 
			(name := XMLNodeName new)
				addObserver: self;
				yourself ]! !

!XMLElement methodsFor: 'notifications' stamp: 'JAAyer 7/11/2010 02:25'!
nodeNameChangedFrom: anOldName to: aNewName
	self validateName.
	self
		notifyOf: #renamedElement:from:to:
		with: self
		with: anOldName
		with: aNewName! !

!XMLElement methodsFor: 'namespaces' stamp: 'JAAyer 3/17/2010 03:48'!
prefix
	^ self nodeName prefix! !

!XMLElement methodsFor: 'namespaces' stamp: 'JAAyer 6/15/2010 15:15'!
prefix: aPrefix
	self nodeName prefix: aPrefix! !

!XMLElement methodsFor: 'accessing' stamp: 'JAAyer 10/30/2010 19:57'!
previousElement
	^ self parent ifNotNil: [self parent elementBefore: self]! !

!XMLElement methodsFor: 'printing' stamp: 'JAAyer 11/24/2010 14:30'!
printNamespaceDeclarationsOn: aTagWriter
	(self declaresDefaultNamespace
		or: [self defaultNamespace notEmpty and: [aTagWriter isEmbedded not]])
		ifTrue: [aTagWriter xmlns: self defaultNamespace].

	self namespaceScope prefixesAndURIsDo: [:prefix :uri |
		((self declaresPrefix: prefix uri: uri) or: [aTagWriter isEmbedded not])
			ifTrue: [aTagWriter xmlnsAt: prefix put: uri]]! !

!XMLElement methodsFor: 'accessing' stamp: 'JAAyer 10/19/2010 10:58'!
qualifiedName
	self deprecated: 'use #name instead'.

	^ self nodeName qualified! !

!XMLElement methodsFor: 'accessing' stamp: 'JAAyer 12/6/2010 21:24'!
removeAttribute: aName
	^ (self attributeNodes removeAttributeNodeAt: aName) value.! !

!XMLElement methodsFor: 'accessing' stamp: 'JAAyer 7/22/2010 08:54'!
removeAttributeNode: aNode
	^ self attributeNodes remove: aNode! !

!XMLElement methodsFor: 'accessing' stamp: 'JAAyer 11/28/2010 08:42'!
removeAttributes
	^ self attributeNodes removeAll! !

!XMLElement methodsFor: 'notifications' stamp: 'JAAyer 10/24/2010 18:08'!
removedAttribute: anAttribute
	anAttribute
		removeObserver: self;
		element: nil! !

!XMLElement methodsFor: 'notifications' stamp: 'JAAyer 8/6/2010 20:16'!
renamedAttribute: anAttribute from: anOldName to: aNewName
	self validateAttributeName: aNewName! !

!XMLElement methodsFor: 'namespaces' stamp: 'JAAyer 6/20/2010 23:08'!
resolvePrefix: aPrefix
	^ self namespaceScope resolvePrefix: aPrefix! !

!XMLElement methodsFor: 'initialization' stamp: 'JAAyer 7/12/2010 14:40'!
setAttributes: aDictionaryOrArray
	self attributeNodes removeAll.
	aDictionaryOrArray associationsDo: [:each |
		self attributeNodes
			addName: each key
			value: each value].! !

!XMLElement methodsFor: 'initialization' stamp: 'JAAyer 10/24/2010 18:19'!
setQualifiedName: aQualifiedName prefix: aPrefix localName: aLocalName attributes: anAttributeDictionary namespaceScope: aNamespaceScope
	self nodeName
		setQualified: aQualifiedName
		prefix: aPrefix
		localPart: aLocalName.

	anAttributeDictionary
		ifNotEmpty: [
			"By not observing 'attributes' until after it is filled, we avoid
			re-validating the attributes, which the parser should have already done"
			attributes := XMLAttributeList new: anAttributeDictionary size.
			anAttributeDictionary keysAndValuesDo: [:key :value |
				(attributes addName: key value: value)
					addObserver: self;
					element: self].
			attributes addObserver: self].

	namespaceScope := aNamespaceScope.! !

!XMLElement methodsFor: 'accessing' stamp: 'JAAyer 7/22/2010 08:08'!
stringNodes
	^ self nodes select: [:each | each isStringNode]! !

!XMLElement methodsFor: 'enumerating' stamp: 'JAAyer 7/22/2010 08:08'!
stringNodesDo: aBlock
	self nodes do: [:each | each isStringNode ifTrue: [aBlock value: each]]! !

!XMLElement methodsFor: 'accessing' stamp: 'JAAyer 8/9/2010 01:44'!
strings
	^ self stringNodes collect: [:each | each string]! !

!XMLElement methodsFor: 'enumerating' stamp: 'JAAyer 8/9/2010 01:44'!
stringsDo: aBlock
	self stringNodesDo: [:each | aBlock value: each string]! !

!XMLElement methodsFor: 'accessing' stamp: 'JAAyer 10/19/2010 10:57'!
tag
	self deprecated: 'use #name instead'.

	^ self name.! !

!XMLElement methodsFor: 'namespaces' stamp: 'JAAyer 11/23/2010 13:36'!
undeclarePrefix: aPrefix
	self namespaceScope unmapPrefix: aPrefix! !

!XMLElement methodsFor: 'private' stamp: 'JAAyer 11/24/2010 12:23'!
updateEnclosedScopesWith: aBlock whileTrue: aConditionalBlock
	"Updates the receiver's scope and the scopes of all descendants with aBlock
	where aConditional is true. It stops descending down a path of a subtree
	as soon as aConditional is false, but continues down other paths."

	(aConditionalBlock value: self namespaceScope)
		ifTrue: [aBlock value: self namespaceScope]
		ifFalse: [^ self].

	self elementsDo: [:each |
		each
			updateEnclosedScopesWith: aBlock
			whileTrue: aConditionalBlock]! !

!XMLElement methodsFor: 'private' stamp: 'JAAyer 8/6/2010 23:46'!
validateAttributeName: anAttributeName
	self usesNamespaces ifFalse: [^ self].

	self namespaceScope
		validatePrefixedAttributeName: anAttributeName
		in: self attributeNodes attributesByName.! !

!XMLElement methodsFor: 'private' stamp: 'JAAyer 8/7/2010 13:42'!
validateName
	(self usesNamespaces and: [self hasPrefix])
		ifTrue: [
			self namespaceScope
				validatePrefix: self prefix
				ofElementName: self name]! !

!XMLElement methodsFor: 'printing' stamp: 'JAA 3/23/2011 09:42'!
writeXMLOn: aWriter
	| tagWriter |

	(tagWriter := aWriter tag)
		name: self name.

	self hasNamespaces
		ifTrue: [self printNamespaceDeclarationsOn: tagWriter].

	tagWriter
		attributes: self attributes;
		with: [self writeInnerXMLOn: aWriter].! !

!XMLNodeWithElements methodsFor: 'accessing' stamp: 'JAAyer 2/25/2010 14:37'!
addElement: anElement
	self addNode: anElement! !

!XMLNodeWithElements methodsFor: 'notifications' stamp: 'JAAyer 11/24/2010 07:16'!
addedElement: anElement
	anElement
		configuration: self configuration;
		document: self document! !

!XMLNodeWithElements methodsFor: 'accessing' stamp: 'JAAyer 12/4/2010 12:32'!
allElements
	^ self allElementsCollect: [:each | each]! !

!XMLNodeWithElements methodsFor: 'enumerating' stamp: 'JAAyer 11/28/2010 12:21'!
allElementsCollect: aBlock
	| collectedElements |

	collectedElements := self elements copyEmpty.
	self allElementsDo: [:each | collectedElements add: (aBlock value: each)].
	^ collectedElements.! !

!XMLNodeWithElements methodsFor: 'enumerating' stamp: 'JAAyer 11/25/2010 10:47'!
allElementsDetect: aBlock
	"Descend depth-first visiting each element with aBlock until one such evaluation
	of aBlock with an element is true, then return that element. If no evaluation is
	true, then return nil."

	^ self allElementsDetect: aBlock ifNone: [nil]! !

!XMLNodeWithElements methodsFor: 'enumerating' stamp: 'JAAyer 11/25/2010 14:20'!
allElementsDetect: aBlock ifNone: anExceptionBlock
	"Descend depth-first visiting each element with aBlock until one such evaluation
	of aBlock with an element is true, then return that element. If no evaluation is
	true, then return nil."

	self allElementsDo: [:each | (aBlock value: each) ifTrue: [^ each]].
	^ anExceptionBlock value.! !

!XMLNodeWithElements methodsFor: 'enumerating' stamp: 'JAAyer 11/25/2010 10:37'!
allElementsDo: aBlock
	"Descend depth-first visiting each element with aBlock."

	self hasElements
		ifTrue: [self elementsDo: [:each | each allElementsDo: aBlock]]! !

!XMLNodeWithElements methodsFor: 'accessing' stamp: 'JAAyer 11/25/2010 13:58'!
allElementsNamed: aName
	^ self allElementsSelect: [:each | each isNamed: aName]! !

!XMLNodeWithElements methodsFor: 'enumerating' stamp: 'JAAyer 11/25/2010 13:17'!
allElementsNamed: aName do: aBlock
	self allElementsDo: [:each |
		(each isNamed: aName)
			ifTrue: [aBlock value: each]]! !

!XMLNodeWithElements methodsFor: 'accessing' stamp: 'JAAyer 11/25/2010 13:30'!
allElementsNamed: aName namespaceURI: aUri
	^ (self allElementsNamed: aName) select: [:each | each namespaceURI = aUri]! !

!XMLNodeWithElements methodsFor: 'enumerating' stamp: 'JAAyer 11/28/2010 13:12'!
allElementsSelect: aBlock
	| selectedElements |

	selectedElements := self elements copyEmpty.
	self allElementsDo: [:each |
		(aBlock value: each)
			ifTrue: [selectedElements add: each]].
	^ selectedElements.! !

!XMLNodeWithElements methodsFor: 'accessing' stamp: 'JAAyer 11/23/2010 14:49'!
configuration
	^ configuration ifNil: [
		self configuration: XMLDOMConfiguration new.
		configuration]! !

!XMLNodeWithElements methodsFor: 'accessing' stamp: 'JAAyer 11/26/2010 18:19'!
configuration: aConfiguration
	configuration := aConfiguration.

	self hasElements
		ifTrue: [self elementsDo: [:each | each configuration: aConfiguration]]! !

!XMLNodeWithElements methodsFor: 'accessing' stamp: 'JAAyer 7/22/2010 08:09'!
contentNodes
	^ self nodes select: [:each | each isContentNode]! !

!XMLNodeWithElements methodsFor: 'enumerating' stamp: 'JAAyer 7/22/2010 08:09'!
contentNodesDo: aBlock
	self nodes do: [:each | each isContentNode ifTrue: [aBlock value: each]]! !

!XMLNodeWithElements methodsFor: 'accessing' stamp: 'JAAyer 10/24/2010 16:14'!
contentStringAt: aName
	^ (self elementAt: aName ifAbsent: [^ '']) contentString! !

!XMLNodeWithElements methodsFor: 'accessing' stamp: 'JAAyer 7/12/2010 03:19'!
document
	self subclassResponsibility! !

!XMLNodeWithElements methodsFor: 'accessing' stamp: 'JAAyer 11/24/2010 09:26'!
document: aDocument 
	self subclassResponsibility! !

!XMLNodeWithElements methodsFor: 'accessing' stamp: 'JAAyer 3/12/2010 11:45'!
elementAfter: anElement
	^ self elements after: anElement! !

!XMLNodeWithElements methodsFor: 'accessing' stamp: 'JAAyer 11/26/2010 12:31'!
elementAt: aName
	^ self nodes elementAt: aName! !

!XMLNodeWithElements methodsFor: 'accessing' stamp: 'JAAyer 11/26/2010 12:32'!
elementAt: aName ifAbsent: aBlock
	^ self nodes elementAt: aName ifAbsent: aBlock! !

!XMLNodeWithElements methodsFor: 'accessing' stamp: 'JAAyer 11/26/2010 12:32'!
elementAt: aName namespaceURI: aUri
	^ self nodes elementAt: aName namespaceURI: aUri! !

!XMLNodeWithElements methodsFor: 'accessing' stamp: 'JAAyer 11/26/2010 12:32'!
elementAt: aName namespaceURI: aUri ifAbsent: aBlock
	^ self nodes elementAt: aName namespaceURI: aUri ifAbsent: aBlock! !

!XMLNodeWithElements methodsFor: 'accessing' stamp: 'JAAyer 3/12/2010 11:45'!
elementBefore: anElement
	^ self elements before: anElement! !

!XMLNodeWithElements methodsFor: 'accessing' stamp: 'JAAyer 5/4/2010 13:18'!
elements
	^ self nodes elements! !

!XMLNodeWithElements methodsFor: 'accessing' stamp: 'JAAyer 11/25/2010 13:27'!
elementsAndContents
	self deprecated: 'use #contentNodes instead'.

	^ self contentNodes.! !

!XMLNodeWithElements methodsFor: 'enumerating' stamp: 'JAAyer 11/25/2010 13:27'!
elementsAndContentsDo: aBlock
	self deprecated: 'use contentNodesDo: instead'.

	self contentNodesDo: aBlock.! !

!XMLNodeWithElements methodsFor: 'accessing' stamp: 'JAAyer 11/26/2010 12:32'!
elementsAt: aName
	^ self nodes elementsAt: aName! !

!XMLNodeWithElements methodsFor: 'enumerating' stamp: 'JAAyer 11/26/2010 12:32'!
elementsAt: aName do: aBlock
	(self nodes elementsAt: aName) do: aBlock! !

!XMLNodeWithElements methodsFor: 'accessing' stamp: 'JAAyer 11/26/2010 12:33'!
elementsAt: aName namespaceURI: aUri
	^ self nodes elementsAt: aName namespaceURI: aUri! !

!XMLNodeWithElements methodsFor: 'enumerating' stamp: 'JAAyer 11/26/2010 12:35'!
elementsCollect: aBlock
	^ self nodes elementsCollect: aBlock! !

!XMLNodeWithElements methodsFor: 'enumerating' stamp: 'JAAyer 11/26/2010 12:35'!
elementsDetect: aBlock
	^ self nodes elementsDetect: aBlock! !

!XMLNodeWithElements methodsFor: 'enumerating' stamp: 'JAAyer 11/26/2010 12:35'!
elementsDetect: aBlock ifNone: anExceptionBlock
	^ self nodes elementsDetect: aBlock ifNone: anExceptionBlock! !

!XMLNodeWithElements methodsFor: 'enumerating' stamp: 'JAAyer 6/26/2010 00:55'!
elementsDo: aBlock
	self nodes elementsDo: aBlock! !

!XMLNodeWithElements methodsFor: 'enumerating' stamp: 'JAAyer 11/26/2010 12:36'!
elementsSelect: aBlock
	^ self nodes elementsSelect: aBlock! !

!XMLNodeWithElements methodsFor: 'searching' stamp: 'JAAyer 11/25/2010 17:24'!
findElementNamed: aName namespaceURI: aUri
	^ self allElementsDetect: [:each | (each isNamed: aName) and: [each namespaceURI = aUri]]! !

!XMLNodeWithElements methodsFor: 'searching' stamp: 'JAAyer 11/25/2010 17:24'!
findElementNamed: aName namespaceURI: aUri with: aBlock
	^ self allElementsDetect: [:each |
			(each isNamed: aName)
				and: [each namespaceURI = aUri
					and: [aBlock value: each]]]! !

!XMLNodeWithElements methodsFor: 'searching' stamp: 'JAAyer 11/25/2010 13:48'!
findElementNamed: aName with: aBlock
	^ self allElementsDetect: [:each | (each isNamed: aName) and: [aBlock value: each]]! !

!XMLNodeWithElements methodsFor: 'accessing' stamp: 'OttoBehrens 2/16/2011 17:35'!
firstElement
	^ self elements ifNotEmpty: [:elements | elements first] ifEmpty: [nil]! !

!XMLNodeWithElements methodsFor: 'searching' stamp: 'MohammadHoussami 7/12/2013 14:30'!
firstTagNamed: aName
	self deprecated: 'use #findElementNamed: instead'.

	^ self hasElementNamed: aName.! !

!XMLNodeWithElements methodsFor: 'searching' stamp: 'JAAyer 11/25/2010 13:51'!
firstTagNamed: aName with: aBlock
	self deprecated: 'use #findElementNamed:with: instead'.

	^ self findElementNamed: aName with: aBlock.! !

!XMLNodeWithElements methodsFor: 'testing' stamp: 'JAAyer 10/19/2010 11:13'!
hasContentNodes
	^ self nodes anySatisfy: [:each | each isContentNode]! !

!XMLNodeWithElements methodsFor: 'searching' stamp: 'MohammadHoussami 7/12/2013 14:30'!
hasElementNamed: aName
	^ self allElementsDetect: [:each | each isNamed: aName]! !

!XMLNodeWithElements methodsFor: 'testing' stamp: 'JAAyer 11/24/2010 09:27'!
hasElements
	"Faster version used when traversing to avoid initializing the node list
	in leaf elements"
	^ nodes notNil and: [nodes hasElements]! !

!XMLNodeWithElements methodsFor: 'testing' stamp: 'JAAyer 12/6/2010 21:21'!
includesElement: aName
	^ self nodes includesElementAt: aName! !

!XMLNodeWithElements methodsFor: 'private' stamp: 'JAAyer 11/27/2010 09:19'!
innerXMLParserOn: aStringOrStream
	^ (self configuration parserClass on: aStringOrStream withRootNode: self)
		configuration: self configuration.! !

!XMLNodeWithElements methodsFor: 'testing' stamp: 'JAAyer 7/20/2010 10:15'!
isContentNode
	^ true! !

!XMLNodeWithElements methodsFor: 'accessing' stamp: 'OttoBehrens 2/16/2011 17:36'!
lastElement
	^ self elements ifNotEmpty: [:elements | elements last] ifEmpty: [nil]! !

!XMLNodeWithElements methodsFor: 'private' stamp: 'JAAyer 8/7/2010 20:11'!
namespaceScope
	^ XMLNamespaceScope new! !

!XMLNodeWithElements methodsFor: 'private' stamp: 'JAAyer 7/20/2010 12:31'!
nodesClass
	^ XMLNodeList! !

!XMLNodeWithElements methodsFor: 'printing' stamp: 'JAAyer 5/5/2010 15:38'!
printContentOn: aStream
	self contentNodesDo: [:each | each printContentOn: aStream]! !

!XMLNodeWithElements methodsFor: 'notifications' stamp: 'JAAyer 7/21/2010 12:01'!
removedElement: anElement
	anElement document: nil! !

!XMLNodeWithElements methodsFor: 'notifications' stamp: 'JAAyer 6/22/2010 22:41'!
renamedElement: anElement from: anOldName to: aNewName! !

!XMLNodeWithElements methodsFor: 'enumerating' stamp: 'JAAyer 11/25/2010 14:01'!
tagsNamed: aName childrenDo: aOneArgumentBlock
	self deprecated: 'use #elementsAt:do: instead'.

	self elementsAt: aName do: aOneArgumentBlock.! !

!XMLNodeWithElements methodsFor: 'enumerating' stamp: 'JAAyer 11/25/2010 17:13'!
tagsNamed: aName childrenDoAndRecurse: aOneArgumentBlock
	self deprecated: 'use "node elementsDo: [:each | each allElementsNamed: aName do: [...]]" instead'.

	self elementsDo: [:each | each allElementsNamed: aName do: aOneArgumentBlock].! !

!XMLNodeWithElements methodsFor: 'enumerating' stamp: 'JAAyer 11/25/2010 14:18'!
tagsNamed: aName contentsDo: aBlock
	self deprecated: 'use "node allElementsNamed: aName do: [:each | each stringNodes...]" instead'.

	self allElementsNamed: aName do: [:each | each stringNodesDo: aBlock].! !

!XMLNodeWithElements methodsFor: 'enumerating' stamp: 'JAAyer 11/25/2010 13:17'!
tagsNamed: aName do: aBlock
	self deprecated: 'use #allElementsNamed:do: instead'.

	self allElementsNamed: aName do: aBlock.! !

!XMLNodeWithElements methodsFor: 'accessing' stamp: 'JAAyer 7/12/2010 03:23'!
topElement
	self deprecated: 'use #firstElement instead'.

	^ self firstElement.! !

!XMLNodeWithElements methodsFor: 'testing' stamp: 'JAAyer 11/23/2010 14:48'!
usesNamespaces
	"Uses direct access and isNil test for performance"
	^ configuration isNil or: [configuration usesNamespaces]! !

!XMLNodeWithElements methodsFor: 'accessing' stamp: 'JAAyer 11/28/2010 12:59'!
usesNamespaces: aBoolean
	self configuration usesNamespaces: aBoolean! !

XMLNode subclass: #XMLPI
	instanceVariableNames: 'target data'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-Nodes'!

!XMLPI commentStamp: 'JAAyer 6/25/2010 14:06' prior: 0!
This class represents a processing instruction.
!

!XMLPI class methodsFor: 'instance creation' stamp: 'JAAyer 2/25/2010 01:48'!
target: targetName data: aString
	^ self new
		target: targetName;
		data: aString! !

!XMLPI methodsFor: 'accessing' stamp: 'JAAyer 7/22/2010 03:02'!
data
	^ data ifNil: [data := '']! !

!XMLPI methodsFor: 'accessing' stamp: 'mir 1/17/2002 13:02'!
data: aString
	data := aString! !

!XMLPI methodsFor: 'testing' stamp: 'JAAyer 4/13/2010 21:43'!
isPI
	^ true! !

!XMLPI methodsFor: 'accessing' stamp: 'JAAyer 7/21/2010 12:42'!
target
	^ target ifNil: [target := '']! !

!XMLPI methodsFor: 'accessing' stamp: 'mir 1/17/2002 13:02'!
target: aString
	target := aString! !

!XMLPI methodsFor: 'printing' stamp: 'JAA 3/22/2011 15:15'!
writeXMLOn: aWriter
	aWriter pi
		target: self target;
		data: self data! !

XMLNode subclass: #XMLString
	instanceVariableNames: 'string'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-Nodes'!

!XMLString commentStamp: '<historical>' prior: 0!
This class represents string nodes, or nodes that contain no markup. String nodes can be created directly using the class-side #string: message or by sending a String object #asXMLString. When printing, they escape special characters in their contents.!

XMLString subclass: #XMLCData
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-Nodes'!

!XMLCData commentStamp: '<historical>' prior: 0!
This class represents a preserved CDATA section from a parsed document. It behaves like its superclass, except that its printed form differs in that it contains a "<!![CDATA[" prefix and a "]]>" suffix and its contents are NOT escaped. By default, CDATA sections are converted to regular string nodes during parsing. To preserve them as CDATA nodes, use #preservesCDataNodes: with the DOM parser before parsing.!

!XMLCData methodsFor: 'testing' stamp: 'JAAyer 8/11/2010 22:16'!
isCData
	^ true! !

!XMLCData methodsFor: 'printing' stamp: 'JAA 3/22/2011 15:15'!
writeXMLOn: aWriter
	aWriter cdata: self string! !

!XMLString class methodsFor: 'instance creation' stamp: 'JAAyer 2/25/2010 01:48'!
string: aString
	^ self new string: aString! !

!XMLString methodsFor: 'accessing' stamp: 'JAAyer 6/10/2010 04:54'!
addContent: aStringOrStringNode
	self string: self string, (self toString: aStringOrStringNode)! !

!XMLString methodsFor: 'converting' stamp: 'JAAyer 7/22/2010 05:19'!
asXMLString
	^ self! !

!XMLString methodsFor: 'testing' stamp: 'JAAyer 7/20/2010 10:26'!
isContentNode
	^ true! !

!XMLString methodsFor: 'testing' stamp: 'JAAyer 4/12/2010 22:35'!
isStringNode
	^ true! !

!XMLString methodsFor: 'printing' stamp: 'JAAyer 9/2/2010 13:25'!
printContentOn: aStream
	| lastChar |

	self string ifEmpty: [^ self].

	((lastChar := aStream peekLast) isNil
		or: [lastChar isSeparator
			or: [self string first isSeparator]])
		ifFalse: [aStream space].

	aStream nextPutAll: self string.! !

!XMLString methodsFor: 'accessing' stamp: 'JAAyer 7/22/2010 05:19'!
string
	^ string ifNil: [string := '']! !

!XMLString methodsFor: 'accessing' stamp: 'JAAyer 2/23/2010 02:55'!
string: aString
	string := self toString: aString! !

!XMLString methodsFor: 'private' stamp: 'JAAyer 10/28/2010 15:22'!
toString: aStringOrStringNode
	^ (aStringOrStringNode isKindOf: XMLString)
		ifTrue: [aStringOrStringNode string]
		ifFalse: [aStringOrStringNode]! !

!XMLString methodsFor: 'printing' stamp: 'JAA 3/22/2011 15:15'!
writeXMLOn: aWriter
	aWriter string: self string! !

Object subclass: #XMLNodeFactory
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-Nodes'!

!XMLNodeFactory commentStamp: '<historical>' prior: 0!
This class is the default node factory. Its accessor messages return the class objects used by the DOM parser to build a node tree. You can override these messages to return different class objects and inject an instance of your factory using the DOM parser's #nodeFactory: accessor before parsing.!

!XMLNodeFactory methodsFor: 'accessing' stamp: 'JAAyer 11/10/2010 03:18'!
cdataClass
	^ XMLCData! !

!XMLNodeFactory methodsFor: 'accessing' stamp: 'JAAyer 11/10/2010 03:20'!
classForElement: aQualifiedName prefix: aPrefix uri: aNamespaceUri localName: aLocalName
	^ self elementClass! !

!XMLNodeFactory methodsFor: 'accessing' stamp: 'JAAyer 11/10/2010 03:18'!
commentClass
	^ XMLComment! !

!XMLNodeFactory methodsFor: 'accessing' stamp: 'JAAyer 11/10/2010 03:19'!
doctypeDeclarationClass
	^ XMLDoctypeDeclaration! !

!XMLNodeFactory methodsFor: 'accessing' stamp: 'JAAyer 11/10/2010 03:19'!
documentClass
	^ XMLDocument! !

!XMLNodeFactory methodsFor: 'accessing' stamp: 'JAAyer 11/10/2010 03:18'!
elementClass
	^ XMLElement! !

!XMLNodeFactory methodsFor: 'accessing' stamp: 'JAAyer 11/10/2010 03:18'!
piClass
	^ XMLPI! !

!XMLNodeFactory methodsFor: 'accessing' stamp: 'JAAyer 11/10/2010 03:18'!
stringClass
	^ XMLString! !

XMLNodeFactory subclass: #XMLPluggableElementFactory
	instanceVariableNames: 'elementClass elementClassesByName'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-Nodes'!

!XMLPluggableElementFactory commentStamp: '<historical>' prior: 0!
This class is a pluggable factory that can map elements to different XMLElement subclasses based on the name and namespace information of those elements. You have to create an instance, configure it to handle certain elements with certain classes, and then inject your instance into a DOM parser using #nodeFactory: before parsing. Here is an example of its use:

	doc := (XMLDOMParser on: someXML)
		nodeFactory:
			(XMLPluggableElementFactory new
				elementClass: GenericElement;
				handleElement: 'user' withClass: UserElement;
				handleElement: 'report' withClass: ReportElement;
				handleElement: 'report' namespaceURI: 'urn:specialreprot' withClass: SpecialReportElement)
		parseDocument.
		
The #handleElementsMatchingClass*: forms try to match elements to the specified classes based on the names of those classes when stripped of any prefix (like XML) and "Element" suffix and converted to both camel case and a hyphenated form. So this:
	XMLPluggableElementFactory new
		handleElementsMatchingClass: MYRootUserElement
		....
will handle 'rootUser' and 'root-user' elements with the MYRootUserElement class.!

!XMLPluggableElementFactory methodsFor: 'accessing' stamp: 'JAAyer 11/14/2010 21:57'!
classForElement: aName prefix: aPrefix uri: aUri localName: aLocalName
	^ (self classForElement: aName uri: aUri)
		ifNil: [
			aPrefix
				ifEmpty: [self elementClass]
				ifNotEmpty: [
					(self classForElement: aLocalName uri: aUri)
						ifNil: [self elementClass]]]! !

!XMLPluggableElementFactory methodsFor: 'private' stamp: 'JAAyer 11/14/2010 21:55'!
classForElement: aName uri: aUri
	| classesForElement |

	classesForElement :=
		self elementClassesByName
			at: aName
			ifAbsent: [^ nil].

	^ classesForElement
		at: aUri
		ifAbsent: [
			classesForElement
				at: ''
				ifAbsent: [nil]].! !

!XMLPluggableElementFactory methodsFor: 'accessing' stamp: 'JAAyer 11/15/2010 05:05'!
elementClass
	^ elementClass ifNil: [elementClass := super elementClass]! !

!XMLPluggableElementFactory methodsFor: 'accessing' stamp: 'JAAyer 11/15/2010 05:04'!
elementClass: anElementClass
	elementClass := anElementClass! !

!XMLPluggableElementFactory methodsFor: 'private' stamp: 'JAAyer 11/13/2010 08:36'!
elementClassesByName
	^ elementClassesByName ifNil: [elementClassesByName := Dictionary new]! !

!XMLPluggableElementFactory methodsFor: 'element handling' stamp: 'JAAyer 11/13/2010 19:37'!
handleElement: anElementName namespaceURI: aUri withClass: aClass
	(self elementClassesByName
		at: anElementName
		ifAbsentPut: [Dictionary new])
			at: aUri put: aClass! !

!XMLPluggableElementFactory methodsFor: 'element handling' stamp: 'JAAyer 11/13/2010 19:37'!
handleElement: anElementName withClass: aClass
	self
		handleElement: anElementName
		namespaceURI: ''
		withClass: aClass! !

!XMLPluggableElementFactory methodsFor: 'element handling' stamp: 'JAAyer 11/20/2010 11:39'!
handleElements: anElementNameCollection namespaceURI: aNamespaceURI withClass: aClass
	anElementNameCollection do: [:each |
		self
			handleElement: each
			namespaceURI: aNamespaceURI
			withClass: aClass]! !

!XMLPluggableElementFactory methodsFor: 'element handling' stamp: 'JAAyer 11/20/2010 11:36'!
handleElements: anElementNameCollection withClass: aClass
	anElementNameCollection do: [:each | self handleElement: each withClass: aClass]! !

!XMLPluggableElementFactory methodsFor: 'element handling' stamp: 'JAAyer 11/22/2010 11:10'!
handleElementsMatchingClass: aClass
	| className |

	(className := self toNameWithoutPrefixAndElementSuffix: aClass name asString)
		ifEmpty: [^ self].

	self
		handleElement: (self toCamelCasedName: className)
		withClass: aClass.
	self
		handleElement: (self toHyphenatedName: className)
		withClass: aClass.! !

!XMLPluggableElementFactory methodsFor: 'element handling' stamp: 'JAAyer 11/20/2010 11:26'!
handleElementsMatchingClasses: aClassCollection
	aClassCollection do: [:each | self handleElementsMatchingClass: each]! !

!XMLPluggableElementFactory methodsFor: 'private' stamp: 'JAAyer 11/20/2010 11:29'!
toCamelCasedName: aString
	^ aString allButFirst copyWithFirst: aString first asLowercase! !

!XMLPluggableElementFactory methodsFor: 'private' stamp: 'JAAyer 11/20/2010 11:34'!
toHyphenatedName: aString
	| nameStream hyphenatedStream previousChar nextChar |

	nameStream := aString readStream.
	hyphenatedStream := (String new: nameStream size) writeStream.
	[nameStream atEnd]
		whileFalse: [
			((nextChar := nameStream next) isUppercase
				and: [previousChar notNil and: [previousChar isLowercase]])
				ifTrue: [hyphenatedStream nextPut: $-].
			hyphenatedStream nextPut: nextChar asLowercase.
			previousChar := nextChar].
	^ hyphenatedStream contents.! !

!XMLPluggableElementFactory methodsFor: 'private' stamp: 'JAAyer 11/22/2010 11:10'!
toNameWithoutPrefixAndElementSuffix: aString
	| nameStream |

	nameStream := aString readStream.
	[nameStream atEnd not and: [nameStream peek isUppercase]]
		whileTrue: [nameStream next].
	nameStream atEnd
		ifTrue: [^ ''].
	nameStream back.

	^ nameStream upToEnd copyReplaceAll: 'Element' with: ''.! !

Object subclass: #XMLNodeName
	instanceVariableNames: 'qualified prefix localPart notifier'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-Nodes'!

!XMLNodeName commentStamp: '<historical>' prior: 0!
The class represents a name for an element or attribute. It stores the prefix, local part and qualified name. It also notifies any observers whenever any part of the name it models changes using #nodeNameChangedFrom:to:. You should not need to use this class directly.!

!XMLNodeName class methodsFor: 'utility' stamp: 'OttoBehrens 2/15/2011 08:18'!
notifications
	| notifications parser parsedMethods |
	notifications := WriteStream on: (Array new: 10).
	parser := Parser new.
	parsedMethods := IdentityDictionary new.
	(self allMethodsInCategory: #notification)
		do: [ :notifierSelector | 
			| methodNode |
			(SystemNavigation default allSendersOf: notifierSelector inClass: self)
				do: [ :methodReference | 
					methodNode := parsedMethods
						at: methodReference selector
						ifAbsentPut: (parser parse: methodReference sourceCode class: self).
					methodNode
						nodesDo: [ :node | 
							| firstArgument |
							(node isMessage
								and: [ node selector key asSymbol = notifierSelector and: [ (firstArgument := node arguments first) isLiteral ] ])
								ifTrue: [ notifications nextPut: firstArgument key asSymbol ] ] ] ].
	^ notifications contents! !

!XMLNodeName methodsFor: 'observing' stamp: 'OttoBehrens 2/15/2011 08:18'!
addObserver: anObserver
	self notifier addObserver: anObserver! !

!XMLNodeName methodsFor: 'private' stamp: 'JAAyer 11/1/2010 04:36'!
changeWith: aBlock
	| oldName newName |

	oldName := self qualified.
	aBlock value.
	newName := self qualified.
	newName = oldName
		ifFalse: [self notifyOf: #nodeNameChangedFrom:to: with: oldName with: newName].! !

!XMLNodeName methodsFor: 'accessing' stamp: 'JAAyer 8/9/2010 15:01'!
expandWith: aUri
	^ aUri
		ifEmpty: [self localPart]
		ifNotEmpty: ['{', aUri, '}', self localPart]! !

!XMLNodeName methodsFor: 'testing' stamp: 'JAAyer 8/9/2010 14:46'!
isPrefixed
	^ self prefix notEmpty! !

!XMLNodeName methodsFor: 'testing' stamp: 'JAAyer 10/24/2010 18:00'!
isSameAs: aNameString
	^ self qualified = aNameString or: [self localPart = aNameString]! !

!XMLNodeName methodsFor: 'accessing' stamp: 'JAAyer 7/26/2010 14:23'!
localPart
	^ localPart ifNil: [localPart := '']! !

!XMLNodeName methodsFor: 'accessing' stamp: 'JAAyer 11/21/2010 04:57'!
localPart: aLocalPart
	self changeWith: [self setPrefix: self prefix localPart: aLocalPart]! !

!XMLNodeName methodsFor: 'accessing' stamp: 'JAAyer 6/15/2010 15:18'!
notifier
	^ notifier ifNil: [notifier := XMLNotifier new]! !

!XMLNodeName methodsFor: 'notification' stamp: 'OttoBehrens 2/15/2011 08:18'!
notifyOf: aSelector with: anArgument
	self notifier notifyOf: aSelector with: anArgument! !

!XMLNodeName methodsFor: 'notification' stamp: 'OttoBehrens 2/15/2011 08:18'!
notifyOf: aSelector with: aFirstArgument with: aSecondArgument
	self notifier notifyOf: aSelector with: aFirstArgument with: aSecondArgument! !

!XMLNodeName methodsFor: 'notification' stamp: 'OttoBehrens 2/15/2011 08:18'!
notifyOf: aSelector with: aFirstArgument with: aSecondArgument with: aThirdArgument
	self notifier
		notifyOf: aSelector
		with: aFirstArgument
		with: aSecondArgument
		with: aThirdArgument! !

!XMLNodeName methodsFor: 'notification' stamp: 'OttoBehrens 2/15/2011 08:18'!
notifyOf: aSelector withArguments: anArray
	self notifier notifyOf: aSelector withArguments: anArray! !

!XMLNodeName methodsFor: 'accessing' stamp: 'JAAyer 8/9/2010 14:43'!
prefix
	^ prefix ifNil: [prefix := '']! !

!XMLNodeName methodsFor: 'accessing' stamp: 'JAAyer 6/27/2010 15:17'!
prefix: aPrefix
	self changeWith: [self setPrefix: aPrefix localPart: self localPart]! !

!XMLNodeName methodsFor: 'printing' stamp: 'JAAyer 6/27/2010 15:15'!
printOn: aStream
	aStream nextPutAll: self qualified! !

!XMLNodeName methodsFor: 'accessing' stamp: 'JAAyer 7/26/2010 14:23'!
qualified
	^ qualified ifNil: [qualified := '']! !

!XMLNodeName methodsFor: 'accessing' stamp: 'JAAyer 6/27/2010 15:19'!
qualified: aQualifiedName
	aQualifiedName splitQualifiedNameInto: [:prefixPart :localName |
		self changeWith: [self setPrefix: prefixPart localPart: localName]]! !

!XMLNodeName methodsFor: 'observing' stamp: 'OttoBehrens 2/15/2011 08:18'!
removeObserver: anObserver
	self notifier removeObserver: anObserver! !

!XMLNodeName methodsFor: 'private' stamp: 'JAAyer 8/14/2010 20:50'!
setPrefix: aPrefix localPart: aLocalPart
	aPrefix isEmptyOrNil
		ifTrue: [
			self
				setQualified: aLocalPart
				prefix: ''
				localPart: aLocalPart]
		ifFalse: [
			self
				setQualified: aPrefix, ':', aLocalPart
				prefix: aPrefix
				localPart: aLocalPart]! !

!XMLNodeName methodsFor: 'private' stamp: 'JAAyer 6/27/2010 15:20'!
setQualified: aQualifiedName prefix: aPrefix localPart: aLocalPart
	qualified := aQualifiedName.
	prefix := aPrefix.
	localPart := aLocalPart.! !

Object subclass: #XMLNotifier
	instanceVariableNames: 'observers'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-Nodes'!

!XMLNotifier commentStamp: '<historical>' prior: 0!
This class is used by objects using the TXMLObservable Trait to send notifications to objects that register themselves as observers. The notifications are selectors that observers are expected to implement, typically under the category "notifications."

It stores observers in an Array grown and shrunk with #copyWith: and #copyWithout: rather than an OrderedCollection for better memory usage and faster performance (because each node only has at most two observers).!

!XMLNotifier methodsFor: 'observing' stamp: 'JAAyer 7/20/2010 10:05'!
addObserver: anObject
	observers :=
		observers
			ifNil: [Array with: anObject]
			ifNotNil: [observers copyWith: anObject]! !

!XMLNotifier methodsFor: 'notification' stamp: 'JAAyer 7/20/2010 10:04'!
notifyOf: aSelector with: anArgument
	observers ifNotNil: [
		observers do: [:each |
			each perform: aSelector with: anArgument]]! !

!XMLNotifier methodsFor: 'notification' stamp: 'JAAyer 7/20/2010 10:04'!
notifyOf: aSelector with: aFirstArgument with: aSecondArgument
	observers ifNotNil: [
		observers do: [:each |
			each
				perform: aSelector
				with: aFirstArgument
				with: aSecondArgument]]! !

!XMLNotifier methodsFor: 'notification' stamp: 'JAAyer 7/20/2010 10:04'!
notifyOf: aSelector with: aFirstArgument with: aSecondArgument with: aThirdArgument
	observers ifNotNil: [
		observers do: [:each |
			each
				perform: aSelector
				with: aFirstArgument
				with: aSecondArgument
				with: aThirdArgument]]! !

!XMLNotifier methodsFor: 'notification' stamp: 'JAAyer 7/20/2010 10:05'!
notifyOf: aSelector withArguments: anArray
	observers ifNotNil: [
		observers do: [:each |
			each perform: aSelector withArguments: anArray]]! !

!XMLNotifier methodsFor: 'accessing' stamp: 'JAAyer 7/20/2010 10:02'!
observers
	^ observers ifNil: [#()]! !

!XMLNotifier methodsFor: 'observing' stamp: 'JAAyer 7/20/2010 10:03'!
removeObserver: anObject
	(observers := observers copyWithout: anObject)
		ifEmpty: [observers := nil]! !

Object subclass: #XMLParser
	instanceVariableNames: 'tokenizer driver'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser'!

!XMLParser commentStamp: '<historical>' prior: 0!
This class is a facade that parses XML using a driver and tokenizer. See SAXHandler and XMLDOMParser for ready-made front ends to it.!

!XMLParser class methodsFor: 'instance creation' stamp: 'JAAyer 11/28/2010 07:09'!
driver: aDriver on: aStringOrStream
	^ self new setDriver: aDriver tokenizer: (aDriver tokenizerOn: aStringOrStream)! !

!XMLParser methodsFor: 'testing' stamp: 'JAAyer 11/27/2010 07:17'!
atEnd
	^ self tokenizer atEnd! !

!XMLParser methodsFor: 'accessing' stamp: 'JAAyer 11/28/2010 07:07'!
driver
	^ driver! !

!XMLParser methodsFor: 'parsing' stamp: 'JAAyer 11/27/2010 07:15'!
parseDocument
	[self atEnd] whileFalse: [self parseToken]! !

!XMLParser methodsFor: 'parsing' stamp: 'JAAyer 11/27/2010 07:16'!
parseToken
	^ self tokenizer nextToken! !

!XMLParser methodsFor: 'accessing' stamp: 'JAAyer 11/28/2010 07:09'!
setDriver: aDriver tokenizer: aTokenizer
	driver := aDriver.
	tokenizer := aTokenizer.! !

!XMLParser methodsFor: 'accessing' stamp: 'JAAyer 11/27/2010 07:16'!
tokenizer
	^ tokenizer! !

Object subclass: #XMLParserDriver
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser'!

!XMLParserDriver commentStamp: '<historical>' prior: 0!
This is a base class for parser drivers. Drivers are sent messages by a tokenizer to handle various XML tokens as they are read. To create a driver, subclass this class and override the handler messages and testing messages that control tokenization.!

XMLParserDriver subclass: #SAXDriver
	instanceVariableNames: 'saxHandler configuration elementNester scopeAllocator languageEnvironment'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser'!

!SAXDriver commentStamp: '<historical>' prior: 0!
SAXDriver together with SAXHandler provides a SAX-like API for XML parsing. A SAXDriver processes XML tokens and signals SAX events to a SAXHandler. To use it, create a subclass of SAXHandler and override messages to handle SAX events of interest. See "instance creation" on the class-side of SAXHandler for ways to instantiate such a parser.!

!SAXDriver class methodsFor: 'instance creation' stamp: 'JAAyer 11/28/2010 13:18'!
saxHandler: aHandler
	^ self new
		setSaxHandler: aHandler
		configuration: aHandler configuration
		elementNester: aHandler elementNester
		scopeAllocator: aHandler scopeAllocator! !

!SAXDriver methodsFor: 'accessing' stamp: 'JAAyer 11/28/2010 11:44'!
configuration
	^ configuration! !

!SAXDriver methodsFor: 'accessing' stamp: 'JAAyer 11/27/2010 20:36'!
currentScope
	^ self usesNamespaces
		ifTrue: [self scopeAllocator currentScope]
		ifFalse: [nil]! !

!SAXDriver methodsFor: 'namespaces' stamp: 'JAAyer 11/27/2010 20:36'!
declareNamespaces: aDictionary
	aDictionary keysAndValuesDo: [:prefix :uri |
		self scopeAllocator declarePrefix: prefix uri: uri.
		self saxHandler startPrefixMapping: prefix uri: uri]! !

!SAXDriver methodsFor: 'accessing' stamp: 'JAAyer 11/28/2010 11:44'!
elementNester
	^ elementNester! !

!SAXDriver methodsFor: 'handling tokens' stamp: 'JAAyer 11/27/2010 07:22'!
handleCData: aString
	self saxHandler
		startCData;
		characters: aString;
		endCData! !

!SAXDriver methodsFor: 'handling tokens' stamp: 'JAAyer 11/27/2010 07:22'!
handleComment: aString
	self saxHandler 
		comment: aString! !

!SAXDriver methodsFor: 'handling tokens' stamp: 'JAAyer 11/28/2010 11:36'!
handleEndDocument
	self elementNester ensureNoElementsRemainOpen.
	self saxHandler endDocument! !

!SAXDriver methodsFor: 'handling tokens' stamp: 'JAA 3/21/2011 19:45'!
handleEndTag: anElementName
	| prefix localName namespaceUri |

	self elementNester endedElement: anElementName.

	anElementName splitQualifiedNameInto: [:prefixPart :localPart |
		prefix := prefixPart.
		localName := localPart].
	self usesNamespaces
		ifTrue: [namespaceUri := self scopeAllocator resolvePrefix: prefix].

	self saxHandler 
		endElement: anElementName
		prefix: prefix
		uri: namespaceUri
		localName: localName.

	self usesNamespaces
		ifTrue: [
			self undeclareNamespaces.
			self scopeAllocator exitScope].! !

!SAXDriver methodsFor: 'handling tokens' stamp: 'JAAyer 11/27/2010 07:22'!
handlePCData: aString
	self languageEnvironment
		ifNotNil: [aString applyLanguageInfomation: self languageEnvironment].
	self saxHandler 
		characters: aString! !

!SAXDriver methodsFor: 'handling tokens' stamp: 'JAAyer 11/27/2010 07:22'!
handlePI: piTarget data: piData
	self saxHandler 
		processingInstruction: piTarget data: piData! !

!SAXDriver methodsFor: 'handling tokens' stamp: 'mir 8/14/2000 18:29'!
handleStartDocument
	self saxHandler startDocument! !

!SAXDriver methodsFor: 'handling tokens' stamp: 'JAAyer 11/28/2010 11:36'!
handleStartTag: anElementName attributes: anAttributeDictionary namespaces: aNamespaceDictionary
	| prefix localName namespaceUri |

	self elementNester startedElement: anElementName.
	(anAttributeDictionary includesKey: 'xml:lang')
		ifTrue: [self languageEnvironment: (anAttributeDictionary at: 'xml:lang')].
	anElementName splitQualifiedNameInto: [:prefixPart :localPart |
		prefix := prefixPart.
		localName := localPart.
		namespaceUri := ''].

	self usesNamespaces ifTrue: [
		self scopeAllocator enterScope.
		self declareNamespaces: aNamespaceDictionary.
		namespaceUri := self scopeAllocator resolvePrefix: prefix.
		self
			validateStartTag: anElementName
			prefix: prefix
			attributes: anAttributeDictionary].

	self saxHandler 
		startElement: anElementName
		prefix: prefix
		uri: namespaceUri
		localName: localName
		attributes: anAttributeDictionary! !

!SAXDriver methodsFor: 'handling tokens' stamp: 'JAAyer 11/27/2010 07:22'!
handleWhitespace: aString
	self saxHandler 
		ignorableWhitespace: aString! !

!SAXDriver methodsFor: 'handling tokens' stamp: 'JAAyer 11/24/2010 13:29'!
handleXMLVersion: aVersion encoding: anEncoding standalone: aYesOrNo
	| versionNumber standaloneBoolean |

	aVersion
		ifNotEmpty: [versionNumber := aVersion asNumber].

	(#('yes' 'no') includes: aYesOrNo)
		ifTrue: [standaloneBoolean := aYesOrNo = 'yes'].

	self saxHandler
		xmlVersion: versionNumber
		encoding: anEncoding
		standalone: standaloneBoolean! !

!SAXDriver methodsFor: 'testing' stamp: 'JAAyer 11/28/2010 11:45'!
isValidating
	^ self configuration isValidating! !

!SAXDriver methodsFor: 'accessing' stamp: 'JAAyer 3/11/2010 07:21'!
languageEnvironment
	^ languageEnvironment! !

!SAXDriver methodsFor: 'accessing' stamp: 'JAAyer 3/2/2010 03:31'!
languageEnvironment: anIsoString
	languageEnvironment := LanguageEnvironment localeID: (LocaleID isoString: anIsoString)! !

!SAXDriver methodsFor: 'accessing' stamp: 'JAAyer 11/27/2010 08:00'!
saxHandler
	^ saxHandler! !

!SAXDriver methodsFor: 'accessing' stamp: 'JAAyer 11/28/2010 11:43'!
scopeAllocator
	^ scopeAllocator! !

!SAXDriver methodsFor: 'initialization' stamp: 'JAAyer 11/28/2010 13:17'!
setSaxHandler: aHandler configuration: aConfiguration elementNester: anElementNester scopeAllocator: aScopeAllocator
	saxHandler := aHandler.
	configuration := aConfiguration.
	elementNester := anElementNester.
	scopeAllocator := aScopeAllocator.! !

!SAXDriver methodsFor: 'namespaces' stamp: 'JAAyer 11/27/2010 20:36'!
undeclareNamespaces
	| currentScope enclosingScope |

	currentScope := self scopeAllocator currentScope.
	currentScope hasNamespaces
		ifFalse: [^ self].
	enclosingScope := self scopeAllocator enclosingScope.

	currentScope prefixesAndURIsDo: [:prefix :uri |
		(enclosingScope includesPrefix: prefix mappedTo: uri)
			ifFalse: [self saxHandler endPrefixMapping: prefix]].

	(currentScope defaultNamespace notNil
		and: [currentScope defaultNamespace ~= enclosingScope defaultNamespace])
		ifTrue: [self saxHandler endPrefixMapping: '']! !

!SAXDriver methodsFor: 'testing' stamp: 'JAAyer 11/28/2010 11:45'!
usesNamespaces
	^ self configuration usesNamespaces! !

!SAXDriver methodsFor: 'namespaces' stamp: 'JAAyer 11/27/2010 20:36'!
validateStartTag: aStartTag prefix: aPrefix attributes: anAttributeDictionary
	aPrefix ifNotEmpty: [
		self scopeAllocator
			validatePrefix: aPrefix
			ofElementName: aStartTag].

	self scopeAllocator validatePrefixedAttributeNames: anAttributeDictionary.! !

!XMLParserDriver methodsFor: 'handling tokens' stamp: 'JAAyer 11/27/2010 07:23'!
handleCData: aString! !

!XMLParserDriver methodsFor: 'handling tokens' stamp: 'JAAyer 11/27/2010 07:23'!
handleComment: aString! !

!XMLParserDriver methodsFor: 'handling tokens' stamp: 'JAAyer 11/27/2010 07:23'!
handleEndDocument! !

!XMLParserDriver methodsFor: 'handling tokens' stamp: 'JAAyer 11/27/2010 07:23'!
handleEndTag: anElementName! !

!XMLParserDriver methodsFor: 'handling tokens' stamp: 'JAAyer 11/27/2010 07:23'!
handlePCData: aString! !

!XMLParserDriver methodsFor: 'handling tokens' stamp: 'JAAyer 11/27/2010 07:23'!
handlePI: piTarget data: piData! !

!XMLParserDriver methodsFor: 'handling tokens' stamp: 'JAAyer 11/27/2010 07:23'!
handleStartDocument! !

!XMLParserDriver methodsFor: 'handling tokens' stamp: 'JAAyer 11/27/2010 07:23'!
handleStartTag: anElementName attributes: anAttributeDictionary namespaces: aNamespaceDictionary! !

!XMLParserDriver methodsFor: 'handling tokens' stamp: 'JAAyer 11/27/2010 07:23'!
handleWhitespace: aString! !

!XMLParserDriver methodsFor: 'handling tokens' stamp: 'JAAyer 11/27/2010 07:23'!
handleXMLVersion: aVersion encoding: anEncoding standalone: aYesOrNo! !

!XMLParserDriver methodsFor: 'testing' stamp: 'JAAyer 11/27/2010 07:24'!
isValidating
	self subclassResponsibility! !

!XMLParserDriver methodsFor: 'private' stamp: 'JAAyer 11/27/2010 07:42'!
tokenizerClass
	^ XMLTokenizer! !

!XMLParserDriver methodsFor: 'initialization' stamp: 'JAAyer 11/28/2010 07:16'!
tokenizerOn: aStringOrStream
	^ self tokenizerClass driver: self on: aStringOrStream! !

!XMLParserDriver methodsFor: 'testing' stamp: 'JAAyer 11/27/2010 07:24'!
usesNamespaces
	self subclassResponsibility! !

Object subclass: #XMLPeekableStreamAdapter
	instanceVariableNames: 'stream peekChar source'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser'!

!XMLPeekableStreamAdapter commentStamp: '<historical>' prior: 0!
This class wraps any readable stream that at the very least supports #next and #atEnd and makes it peekable. Additionally, it ensures that #next and #peek can be used at the end of a stream safely without raising an exception (the Squeak and Pharo behavior).!

!XMLPeekableStreamAdapter class methodsFor: 'instance creation' stamp: 'JAAyer 12/14/2010 02:23'!
on: aStream
	^ self on: aStream from: nil! !

!XMLPeekableStreamAdapter class methodsFor: 'instance creation' stamp: 'JAAyer 12/14/2010 02:12'!
on: aStream from: aSource
	^ self new setStream: aStream source: aSource! !

!XMLPeekableStreamAdapter methodsFor: 'testing' stamp: 'JAAyer 12/14/2010 02:48'!
atEnd
	^ peekChar isNil and: [stream atEnd]! !

!XMLPeekableStreamAdapter methodsFor: 'file open/close' stamp: 'JAAyer 12/14/2010 02:19'!
close
	stream close! !

!XMLPeekableStreamAdapter methodsFor: 'accessing' stamp: 'JAAyer 12/14/2010 02:34'!
converter: aConverter
	(stream respondsTo: #converter:)
		ifTrue: [stream converter: aConverter]! !

!XMLPeekableStreamAdapter methodsFor: 'accessing' stamp: 'JAAyer 12/14/2010 02:37'!
next
	| nextChar |

	peekChar
		ifNotNil: [
			nextChar := peekChar.
			peekChar := nil]
		ifNil: [
			nextChar := stream atEnd
				ifTrue: [nil]
				ifFalse: [stream next]].
	^ nextChar.! !

!XMLPeekableStreamAdapter methodsFor: 'reading' stamp: 'JAAyer 12/14/2010 02:18'!
peek
	^ peekChar
		ifNil: [
			stream atEnd
				ifFalse: [peekChar := stream next].
			peekChar]! !

!XMLPeekableStreamAdapter methodsFor: 'initialization' stamp: 'JAAyer 12/14/2010 02:13'!
setStream: aStream source: aSource
	stream := aStream.
	peekChar := nil.
	source := aSource.! !

!XMLPeekableStreamAdapter methodsFor: 'accessing' stamp: 'JAAyer 12/14/2010 02:22'!
source
	^ source ifNil: [source := '']! !

Object subclass: #XMLTokenizer
	instanceVariableNames: 'driver state streamReader streamWriter entities parameterEntities'
	classVariableNames: 'AdditionalNameChars CharEscapes LiteralChars NameStartChars'
	poolDictionaries: ''
	category: 'XML-Parser'!

!XMLTokenizer commentStamp: '<historical>' prior: 0!
This class reads XML tokens from a stream using a nested stream reader. Sending it #nextToken causes a token to be read and one or more handler messages to be dispatched to a driver.!

!XMLTokenizer class methodsFor: 'instance creation' stamp: 'JAAyer 11/28/2010 07:15'!
driver: aDriver on: aStringOrStream
	^self new
		setDriver: aDriver
		stream:
			(aStringOrStream isStream
				ifTrue: [aStringOrStream]
				ifFalse: [aStringOrStream readStream])! !

!XMLTokenizer class methodsFor: 'class initialization' stamp: 'JAAyer 1/20/2011 02:37'!
initialize
	"self initialize"

	CharEscapes := CharacterSet newFrom: #( $& $" $' $> $< ).
	(LiteralChars := CharacterSet new)
		addAll: #( $: $- $: $= $.);
		addAll: ($a to: $z);
		addAll: ($A to: $Z);
		addAll: ($0 to: $9).

	self
		initializeNameStartChars;
		initializeAdditionalNameChars.! !

!XMLTokenizer class methodsFor: 'class initialization' stamp: 'JAA 3/22/2011 12:43'!
initializeAdditionalNameChars
	"This creates a character set that contains all those defined in the XML spec for the
	NameChar production rule minus those also defined for NameStartChar
	(NameChar - NameStartChar)"

	(AdditionalNameChars := BitmapCharacterSet new: 16r2041)
		add: $-;
		add: $.;
		addAll: ($0 to: $9);
		add: (Character value: 16rB7).
	16r0300 to: 16r036F do: [:each | AdditionalNameChars add: (Character value: each)].
	16r203F to: 16r2040 do: [:each | AdditionalNameChars add: (Character value: each)].! !

!XMLTokenizer class methodsFor: 'class initialization' stamp: 'JAA 3/22/2011 12:43'!
initializeNameStartChars
	"This creates a character set that coveres the entire range of characters speicified for
	the NameStartChar production rule in the XML spec."

	(NameStartChars := BitmapCharacterSet new: 16rF0000)
		add: $:;
		addAll: ($A to: $Z);
		add: $_;
		addAll: ($a to: $z).
	16rC0 to: 16rD6 do: [:each | NameStartChars add: (Character value: each)].
	16rD8 to: 16rF6 do: [:each | NameStartChars add: (Character value: each)].
	16rF8 to: 16r2FF do: [:each | NameStartChars add: (Character value: each)].
	16r370 to: 16r37D do: [:each | NameStartChars add: (Character value: each)].
	16r37F to: 16r1FFF do: [:each | NameStartChars add: (Character value: each)].
	16r200C to: 16r200D do: [:each | NameStartChars add: (Character value: each)].
	16r2070 to: 16r218F do: [:each | NameStartChars add: (Character value: each)].
	16r2C00 to: 16r2FEF do: [:each | NameStartChars add: (Character value: each)].
	16r3001 to: 16rD7FF do: [:each | NameStartChars add: (Character value: each)].
	16rF900 to: 16rFDCF do: [:each | NameStartChars add: (Character value: each)].
	16rFDF0 to: 16rFFFD do: [:each | NameStartChars add: (Character value: each)].
	16r10000 to: 16rEFFFF do: [:each | NameStartChars add: (Character value: each)].! !

!XMLTokenizer class methodsFor: 'accessing' stamp: 'JAAyer 3/11/2010 07:53'!
isCharEscape: entityValue
	^entityValue size = 1
		and: [CharEscapes includes: entityValue first]! !

!XMLTokenizer methodsFor: 'testing' stamp: 'JAAyer 11/27/2010 07:38'!
atEnd
	state == #atEnd
		ifTrue: [^ true].
	(state ~~ #atStart and: [streamReader atEnd])
		ifTrue: [
			state := #atEnd.
			driver handleEndDocument.
			^ true].
	^ false.! !

!XMLTokenizer methodsFor: 'tokenizing' stamp: 'JAAyer 11/28/2010 21:08'!
checkAndExpandReference: parsingContext
	| referenceString nextChar |
	nextChar := streamReader peek.
	driver isValidating
		ifFalse: [^ nil].

	nextChar == $&
		ifTrue: [
			streamReader next.
			streamReader peek == $#
				ifTrue: [^ streamReader pushStream: (ReadStream on: self nextCharReference asString)].
			referenceString := self nextLiteral.
			self expectNext: $;.
			self replaceEntity: referenceString in: parsingContext ]
		ifFalse: [
			((nextChar == $%
				and: [self isParsingMarkup])
				and: [parsingContext == #entityValue])
				ifTrue: [
					streamReader skipSeparators.
					referenceString := self nextLiteral.
					self replaceEntity: referenceString in: parsingContext]].

	self expectMore.
	^ nextChar.! !

!XMLTokenizer methodsFor: 'tokenizing' stamp: 'JAAyer 3/11/2010 09:46'!
conditionalInclude: aKeyword
	aKeyword = 'INCLUDE'
		ifTrue: [^ true].
	aKeyword = 'IGNORE'
		ifTrue: [^ false].
	^ self conditionalInclude: (self parameterEntity: aKeyword) value! !

!XMLTokenizer methodsFor: 'accessing' stamp: 'JAAyer 11/27/2010 07:27'!
driver
	^ driver! !

!XMLTokenizer methodsFor: 'private' stamp: 'JAAyer 11/27/2010 07:26'!
endParsingMarkup
	state := #parsing.! !

!XMLTokenizer methodsFor: 'entities' stamp: 'JAAyer 3/16/2010 16:39'!
entities
	^ entities ifNil: [entities := self initEntities]! !

!XMLTokenizer methodsFor: 'entities' stamp: 'JAAyer 11/28/2010 11:49'!
entity: aReference
	driver isValidating
		ifFalse: [^ DTDEntityDeclaration name: aReference value: ''].

	^ self entities
		at: aReference
		ifAbsentPut: [self parseError: 'Undefined entity ', aReference printString]! !

!XMLTokenizer methodsFor: 'entities' stamp: 'JAAyer 3/11/2010 09:47'!
entity: aReference put: aValue
	"Only the first declaration of an entity is valid so if there is already
	one don't register the new value."
	self entities at: aReference ifAbsentPut: [aValue]! !

!XMLTokenizer methodsFor: 'errors' stamp: 'JAAyer 3/14/2010 12:02'!
errorExpected: expectedString
	self parseError: 'Expected ', expectedString! !

!XMLTokenizer methodsFor: 'errors' stamp: 'JAAyer 3/16/2010 12:49'!
errorExpected: anExpectedCharacterOrString butGot: aReceivedCharacterOrString
	| expectedString receivedString |

	expectedString := anExpectedCharacterOrString asString.	
	(receivedString := (aReceivedCharacterOrString ifNil: ['']) asString)
			ifEmpty: [receivedString := 'nothing'].

	self errorExpected: expectedString, ' but got ', receivedString.! !

!XMLTokenizer methodsFor: 'tokenizing' stamp: 'JAAyer 3/16/2010 12:49'!
expectLiteral: anExpectedLiteral
	| nextLiteral |

	((nextLiteral := self nextLiteral) = anExpectedLiteral)
		ifFalse: [self errorExpected: anExpectedLiteral butGot: nextLiteral].
	^ nextLiteral.! !

!XMLTokenizer methodsFor: 'tokenizing' stamp: 'JAAyer 3/16/2010 16:40'!
expectMore
	streamReader peek ifNil: [self errorExpected: 'more characters']! !

!XMLTokenizer methodsFor: 'tokenizing' stamp: 'JAAyer 3/16/2010 12:49'!
expectNext: aCharacter
	| nextChar |

	(nextChar := streamReader next) == aCharacter
		ifFalse: [
			self
				errorExpected: aCharacter
				butGot: nextChar].
	^ nextChar.! !

!XMLTokenizer methodsFor: 'tokenizing' stamp: 'JAAyer 11/27/2010 07:40'!
expectQuote
	| nextChar |

	(((nextChar := streamReader next) == $")
		or: [nextChar == $'])
			ifFalse: [
				self
					errorExpected: 'quote character delimiter'
					butGot: nextChar].
	^ nextChar.! !

!XMLTokenizer methodsFor: 'entities' stamp: 'JAAyer 3/11/2010 09:48'!
externalEntity: aReference
	^ self entities at: aReference ifAbsentPut: ['']! !

!XMLTokenizer methodsFor: 'entities' stamp: 'mir 11/23/2007 15:45'!
initEntities
	| ents |
	ents := Dictionary new.
	ents
		at: 'amp' put: (DTDEntityDeclaration name: 'amp' value: '&');
		at: 'quot' put: (DTDEntityDeclaration name: 'quot' value: '"');
		at: 'apos' put: (DTDEntityDeclaration name: 'apos' value: '''');
		at: 'gt' put: (DTDEntityDeclaration name: 'gt' value: '>');
		at: 'lt' put: (DTDEntityDeclaration name: 'lt' value: '<').
	^ents! !

!XMLTokenizer methodsFor: 'initialization' stamp: 'JAAyer 11/27/2010 07:26'!
initialize
	streamReader := XMLNestedStreamReader new.
	streamWriter := XMLNestedStreamWriter new.
	state := #atStart.! !

!XMLTokenizer methodsFor: 'private' stamp: 'JAAyer 11/27/2010 07:28'!
isParsingMarkup
	^ state == #isParsingMarkup! !

!XMLTokenizer methodsFor: 'tokenizing' stamp: 'JAAyer 11/28/2010 11:50'!
nextAttributeInto: attributes namespaces: namespaces

	| attrName attrValue |
	attrName := self nextName.
	streamReader skipSeparators.
	self expectNext: $=.
	streamReader skipSeparators.
	attrValue := self nextAttributeValue.

	(driver usesNamespaces
		and: [attrName beginsWith: 'xmlns'])
		ifTrue: [attrName size > 6
			ifTrue: [namespaces at: (attrName copyFrom: 7 to: attrName size) put: attrValue]
			ifFalse: [namespaces at: '' put: attrValue]]
		ifFalse: [attributes at: attrName put: attrValue].! !

!XMLTokenizer methodsFor: 'tokenizing' stamp: 'JAA 3/22/2011 11:48'!
nextAttributeValue
	| delimiterChar |

	delimiterChar := self expectQuote.
	^ streamWriter writeWith: [:writeStream |
		self nextPCDataDelimitedBy: delimiterChar putOn: writeStream.
		self expectNext: delimiterChar.
		writeStream contents]! !

!XMLTokenizer methodsFor: 'tokenizing' stamp: 'JAAyer 11/27/2010 07:31'!
nextCDataContent
	| cdata |

	cdata := streamReader upToAll: ']]>'.
	driver handleCData: cdata
! !

!XMLTokenizer methodsFor: 'tokenizing' stamp: 'JAAyer 11/27/2010 07:28'!
nextCDataOrConditional

	| nextChar conditionalKeyword |
	"Skip ["
	streamReader next.
	streamReader skipSeparators.
	nextChar := streamReader peek.
	nextChar == $%
		ifTrue: [
			self checkAndExpandReference: (self isParsingMarkup ifTrue: [#dtd] ifFalse: [#content]).
			conditionalKeyword := self nextLiteral.
			streamReader skipSeparators.
			self expectNext: $[.
			streamReader skipSeparators.
			^ self nextIncludeSection: (self conditionalInclude: conditionalKeyword)].

	self
		expectLiteral: 'CDATA';
		expectNext: $[;
		nextCDataContent.! !

!XMLTokenizer methodsFor: 'tokenizing' stamp: 'JAA 3/23/2011 10:54'!
nextCharReference
	| base charValue |

	streamReader next == $#
		ifFalse: [self errorExpected: 'character reference'].
	(charValue := streamReader nextInteger) > 0
		ifFalse: [self errorExpected: 'integral character value'].
	self expectNext: $;.
	^ Unicode value: charValue.! !

!XMLTokenizer methodsFor: 'tokenizing' stamp: 'JAAyer 11/27/2010 07:31'!
nextComment
	"Skip first -"
	streamReader next.
	self expectNext: $-.
	driver handleComment: (streamReader upToAll: '-->')! !

!XMLTokenizer methodsFor: 'tokenizing dtd' stamp: 'JAAyer 3/15/2010 04:32'!
nextDocType
	self expectLiteral: 'DOCTYPE'.
	self startParsingMarkup.
	^ self nextDocTypeDecl.! !

!XMLTokenizer methodsFor: 'tokenizing dtd' stamp: 'JAAyer 3/19/2010 18:33'!
nextDocTypeDecl
	| nextChar |
	streamReader skipSeparators.
	self nextLiteral.
	streamReader skipSeparators.
	streamReader peek == $[
		ifFalse: [[nextChar := streamReader peek.
				nextChar == $> or: [nextChar == $[ ]] whileFalse: [streamReader next]].
	streamReader peek == $[
		ifTrue: [
			streamReader next.
			[streamReader skipSeparators.
			streamReader peek == $]] whileFalse: [
				self checkAndExpandReference: #dtd.
				self nextMarkupToken].
			self expectNext: $]].
	streamReader skipSeparators.
	self expectNext: $>.

	self endParsingMarkup! !

!XMLTokenizer methodsFor: 'tokenizing' stamp: 'JAAyer 11/27/2010 07:33'!
nextEndTag
	| tagName |
	"Skip /"
	streamReader next.
	tagName := self nextName.
	streamReader skipSeparators.
	self expectNext: $>.
	driver handleEndTag: tagName! !

!XMLTokenizer methodsFor: 'tokenizing dtd' stamp: 'JAAyer 3/11/2010 08:25'!
nextEntityDeclaration
	| entityName entityDef referenceClass reference |
	streamReader skipSeparators.
	referenceClass := streamReader peek == $%
		ifTrue: [
			streamReader next.
			streamReader skipSeparators.
			DTDParameterEntityDeclaration]
		ifFalse: [DTDEntityDeclaration].
	entityName := self nextLiteral.
	streamReader skipSeparators.
	entityDef := (streamReader peek == $" or: [streamReader peek == $'])
		ifTrue: [self nextEntityValue]
		ifFalse: [self nextExternalId].
	streamReader skipUpTo: $>.
	reference := referenceClass name: entityName value: entityDef.
	reference registerIn: self.
	^reference! !

!XMLTokenizer methodsFor: 'tokenizing' stamp: 'JAA 3/22/2011 11:48'!
nextEntityValue
	| delimiterChar entityValueStream nextChar nextPeek referenceString entity entityValue |
	delimiterChar := self expectQuote.
	entityValueStream := WriteStream on: (String new: 32).
	[
	self expectMore.
	nextPeek := nextChar := streamReader peek.
	nextChar == $&
		ifTrue: [
			streamReader next.
			streamReader peek == $#
				ifTrue: [
					nextPeek := nil.
					nextChar := self nextCharReference]
				ifFalse: [
					referenceString := self nextLiteral.
					self expectNext: $;.
					entity := self entity: referenceString.
					entityValue := entity valueForContext: #entityValue.
					streamReader pushStream: (ReadStream on: entityValue asString).
					nextPeek := nextChar := streamReader next]]
		ifFalse: [
			nextChar == $%
				ifTrue: [
					streamReader skipSeparators.
					referenceString := self nextLiteral.
					nextChar := self replaceEntity: referenceString in: #entityValue.
					nextPeek := nextChar := streamReader next]
				ifFalse: [streamReader next]].
	nextPeek == delimiterChar]
		whileFalse: [
			nextChar ifNotNil: [entityValueStream nextPut: nextChar]].
	^ entityValueStream contents.! !

!XMLTokenizer methodsFor: 'tokenizing dtd' stamp: 'JAA 3/22/2011 11:48'!
nextExternalId
	| extDefType systemId dir |
	extDefType := self nextLiteral.
	extDefType = 'PUBLIC'
		ifTrue: [
			streamReader skipSeparators.
			self nextPubidLiteral.
			streamReader skipSeparators.
			streamReader peek == $>
				ifFalse: [
					systemId := self nextSystemLiteral]].

	extDefType = 'SYSTEM'
		ifTrue: [
			streamReader skipSeparators.
			systemId := self nextSystemLiteral].

	systemId
		ifNil: [^nil].

	"The rest of this method only applies if we're reading aFileStream"
	(streamReader stream isKindOf: FileStream)
		ifFalse: [^''].
	dir := streamReader stream directory.
	^(dir fileExists: systemId)
		ifTrue: [(dir readOnlyFileNamed: systemId) contents]
		ifFalse: ['']! !

!XMLTokenizer methodsFor: 'tokenizing' stamp: 'JAAyer 3/16/2010 09:11'!
nextIncludeSection: parseSection
	| section |
	"Read the file up to the next include section delimiter and parse it if parseSection is true"

	section := streamReader upToAll: ']]>'.
	parseSection
		ifTrue: [streamReader pushStream: (ReadStream on: section)]! !

!XMLTokenizer methodsFor: 'tokenizing' stamp: 'JAA 3/22/2011 11:48'!
nextLiteral
	| nextChar |

	((nextChar := streamReader peek) isLetter
		or: [nextChar == $_])
		ifFalse: [self errorExpected: 'name literal.'].

	^ streamWriter writeWith: [:writeStream |
		[LiteralChars includes: nextChar]
			whileTrue: [
				nextChar == $&
					ifTrue: [
						nextChar := streamReader next.
						writeStream nextPut: (streamReader peek == $#
							ifTrue: [self nextCharReference]
							ifFalse: [^ writeStream contents])]
					ifFalse: [writeStream nextPut: streamReader next].
				nextChar := streamReader peek].
		writeStream position > 0
			ifFalse: [self errorExpected: 'name literal'].
		writeStream contents]! !

!XMLTokenizer methodsFor: 'tokenizing dtd' stamp: 'JAAyer 11/28/2010 11:48'!
nextMarkupDeclaration
	| declType |
	declType := self nextLiteral.
	driver isValidating
		ifFalse: [^ self skipMarkupDeclaration].
	declType = 'ENTITY'
		ifTrue: [self nextEntityDeclaration]
		ifFalse: [self skipMarkupDeclaration]! !

!XMLTokenizer methodsFor: 'tokenizing' stamp: 'JAAyer 11/27/2010 07:28'!
nextMarkupToken
	| nextChar |
	"Skip < "
	streamReader next.
	nextChar := streamReader peek.
	nextChar == $!! ifTrue: [
		"Skip !!"
		streamReader next.
		nextChar := streamReader peek.
		nextChar == $- ifTrue: [^ self nextComment].
		nextChar == $[ ifTrue: [^ self nextCDataOrConditional].
		^ self isParsingMarkup
			ifTrue: [self nextMarkupDeclaration]
			ifFalse: [self nextDocType]].
	nextChar == $? ifTrue: [^ self nextPI].
	self nextTag.! !

!XMLTokenizer methodsFor: 'tokenizing' stamp: 'JAA 3/22/2011 11:48'!
nextName
	| nextChar |

	^ streamWriter writeWith: [:writeStream |
		(NameStartChars includes: (nextChar := streamReader next))
			ifFalse: [self errorExpected: 'name'].
		writeStream nextPut: nextChar.

		[streamReader atEnd not
			and: [(NameStartChars includes: (nextChar := streamReader peek))
				or: [AdditionalNameChars includes: nextChar]]]
			whileTrue: [writeStream nextPut: streamReader next].
		writeStream contents]! !

!XMLTokenizer methodsFor: 'tokenizing' stamp: 'JAA 3/22/2011 11:48'!
nextPCData

	streamWriter writeWith: [:writeStream |
		driver isValidating
			ifTrue: [self nextPCDataDelimitedBy: $< putOn: writeStream]
			ifFalse: [
				[streamReader peek == $<]
					whileFalse: [writeStream nextPut: streamReader next]].

		driver handlePCData: writeStream contents]! !

!XMLTokenizer methodsFor: 'tokenizing' stamp: 'JAAyer 3/16/2010 16:45'!
nextPCDataDelimitedBy: aDelimiter putOn: aStream
	| nextChar referenceString entity entityValue |

	[(nextChar := streamReader peek) isNil or: [nextChar == aDelimiter]]
		whileFalse: [
			nextChar == $&
				ifTrue: [
					streamReader next.
					streamReader peek == $#
						ifTrue: [aStream nextPut: self nextCharReference]
						ifFalse: [
							referenceString := self nextLiteral.
							self expectNext: $;.
							entity := self entity: referenceString.
							entityValue := entity valueForContext: #content.
							(self class isCharEscape: entityValue)
								ifTrue: [aStream nextPut: entityValue first]
								ifFalse: [
									entityValue := entityValue asString.
									entityValue isEmpty
										ifFalse: [
											streamReader pushStream:
												(ReadStream on: entityValue asString)]]]]
				ifFalse: [aStream nextPut: streamReader next]].! !

!XMLTokenizer methodsFor: 'tokenizing' stamp: 'JAAyer 11/27/2010 07:33'!
nextPI
	| piTarget piData |
	"Skip ?"
	streamReader next.
	piTarget := self nextLiteral.
	piTarget asUppercase = 'XML'
		ifTrue: [^ self nextXMLDecl].
	streamReader skipSeparators.
	piData := streamReader upToAll: '?>'.
	driver handlePI: piTarget data: piData.! !

!XMLTokenizer methodsFor: 'tokenizing' stamp: 'JAAyer 3/11/2010 09:56'!
nextPubidLiteral
	^ self nextAttributeValue! !

!XMLTokenizer methodsFor: 'tokenizing' stamp: 'JAAyer 3/11/2010 09:56'!
nextSystemLiteral
	^ self nextAttributeValue! !

!XMLTokenizer methodsFor: 'tokenizing' stamp: 'JAAyer 12/6/2010 22:54'!
nextTag
	| tagName attributes nextChar namespaces |
	(streamReader peek = $/)
		ifTrue: [^ self nextEndTag].
	tagName := self nextName.
	streamReader skipSeparators.
	attributes := OrderPreservingDictionary new: 10 withDefaultValue: ''.
	driver usesNamespaces
		ifTrue: [namespaces := OrderPreservingDictionary defaultValue: ''].
	[(nextChar := streamReader peek) == $> or: [nextChar == $/]] whileFalse: [
		self checkAndExpandReference: #content.
		self nextAttributeInto: attributes namespaces: namespaces.
		streamReader skipSeparators.].
	driver handleStartTag: tagName attributes: attributes namespaces: namespaces.
	streamReader next == $/
		ifTrue: [
			driver handleEndTag: tagName.
			self expectNext: $>].! !

!XMLTokenizer methodsFor: 'tokenizing' stamp: 'JAAyer 11/27/2010 07:33'!
nextToken
	| whitespace |

	state == #atStart
		ifTrue: [
			driver handleStartDocument.
			state := #started].

	whitespace := self nextWhitespace.
	self atEnd
		ifTrue: [^ self].

	self checkAndExpandReference: (self isParsingMarkup ifTrue: [#dtd] ifFalse: [#content]).
	^ (streamReader peek = $<)
		ifTrue: [self nextMarkupToken]
		ifFalse: [
			whitespace ifNotEmpty: [streamReader pushBack: whitespace].
			self nextPCData].! !

!XMLTokenizer methodsFor: 'tokenizing' stamp: 'JAAyer 11/27/2010 07:33'!
nextWhitespace
	| whitespace |

	(whitespace := streamReader nextWhitespace)
		ifNotEmpty: [driver handleWhitespace: whitespace].
	^ whitespace.! !

!XMLTokenizer methodsFor: 'tokenizing' stamp: 'JAAyer 12/14/2010 02:39'!
nextXMLDecl
	| attributes namespaces |
	streamReader skipSeparators.
	attributes := Dictionary new.
	namespaces := Dictionary new.
	[streamReader peek == $?]
		whileFalse: [
			self nextAttributeInto: attributes namespaces: namespaces.
			streamReader skipSeparators].
	streamReader next.
	self expectNext: $>.
	(attributes includesKey: 'encoding')
		ifTrue: [streamReader useConverterForEncoding: (attributes at: 'encoding')].
	driver
		handleXMLVersion: (attributes at: 'version' ifAbsent: [''])
		encoding: (attributes at: 'encoding' ifAbsent: [''])
		standalone: (attributes at: 'standalone' ifAbsent: ['']).! !

!XMLTokenizer methodsFor: 'entities' stamp: 'JAAyer 3/14/2010 12:36'!
parameterEntities
	^ parameterEntities ifNil: [parameterEntities := Dictionary new]! !

!XMLTokenizer methodsFor: 'entities' stamp: 'JAAyer 3/19/2010 14:17'!
parameterEntity: refName
	^ self parameterEntities
		at: refName
		ifAbsent: [self parseError: 'undefined parameter entity ', refName printString]! !

!XMLTokenizer methodsFor: 'entities' stamp: 'JAAyer 3/11/2010 09:55'!
parameterEntity: refName put: aReference
	"Only the first declaration of an entity is valid so if there is already
	one don't register the new value."
	self parameterEntities at: refName ifAbsentPut: [aReference]! !

!XMLTokenizer methodsFor: 'errors' stamp: 'JAAyer 11/28/2010 13:22'!
parseError: errorString
	XMLParsingException signal: errorString! !

!XMLTokenizer methodsFor: 'entities' stamp: 'JAAyer 11/28/2010 21:08'!
replaceEntity: referenceString in: parsingContext 

	| entity entityValue |
	entity := self entity: referenceString.
	entityValue := entity valueForContext: parsingContext.
	(self class isCharEscape: entityValue)
		ifTrue: [entityValue := entity reference].
	streamReader pushStream: (ReadStream on: entityValue asString)! !

!XMLTokenizer methodsFor: 'private' stamp: 'JAAyer 11/28/2010 11:46'!
setDriver: aDriver stream: aStream
	driver := aDriver.
	streamReader stream: aStream.! !

!XMLTokenizer methodsFor: 'tokenizing dtd' stamp: 'JAAyer 3/11/2010 08:30'!
skipMarkupDeclaration
	streamReader skipUpTo: $>! !

!XMLTokenizer methodsFor: 'private' stamp: 'JAAyer 11/27/2010 07:28'!
startParsingMarkup
	state := #isParsingMarkup! !

!XMLTokenizer methodsFor: 'private' stamp: 'JAAyer 3/11/2010 08:04'!
stream
	^ streamReader stream! !

!XMLTokenizer methodsFor: 'accessing' stamp: 'JAAyer 3/11/2010 09:03'!
streamReader
	^ streamReader! !
XMLNestedStreamReader initialize!
XMLTokenizer initialize!
DTDEntityDeclaration initialize!
DTDParameterEntityDeclaration initialize!
